package clarion.acs.gs;
import clarion.acs.net.*;
import clarion.system.*;
import clarion.nacs.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the goal action-decision network. It extends the
 * generic class <em>AcsNet</em>.
 *
 * GS action is for internal use, that is, for CLARION system use.
    the format is the following:
    - dimension 0 :
        value 0 : do-nothing action
        value 1 : push action
        value 2 : pop action
    - dimension 1 : ( goal dimension )
        value i : goal i is currently active
        the length of the dimension is the # of goals user defines.
    - other dimensions
        these dimensions are for the goal parameters along with the currently active goal.
        Since the goal parameters are task-specific, so, these dimensions should be defined by
        user. They are maybe empty sometimes.
 *
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public void reasoning(): the process of reasoning for determining an goal
 * action to perform given current input.
 * <li> public void perform(): the process of performing the goal action.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class implements the goal action-decision network which is one of three
 * basic action-decision networks.
 * <li> Usage of this class. <br>
 * It is instantiated inside an <em>ACS</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class GSNet extends AcsNet {

  /** goal structure option: constant or variable. */
  protected int gsOption;
  /** the goal structure used. */
  protected GoalStructure gs;

  /** constructor.
   *  @param nIdx net index.
   *  @param g the global settings. */
  public GSNet( int nIdx, Global g )
  {
     super( nIdx, g );
     gsOption = global.getGsOption();
     if( gsOption == global.GS_LIST )
         gs = new GoalList( global );
     else
         gs = new GoalStack( global );
  }

  /** The process of reasoning for determining an goal action to perform given
      current input. */
  public void reasoning()
  {
     // get active net comps.
     if( global.getGoalDecisionMode() == Global.ACTION_BY_TL ){
         for( int i = 0; i < COMP_NUM; i++ ){
              if( netCompFlags[ i ] ){
                  netComps[ i ].setStates( state, exState, gsState, wmState );
                  netComps[ i ].reasoning();
              }
         }
         // select which part: BL, RER, IRL or FR to decide action.
         chosenComp = Global.FR;
         // decide an action.
         decideAction( chosenComp );
         // set the chosen action.
         deliverAction();
     }
     else
         super.reasoning();
  }

 /*
   the GS action format is the following:
    - dimension 0 :
         value 0 : DO_NOTHING
         value 1 : SET
         value 2 : RESET
    - dimension 1 :
         value 0 : goal 0 is currently active.
         value 1 : goal 1 is currently active.
           .    .    .    .
           .    .    .    .
         value i : goal i is currently active.
    - dimension i : other paramter dimensions.

  */
  public void perform()
  {
     System.out.println( "GSNet.java, performed action = " + chosenFullAction[0] );

     boolean updateFlag = false;
     if( chosenFullAction[Global.ACTION_NORMAL][0][0] == 0 )   // do-nothing action.
         updateFlag = gs.actionNoGoal();
     if( chosenFullAction[Global.ACTION_NORMAL][0][0] == 1 )    // a set action.
         updateFlag = gs.actionSet( chosenFullAction[Global.ACTION_NORMAL][1][0] );
     if( chosenFullAction[Global.ACTION_NORMAL][0][0] == 2 )   // a reset action.
         updateFlag = gs.actionReset( chosenFullAction[Global.ACTION_NORMAL][1][0] );
     // update current state.
     if( updateFlag ){
         if( gsOption == global.GS_STACK || ((GoalList)gs).goalCompete() )
             gs.updateState( gsState );
     }
  }

}