package clarion.acs.gs;
import java.util.Vector;
import clarion.system.*;
import clarion.common.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the concept of goal item in goal structure. It
 * extends the generic chunk class <em>Chunk</em>.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public boolean isActivated(): returns if this item is activated or not.
 * <li> public boolean isRemoved(): returns if this item is removed or not.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class implements the goal items those compose the goal structure.
 * <li> Usage of this class. <br>
 * It is instantiated inside <em>GoalStack</em> or <em>GoalList</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class GoalChunk extends Chunk{

  /** indicate activated or not. */
  protected boolean activated;
  /** indicates removed or not. */
  protected boolean removed;

  /** constructor.
   *  @param type chunk type.
   *  @param label the item label.
   *  @param list the feature list of the item.
   *  @param g the associated global settings. */
  public GoalChunk( int type, String label, Feature[] list, Global g )
  {
     super( type, label, list, g );
  }

  /** Returns if this item is activated or not. */
  public boolean isActivated()
  {
     return activated;
  }

  /** Returns if this item is removed or not. */
  public boolean isRemoved()
  {
     return removed;
  }

  /** Sets flag of activation.
   *  @param flag the flag indicates this item is activated or not. */
  public void setActivated( boolean flag )
  {
     activated = flag;
  }
  /** Sets flag of removal.
   *  @param flag the flag indicates this item is removed or not. */
  public void setRemoved( boolean flag )
  {
     removed = flag;
  }
}