package clarion.acs.net;
import java.util.TreeMap;
import clarion.acs.net.auxiliary.*;
import clarion.system.*;
import clarion.tools.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement Fixed Rule set. It extends the more generic rule
 * set class <em>TLRuleSet</em>.
 * <li> summary of the important methods. <br>
 * <ol>
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class implements the Fixed Rules which is one of the three rule knowledge
 * types in the top level of an ACS action-decision network.
 * <li> Usage of this class. <br>
 * It is instantiated inside an <em>AcsANet</em> object.
 * @author Xi Zhang
 *
 */

public class FixRuleSet extends TLRuleSet {

   /** decide which FR subset is used. */
   protected int groupId;
   /** utility flags of the FR rules. */
   protected boolean UTL_FLAGs[];
   /** action BLA options of the FR rules. used for RT. */
   protected int actionBLAoptions[];

   /** constructor.
    *  @param netIdx the network index.
    *  @param global the associated global settings. */
   public FixRuleSet( int subsysIdx, int netIdx, Global global )
   {
       super( subsysIdx, netIdx, Global.FR, global );
       groupId = global.getAgentGroupId();
       RULE_NUM = global.getFixRuleNum( netIdx );
       ruleList = new FixRule[ RULE_NUM ];
       for( int i = 0; i < RULE_NUM; i++ )
            ruleList[ i ] = new FixRule( subsysIdx, groupId, netIdx, i, global );
       ruleUtls = new double[ RULE_NUM ];
       recommendRules = new short[ RULE_NUM ];
       actionBLAoptions = new int[ RULE_NUM ];
       for( int i = 0; i < RULE_NUM; i++ ){
            if( !global.isRuleBlaOn( netIdx, global.FR, i ) ){
                actionBLAoptions[ i ] = global.EMPTY;
                continue;
            }
            actionBLAoptions[ i ] = global.getAcsChunkBlaOption( netIdx, global.FR, i );
       }
   }
}