package clarion.common;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to impliment a sortable element object.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public double getKeyValue(): returns key value.
 * <li> public int compareTo(Object obj): compare two <em>SortableElement</em> object.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is a helper class used for managing the chunks.
 * <li> Usage of this class. <br>
 * It is instantiated inside a concrete rule object.
 * </ul>
 * @author Xi Zhang
 *
 */


public class SortableElement implements Comparable{
    protected double activation;
    protected int chunkIdx;

    /** constructor.
     *  @param activation the chunk activation.
     *  @chunkIdx the chunk index.
     */
    public SortableElement( double activation, int chunkIdx )
    {
       this.activation = activation;
       this.chunkIdx = chunkIdx;
    }

    /**
     * returns key value.
     * @return the activation. */
    public double getKeyValue()
    {
       return activation;
    }

    /**
     * returns chunk index.
     * @return chunk index. */
    public int getChunkIdx()
    {
        return chunkIdx;
    }

    /** compare two <em>SortableElement</em> objects.
     *  @return compValue > 0: this element precedences the other element.
     *                    = 0: this element is equal to the other element..
     *                    < 0: this element is behind the other element.
     */
    public int compareTo(Object obj) throws ClassCastException
    {
        SortableElement otherElement = null;
        try{
           otherElement = (SortableElement)obj;
        }
        catch( ClassCastException e ){
               System.out.println( e );
        }
        int compValue = 0;
        if( this.getKeyValue() > otherElement.getKeyValue() )
            compValue = 1;
        if( this.getKeyValue() < otherElement.getKeyValue() )
            compValue = -1;
        return compValue;
    }
}