package clarion.nacs;
import java.util.Vector;
import clarion.system.*;
import clarion.common.*;
import clarion.tools.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the items in EM in GKS.
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class implements the items in Episodic Memory which are used to help learning
 * in AMN, IDN or AEM.
 * <li> Usage of this class. <br>
 * It is instantiated either inside a <em>EM</em> object or wherever a GKS chunk
 * is needed.
 * </ul>
 * @author Xi Zhang
 */

public class EMChunk extends Chunk{

   /** time stamp. */
   long timeStamp;

   /** constructor for simple item.
    *  @param chunk the chunk to compose this item.
    *  @param time time stamp.
    *  @param g the global settings. */
   public EMChunk( Chunk chunk, long time, Global g )
   {
      super( chunk.getChunkType(), chunk.getLabel(), chunk.getFeatures(), g );
      timeStamp = time;
   }

   /** constructor for combined item.
    *  @param type chunk type.
    *  @param label chunk label.
    *  @param chunks chunks to compose this item.
    *  @param time time stamp.
    *  @param g the global settings. */
   public EMChunk( int type, String label, Chunk[] chunks, long time, Global g )
   {
      super( type, label, chunks, g );
      timeStamp = time;
   }

   /** Returns the time stamp.
    *  @return the time stamp. */
   public long getTimeStamp()
   {
      return timeStamp;
   }

   /** Sets the time stamp.
    *  @param time the time stamp to set. */
   public void setTimeStamp( long time )
   {
      timeStamp = time;
   }
}