package clarion.options;
import java.awt.event.*;
import javax.swing.*;
import clarion.system.*;
import clarion.options.inter.*;
import clarion.options.taskspec.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to show the initial screen of configuring task specific options
 * by GUI.
 * It extends the base class <em>GuiTemplate</em>.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public void loadSubSys(): Loads and initializes the CLARION subsystems.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to control the process of configuring the Task Specific options
*  by GUI (on-line configuration).
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>NewSimulator</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class GuiTask extends GuiTemplate{

  final int BTN_INTER = 0, BTN_TS = 1;
  String buttonTexts[] = {
     "Interface Options",
     "Task specific Options"
  };
  /** configurations GUI. */
  SelectionTemplate[] selOptions;

  /** constructor.
   *  @param num the number of components needing configuration.
   *  @param globals the global settings with the running agents. */
  public GuiTask( int num, Global[] globals )
  {
     super( num, "TASK-SPECIFIC PARAMETER SETTINGS", globals );
     setButtonTexts( buttonTexts );
  }

  /** Loads and initializes the CLARION subsystems. */
  public void loadSubSys()
  {
     manager.load( Global.TASK_OPTIONS );
     selOptions = new SelectionTemplate[ 2 ];
     selOptions[ 0 ] = new SelectionInter( this, 440, 320, manager );
     selOptions[ 1 ] = new SelectionTaskSpec( this, 330, 300, manager );
  }

  /** the routine of processing the changes when returning from other GUIs. */
  public void returnFrom()
  {
  }

  /** Adds action listeners to the option buttons. */
  public void addActionListeners( final JFrame mainFrame )
  {
     for( int i = 0; i < OPT_NUM; i++ ){
          final int focus = i;
          final int idx = i;
          optButtons[i].addActionListener( new  ActionListener() {
                     public void actionPerformed( ActionEvent e ){
                        activeButtonIdx = focus;
                        selOptions[ idx ].setEnabled( true );
                        selOptions[ idx ].setVisible( true );
                        mainFrame.setEnabled( false );
                        mainFrame.setVisible( false );
                     }
                 } );
     }
  }
}
