package clarion.options;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import clarion.*;
import clarion.system.*;
import clarion.tools.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is the base class for configuration of the system by GUI.
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void init( JFrame frame ): the routine of initializing this
 * configuration GUI.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * The concrete class for implementing the configuration of a CLARION subsystem
 * by GUI will extends this class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class GuiTemplate extends JFrame {

  /** text indicates main interface of CLARION configuration. */
  protected final String majorEntryText = "CLARION PARAMETER SETTINGS";
  /** text indicates main interface of Task-Specific configuration. */
  protected final String taskSpecEntryText = "TASK-SPECIFIC PARAMETER SETTINGS";
  protected final String note =
            "   ( type ENTER key after typing in any information, applied to all GUI screens )";
  /** the names of all currently simulated task. */
  protected String newTaskNames[];
  /** the names of all setting groups for a loaded task. */
  protected String newSettingNames[];
  /** current number of tasks simulated so far. */
  protected static int currentTaskNum;
  /** current number of configurations for a particular task. */
  protected static int currentSettingNum;
  /** the name of the loaded task. */
  protected static String loadTaskName;
  /** the number of agents running simultaneously. */
  protected static int runningAgentNum;
  /** inidicate whether an agent has been already configured by the options:
   * CLARION and Task Specific. */
  protected static boolean[][] configFlags = new boolean[2][Global.MAX_RUNNING_AGENT_NUM];
  /** the associated global settings of each running agent. */
  protected static Global[] globals;
  /** flag for confirmation of the configuration. */
  protected boolean confirmFlag;
  /** current index of agent requiring configuration. */
  protected int curAgentIdx;
  /** the name of the loaded task configuration. */
  protected String loadSettingName;
  /** number of components in the subsystem needing configuration. */
  protected int OPT_NUM;
  /** GUI components in the interface of configuration. */
  protected String capLabelText, optionButtonTexts[];
  protected JLabel capLabel;
  protected JFrame mainFrame, thisFrame;
  protected JPanel optionPanel, operatePanel;
  //protected JButton loadTaskButton, loadSettingButton, saveButton;
  protected JButton loadSettingButton, saveButton;
  protected static JButton confirmButton;
  protected JButton nextButton, returnButton, optButtons[];
  // protected JTextField loadTaskField, loadSettingField, saveField;
  protected JTextField loadSettingField, saveField;
  protected JPopupMenu loadTaskMenu, loadSettingMenu;
  protected JMenuItem loadTaskItem[], loadSettingItem[];
  protected int activeButtonIdx;
  /** the variable in charge of laoding and saving configuration. */
  protected static TransferManager manager;

  /** constructor.
   *  @param optNum number of components in the subsystem needing configuration.
   *  @param labelText the caption label. */
  public GuiTemplate( int optNum, String labelText )
  {
     OPT_NUM = optNum;
     capLabelText = labelText;
     thisFrame = this;
     optButtons = new JButton[ OPT_NUM ];
     for( int i = 0; i < OPT_NUM; i++ )
         optButtons[ i ] = new JButton();
  }

  /** constructor.
   *  @param optNum number of components in the subsystem needing configuration.
   *  @param labelText the caption label.
   *  @param globals the assoiciated global settings with the running agents. */
  public GuiTemplate( int optNum, String labelText, Global[] globals )
  {
     OPT_NUM = optNum;
     capLabelText = labelText;
     this.globals = globals;
     thisFrame = this;
     optButtons = new JButton[ OPT_NUM ];
     for( int i = 0; i < OPT_NUM; i++ )
         optButtons[ i ] = new JButton();
     //if( this instanceof GuiClarion || this instanceof GuiTask ){
         manager = new TransferManager( globals[0] );
         newTaskNames = manager.searchCurrentTasks();
         currentTaskNum = newTaskNames.length;
         newSettingNames = manager.searchCurrentSettings();
         currentSettingNum = newSettingNames.length;
         loadTaskName = Global.getTaskName();
         loadSettingName = "loaded setting name";
         for( int i = 0; i < OPT_NUM; i++ )
              optButtons[ i ].setEnabled( false );
     //}
  }

  /** Returns if current configuration is confirmed or not. */
  public void resetConfigFlags()
  {
     CommonMethods.resetArray( configFlags, false );
  }

  /** Returns if current configuration is confirmed or not. */
  public boolean isConfirmed( int configPart, int agentIdx )
  {
     return this.configFlags[ configPart ][ agentIdx ];
  }

  /** Returns this GUI frame. */
  public JFrame getFrame()
  {
     return this;
  }

  /** Returns caption. */
  public String getLabelText()
  {
     return capLabelText;
  }

  /** Returns the names of components in this subsystem. */
  public String[] getButtonTexts()
  {
     return optionButtonTexts;
  }

  /** Sets the index of current agent requiring configuration.
   *  @param agentIdx the index of agent to set. */
  public void setCurAgentIdx( int agentIdx )
  {
     curAgentIdx = agentIdx;
     manager.setGlobal( globals[curAgentIdx] );
  }

  /** Sets the index of current agent requiring configuration.
   *  @param num the number of agents to set. */
  public void setRunningAgentNum( int num )
  {
     this.runningAgentNum = num;
  }

  /** Sets the global settings associated with the running agents.
   *  @param globals the array of global settings to set. */
  public void setGlobals( Global[] globals )
  {
     this.globals = globals;
  }

  /** Sets the caption.
   *  @param str the caption to set. */
  public void setLabelText( String str )
  {
     capLabelText = str;
  }

  /** Sets the confirm status.
   *  @param changeFlag indicates if any option is change or not. */
  public void setConfirmStatus( boolean changeFlag )
  {
     if( changeFlag ){
         confirmButton.setEnabled( true );
         confirmFlag = false;
     }
  }

  /** Sets the names of the components in this subsystem.
   *  @param strings the names to set. */
  public void setButtonTexts( String[] strings )
  {
     optionButtonTexts = strings;
  }

  /** Sets the task name.
   *  @param name the task name to set. */
  public void setTaskName( String name )
  {
     loadTaskName = name;
  }

  public void setMainFrame( JFrame frame )
  {
     mainFrame = frame;
  }

  /** the routine of initializing this configuration GUI. */
  public void init( JFrame frame )
  {
    mainFrame = frame;
    if( this instanceof GuiClarion || this instanceof GuiTask ){
        if( this instanceof GuiClarion )
            setSize( 960, 400 );
        else
            setSize( 960, 200 );
    }
    else
        setSize( 640, 480 );
    if( this instanceof GuiClarion || this instanceof GuiTask )
        capLabel = new JLabel( capLabelText + note + "  Current agent = " + curAgentIdx );
    else
        capLabel = new JLabel( capLabelText );
    capLabel.setHorizontalAlignment( SwingConstants.CENTER );
    capLabel.setVerticalAlignment( SwingConstants.BOTTOM );
    getContentPane().add( capLabel, BorderLayout.NORTH );
    optionPanel = new JPanel();
    optionPanel.setLayout( new FlowLayout( FlowLayout.CENTER, 300, 15 ) );
    getContentPane().add( optionPanel, BorderLayout.CENTER );
    operatePanel = new JPanel();
    operatePanel.setLayout( new FlowLayout( FlowLayout.CENTER, 10, 15 ) );
    getContentPane().add( operatePanel, BorderLayout.SOUTH );
    for( int i = 0; i < OPT_NUM; i++ ){
         optButtons[ i ].setText( optionButtonTexts[ i ] );
         optionPanel.add( optButtons[ i ] );
    }

    // task information.
    if( this instanceof GuiClarion || this instanceof GuiTask ){
        // components for loading settings.
        loadSettingButton = new JButton ( "Setting Group" );
        loadSettingName = globals[curAgentIdx].getTaskSettingName() != null ?
                          globals[curAgentIdx].getTaskSettingName() : "input a setting name";
        loadSettingField =  new JTextField( loadSettingName, 9 );
        operatePanel.add( loadSettingButton );
        operatePanel.add( loadSettingField );

        // button for confirm user's selection and save to Global object.
        confirmButton = new JButton( "CONFIRM" );
        confirmButton.setEnabled( false );
        confirmButton.addActionListener(
               new  ActionListener() {
                    public void actionPerformed( ActionEvent e )
                    {
                       if( JOptionPane.showConfirmDialog( null, "Confirm your settings?",
                           "WARNING", JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION ){
                           confirmButton.setEnabled( false );
                           nextButton.setEnabled( curAgentIdx < runningAgentNum-1 );
                           returnButton.setEnabled( curAgentIdx == runningAgentNum-1 );
                           returnButton.setVisible( curAgentIdx == runningAgentNum-1 );
                           confirmFlag = true;
                           configFlags[majorEntryText.equals( capLabelText )?Global.CLARION_OPTIONS:Global.TASK_OPTIONS][curAgentIdx] = true;
                           loadSettingName = loadSettingField.getText().trim();
                           globals[curAgentIdx].setTaskSettingName( loadSettingName );
                           manager.setSaveCode( (majorEntryText.equals( capLabelText )) ? Global.CLARION_OPTIONS : Global.TASK_OPTIONS,
                                                Global.SAVE_TO_GLOBAL );
                           manager.save( (majorEntryText.equals( capLabelText )) ? Global.CLARION_OPTIONS : Global.TASK_OPTIONS );
                           SelectionTemplate.setChangeFlag( false );
                       }
                    }
              } );
        operatePanel.add( confirmButton );
        // Components for saving options.
        saveButton = new JButton( "SAVE TO FILES" );
        saveButton.setEnabled( false );
        saveButton.addActionListener(
               new  ActionListener() {
                    public void actionPerformed( ActionEvent e )
                    {
                       if( JOptionPane.showConfirmDialog( null, "Save as a new setting?",
                           "WARNING", JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION ){
                           loadSettingName = saveField.getText().trim();
                           globals[curAgentIdx].setTaskSettingName( loadSettingName );
                           manager.setSaveCode( (majorEntryText.equals( capLabelText )) ? Global.CLARION_OPTIONS : Global.TASK_OPTIONS,
                                                Global.SAVE_TO_FILES );
                           manager.save( (majorEntryText.equals( capLabelText )) ? Global.CLARION_OPTIONS : Global.TASK_OPTIONS );
                           saveButton.setEnabled( false );
                       }
                    }
              } );
        operatePanel.add( saveButton );
        saveField = new JTextField( loadSettingName, 9 );
        operatePanel.add( saveField );
        // next button.
        nextButton = new JButton( "NEXT" );
        nextButton.setEnabled( false );
        nextButton.setVisible( curAgentIdx < runningAgentNum-1 );
        nextButton.addActionListener(  new  ActionListener() {
                         public void actionPerformed( ActionEvent e )
                         {
                            curAgentIdx++;
                            curAgentIdx = Math.min(curAgentIdx, runningAgentNum-1);
                            nextButton.setEnabled( false );
                            nextButton.setVisible( curAgentIdx < runningAgentNum-1 );
                            returnButton.setEnabled( false );
                            returnButton.setVisible( curAgentIdx == runningAgentNum-1 );
                            capLabel.setText( capLabelText + note + "  Current agent = " + curAgentIdx );
                            loadSettingName = globals[curAgentIdx].getTaskSettingName() != null ?
                                              globals[curAgentIdx].getTaskSettingName() : "input a setting name";
                            loadSettingField.setText( loadSettingName );
                            saveField.setText( loadSettingName );

                            //System.out.println( "cur agent = " + curAgentIdx + ", setting name 0 = " + globals[0].getTaskSettingName()
                            //                     + ", name 1 = " + globals[1].getTaskSettingName() );

                            for( int i = 0; i < OPT_NUM; i++ )
                                 optButtons[i].setEnabled( false );
                            manager.setGlobal( globals[curAgentIdx] );
                         }
                     } );
        operatePanel.add( nextButton );
        addLoadButtonListeners();
        addFieldListeners();
    }
    // return button.
    returnButton = new JButton( "RETURN" );
    if( this instanceof GuiClarion || this instanceof GuiTask ){
        returnButton.setEnabled(false);
        returnButton.setVisible(curAgentIdx == runningAgentNum-1);
    }
    returnButton.addActionListener(  new  ActionListener() {
                     public void actionPerformed( ActionEvent e )
                     {
                         setEnabled( false );
                         setVisible( false );
                         mainFrame.setEnabled( true );
                         mainFrame.setVisible( true );
                         if( !(majorEntryText.equals( capLabelText ) || taskSpecEntryText.equals( capLabelText )) )
                             ((GuiTemplate)mainFrame).returnFrom();
                     }
                 } );
    operatePanel.add( returnButton );
    addActionListeners( mainFrame );
    addWindowListener(
          new WindowAdapter(){
             public void windowClosing( WindowEvent e ){
                 if( confirmFlag ){
                    setVisible( false );
                    setEnabled( false );
                    mainFrame.setVisible( true );
                    mainFrame.setEnabled( true );
                    if( !(majorEntryText.equals( capLabelText ) || taskSpecEntryText.equals( capLabelText )) )
                        ((GuiTemplate)mainFrame).returnFrom();
                 }
                 else
                    JOptionPane.showMessageDialog( null, "Please confirm the option setting", "WARNING",
                                                  JOptionPane.WARNING_MESSAGE );
            }
    }  );
  }

  /** add action listeners. this method needs overriding. */
  public void addActionListeners( JFrame mFrame )
  {
  }

  /** the routine of processing the changes when returning from other GUIs. */
  public void returnFrom()
  {
     if( thisFrame instanceof GuiClarion || thisFrame instanceof GuiTask ){
         returnButton.setEnabled( !confirmButton.isEnabled() );
     }
  }

  /** Updates this GUI. */
  public void updateGui()
  {
  }

/****************************** PRIVATE METHODS ********************************/
  private void addLoadButtonListeners()
  {
      // load a setting group.
    loadSettingButton.addActionListener( new ActionListener() {
       public void actionPerformed( ActionEvent e ) {
           newSettingNames = manager.searchCurrentSettings();
           currentSettingNum = newSettingNames.length;
           loadSettingMenu = new JPopupMenu( "EXISTING SETTINGS" );
           loadSettingItem = new JMenuItem[ currentSettingNum ];
           for( int i = 0; i < currentSettingNum; i++ ){
                loadSettingItem[ i ] = new JMenuItem( newSettingNames[i] );
                loadSettingItem[i].setFont( new Font( "Serif", Font.PLAIN, 14 ) );
                final int focus = i;
                loadSettingItem[ focus ].addActionListener( new ActionListener(){
                   public void actionPerformed( ActionEvent e){
                     if( JOptionPane.showConfirmDialog( null, "Do you want to load a new setting?",
                          "WARNING", JOptionPane.YES_NO_OPTION ) == JOptionPane.NO_OPTION )
                         return;
                     //if( !newSettingNames[focus].equals(loadSettingName)
                     //    || (majorEntryText.equals(capLabelText) && !configFlags[Global.CLARION_OPTIONS][curAgentIdx])
                     //    || (taskSpecEntryText.equals(capLabelText) && !configFlags[Global.TASK_OPTIONS][curAgentIdx])
                     //  )
                      {
                         loadSettingName = newSettingNames[focus];
                         loadSettingField.setText( loadSettingName );
                         processLoad();
                      }
                  }
                } );
                loadSettingMenu.add( loadSettingItem[ i ] );
           }
           loadSettingMenu.show( operatePanel, 300, 10 );
       }
    } );
  }

  private void addFieldListeners()
  {
    // load a task setting.
    loadSettingField.addActionListener( new ActionListener() {
       public void actionPerformed( ActionEvent e ) {
           String name = loadSettingField.getText().trim();
           if( name.length() > 0 && JOptionPane.showConfirmDialog( null, "Do you want to load a new setting?",
                                     "WARNING", JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION ){
               loadSettingName = name;
               //loadSettingButton.setEnabled( true );
               processLoad();
           }
           else{
               if( name.length() == 0 )
                   JOptionPane.showMessageDialog( null, "Invalid setting name, reset!", "WARNING",
                                                  JOptionPane.WARNING_MESSAGE );
               loadSettingField.setText( loadSettingName );
           }
       }
    });
    // save setting.
    saveField.addActionListener( new ActionListener() {
       public void actionPerformed( ActionEvent e ) {
           String name = saveField.getText().trim();
           if( name.length() > 0 ){
               loadSettingName = name;
               globals[curAgentIdx].setTaskSettingName( loadSettingName );
               saveButton.setEnabled( true );
           }
           else{
               JOptionPane.showMessageDialog( null, "Invalid setting name, reset!", "WARNING",
                                             JOptionPane.WARNING_MESSAGE );
               saveButton.setEnabled( false );
               saveField.setText( loadSettingName );
           }
       }
    });
  }

  protected void processLoad()
  {
    globals[curAgentIdx].setTaskSettingName( loadSettingName );
    if( majorEntryText.equals( capLabelText ) )
       ((GuiClarion)thisFrame).loadSubSys();
    else
       ((GuiTask)thisFrame).loadSubSys();
    confirmButton.setEnabled( true );
    for( int i = 0; i < OPT_NUM; i++ )
         optButtons[i].setEnabled( true );
  }
}
