package clarion.options;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import clarion.system.*;
import clarion.tools.*;
import clarion.options.transfer.*;
import clarion.options.acs.net.*;
import clarion.options.nacs.amn.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is the base class for configuration of a concrete CLARION component
 * by GUI such as configuration of RER, IRL or FR. In this class, many routines
 * for handling the GUI components such as the labels, the common buttons, the
 * text fields, the checkboxs, the panels, the 'return' buttons and the the group
 * of components relevant to BLA option.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * The concrete class for implementing the configuration of a concrete CLARION
 * component by GUI will extends this class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionTemplate extends JFrame {
  protected final Font noteFont = new java.awt.Font("Dialog", 1, 14);
  protected final String noteText = "   ( type ENTER key after typing in any information )";
  protected static boolean changeFlag;
  protected int width, height;
  protected JFrame superFrame;
  //protected Global global;
  protected TransferManager manager;
  protected Option[] options;
  protected JPanel parent;
  protected JLabel noteLabel = new JLabel();
  protected JScrollPane scrollPane;
  protected SeeSampleFrame seeFrame;

  public SelectionTemplate()
  {
     seeFrame = new SeeSampleFrame( this );
     seeFrame.setEnabled( false );
     seeFrame.setVisible( false );
  }

  public SelectionTemplate( JFrame superFrame, int width, int height, TransferManager manager )
  {
     this.superFrame = superFrame;
     this.width = width;
     this.height = height;
     //this.global = global;
     this.manager = manager;
     if( height > 720 || width > 1000 ){
         parent = new JPanel();
         Border BORDER = BorderFactory.createLineBorder(Color.black);
         initPanel( parent, BORDER, null, new Rectangle(0, 0, width, height));
         parent.setPreferredSize( new Dimension( width, height ));
         scrollPane = new JScrollPane( parent );
         this.getContentPane().add( scrollPane );
     }
     seeFrame = new SeeSampleFrame( this );
     seeFrame.setEnabled( false );
     seeFrame.setVisible( false );
  }

  //methods for user interface.
  public void initGui()
  {
  }

  /** Sets the flag tp indicate a change. */
  static public void setChangeFlag( boolean flag )
  {
    changeFlag = flag;
  }

  /** initializes a panel with relevant parameters. */
  public void initPanel( JPanel panel, Border border, LayoutManager layout, Rectangle rect )
  {
     panel.setLayout( layout );
     panel.setBorder( border );
     panel.setBounds( rect );
  }

  /** initializes a panel with relevant parameters. */
  public void initPanel( JPanel panel, Rectangle rect )
  {
     panel.setLayout( null );
     panel.setBorder( BorderFactory.createLineBorder(Color.black) );
     panel.setBounds( rect );
  }

  /** initializes a panel with relevant parameters. */
  public void initLabel( JLabel label, String text, Font font, int alignment, Rectangle rect )
  {
     label.setText( text );
     label.setFont( font );
     label.setHorizontalAlignment( alignment );
     label.setBounds( rect );
  }

  /** initializes a label with relevant parameters. */
  public void initLabel( JLabel label, String text, int alignment, Rectangle rect )
  {
     label.setText( text );
     label.setHorizontalAlignment( alignment );
     label.setBounds( rect );
  }

  /** initializes a label with relevant parameters. */
  public void initLabel( JLabel label, String text, Rectangle rect, Color color )
  {
     label.setText( text );
     label.setBounds( rect );
     label.setForeground( color );
  }

  /** initializes a label with relevant parameters. */
  public void initLabel( JLabel label, String text, Rectangle rect )
  {
     label.setText( text );
     label.setBounds( rect );
     label.setForeground( SystemColor.black );
  }

  /** initializes a text field with relevant parameters. */
  public void initField( JTextField field, String text, Rectangle rect, int length )
  {
     field.setText( text );
     field.setBounds( rect );
     field.setSelectionEnd( length );
  }

  /** initializes a text field with relevant parameters. */
  public void initField( JTextField field, String text, Rectangle rect, int length,
                         Option option, int itemIdx, int paramIdx )
  {
     field.setText( text );
     field.setBounds( rect );
     field.setSelectionEnd( length );
     this.addFieldListener( field, option, itemIdx, paramIdx );
  }

  /** initializes a text field with relevant parameters. */
  public void initField( JTextField field, String text, Rectangle rect )
  {
     field.setText( text );
     field.setBounds( rect );
  }

  /** initializes a common button with relevant parameters. */
  public void initButton( AbstractButton button, String text, Rectangle rect )
  {
     button.setText( text );
     button.setBounds( rect );
  }

  /** initializes a common button with relevant parameters. */
  public void initButton( AbstractButton button, String text, Rectangle rect, Border border )
  {
     button.setText( text );
     button.setBounds( rect );
     button.setBorder( border );
  }

  /** initializes the 'see' button with relevant parameters. */
  public void initSeeButton( AbstractButton button, Rectangle rect, final String showText )
  {
     button.setText( "see sample" );
     button.setBounds( rect );
     button.addActionListener( new ActionListener() {
         public void actionPerformed(ActionEvent e) {
              seeFrame.setEnabled( true );
              seeFrame.setVisible( true );
              seeFrame.showSample( showText );
         }
     });
  }

  /** initializes a checkbox with relevant parameters. */
  public void initCheckBox( JCheckBox box, String text, Rectangle rect, boolean status )
  {
     box.setText( text );
     box.setBounds( rect );
     box.setSelected( status );
  }

  /** initializes a group of text fields with relevant parameters (with option label). */
  public void initFieldBlock( JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              JLabel[] labels, String[] labelTexts, Rectangle[] labelRects, JTextField[] fields,
              String[] fieldTexts, Rectangle[] fieldRects, int[] fieldWidths, Option option )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    for( int i = 0; i < labels.length; i++ ){
         initLabel( labels[i], labelTexts[i], labelRects[i] );
         panel.add( labels[i] );
    }
    for( int i = 0; i < fields.length; i++ ){
         initField( fields[i], fieldTexts[i], fieldRects[i], fieldWidths[i] );
         panel.add( fields[i] );
         addFieldListener( fields[i], option, 0, i );
    }
  }

  /** initializes a group of text fields with relevant parameters (without option label). */
  public void initFieldBlock( JPanel panel, JLabel[] labels, String[] labelTexts,
              Rectangle[] labelRects, JTextField[] fields, String[] fieldTexts,
              Rectangle[] fieldRects, int[] fieldWidths, Option option, int itemIdx )
  {
    for( int i = 0; i < labels.length; i++ ){
         initLabel( labels[i], labelTexts[i], labelRects[i] );
         panel.add( labels[i] );
    }
    for( int i = 0; i < fields.length; i++ ){
         initField( fields[i], fieldTexts[i], fieldRects[i], fieldWidths[i] );
         panel.add( fields[i] );
         addFieldListener( fields[i], option, itemIdx, i );
    }
  }

  /** initializes a group of text fields with relevant parameters (without option label). */
  public void initFieldBlock( JPanel panel, JLabel[] labels, String[] labelTexts,
              Rectangle[] labelRects, JTextField[] fields, String[] fieldTexts,
              Rectangle[] fieldRects, Option option, int itemIdx )
  {
    for( int i = 0; i < labels.length; i++ ){
         initLabel( labels[i], labelTexts[i], labelRects[i] );
         panel.add( labels[i] );
    }
    for( int i = 0; i < fields.length; i++ ){
         initField( fields[i], fieldTexts[i], fieldRects[i], 3 );
         addFieldListener( fields[i], option, itemIdx, i );
         panel.add( fields[i] );
    }
  }

  /** initializes a group of text fields with relevant parameters (without option label). */
  public void initFieldBlock( JPanel panel, JLabel[] labels, String[] labelTexts,
              Rectangle[] labelRects, JTextField[] fields, String[] fieldTexts,
              Rectangle[] fieldRects, Option option, int itemIdx, int paramIdx )
  {
    for( int i = 0; i < labels.length; i++ ){
         initLabel( labels[i], labelTexts[i], labelRects[i] );
         panel.add( labels[i] );
    }
    for( int i = 0; i < fields.length; i++ ){
         initField( fields[i], fieldTexts[i], fieldRects[i], 3 );
         panel.add( fields[i] );
    }
    addFieldListener( fields, option, itemIdx, paramIdx );
  }

  /** initializes a group of buttons with relevant parameters. */
  public void initButtonBlock( JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
    }
  }

  /** initializes a group of buttons with relevant parameters. for exclusive option. */
  public void initButtonBlock( JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
         final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects, final Option option )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                    if( option.getUserChoice() != focus )
                        changeFlag = true;
                    option.setUserChoice( focus );
             }
         });
    }
    buttons[option.getUserChoice()].setSelected( true );
  }

  /** initializes a group of buttons with relevant parameters.
   *  for exclusive option with itself has one parameter field.  */
  public void initButtonBlock(JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects,
              final Option option, final int itemIdx, final JTextField field )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                  if( option.getUserChoice() != focus )
                      changeFlag = true;
                  option.setUserChoice( focus );
                  field.setEnabled( itemIdx == focus );
                  //System.out.println( option.getUserChoice() );
             }
         });
    }
    buttons[option.getUserChoice()].setSelected( true );
    // initialize component status
    field.setEnabled( itemIdx == option.getUserChoice() );
    addFieldListener( field, option, itemIdx, 0 );
  }

  /** initializes a group of buttons with relevant parameters.
   *  For exclusive option with itself has serveral buttons. */
  public void initButtonBlock(JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects,
              final Option option, final int itemIdx, final JButton[] itemButtons )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                  if( option.getUserChoice() != focus )
                      changeFlag = true;
                  option.setUserChoice( focus );
                  for( int i = 0; i < itemButtons.length; i++ )
                       itemButtons[i].setEnabled( itemIdx == focus );
             }
         });
    }
    buttons[option.getUserChoice()].setSelected( true );
    // initialize component status
    for( int i = 0; i < itemButtons.length; i++ )
         itemButtons[i].setEnabled( itemIdx == option.getUserChoice() );
  }

  /** initializes a group of buttons with relevant parameters.
   *  Consider dependency, affect only one other option. with '!= itemIdx' relation. */
  public void initButtonBlock( JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects,
              final Option option, final int itemIdx, final JRadioButton[] retComps )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                    if( option.getUserChoice() != focus )
                        changeFlag = true;
                    option.setUserChoice( focus );
                    for( int i = 0; i < retComps.length; i++ )
                         retComps[i].setEnabled( itemIdx != focus );
             }
         });
    }
    buttons[option.getUserChoice()].setSelected( true );
    // initialize component status.
    for( int i = 0; i < retComps.length; i++ )
         retComps[i].setEnabled( itemIdx != option.getUserChoice() );
  }

  /** initializes a group of buttons with relevant parameters.
   *  For exclusive option, affects several parameter fields (one item of its own
   *  or other option has). */
  public void initButtonBlock(JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects,
              final Option option, final int itemIdx, final JTextField[] fields )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                  if( option.getUserChoice() != focus )
                      changeFlag = true;
                  option.setUserChoice( focus );
                  for( int i = 0; i < fields.length; i++ )
                       fields[i].setEnabled( itemIdx == focus );
             }
         });
    }
    buttons[option.getUserChoice()].setSelected( true );
    // initialize component status
    for( int i = 0; i < fields.length; i++ )
         fields[i].setEnabled( itemIdx == option.getUserChoice() );
  }

  /** initializes a group of buttons with relevant parameters.
   *  For exclusive option with each item has some parameter fields (no loading
   *  button). */
  public void initButtonBlock(JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
         final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects,
         final Option option, final JTextField[][] fields )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                 if( option.getUserChoice() != focus )
                     changeFlag = true;
                 option.setUserChoice( focus );
                 for( int j = 0; j < fields.length; j++ )
                      if( fields[j] != null )
                          for( int k = 0; k < fields[j].length; k++ )
                               fields[j][k].setEnabled( j == focus );
             }
         });
    }
    buttons[option.getUserChoice()].setSelected( true );
    // initialize component status
    for( int j = 0; j < fields.length; j++ )
         if( fields[j] != null )
             for( int k = 0; k < fields[j].length; k++ )
                  fields[j][k].setEnabled( j == option.getUserChoice() );
  }

  /** initializes a group of buttons with relevant parameters.
   *  For exclusive option with each item has some parameter fields (loading button). */
  public void initButtonBlock(JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
         final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects, final Option option,
         final JTextField[][] fields, final int itemIdx, final JButton[] userDefinedButtons )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                 if( option.getUserChoice() != focus )
                     changeFlag = true;
                 option.setUserChoice( focus );
                 for( int i = 0; i < userDefinedButtons.length; i++ )
                      userDefinedButtons[i].setEnabled( itemIdx == focus );
                 for( int j = 0; j < fields.length; j++ )
                      if( fields[j] != null )
                          for( int k = 0; k < fields[j].length; k++ )
                               fields[j][k].setEnabled( j == focus );
             }
         });
    }
    buttons[option.getUserChoice()].setSelected( true );
    // initialize component status
    for( int i = 0; i < userDefinedButtons.length; i++ )
         userDefinedButtons[i].setEnabled( itemIdx == option.getUserChoice() );
    for( int j = 0; j < fields.length; j++ )
         if( fields[j] != null )
             for( int k = 0; k < fields[j].length; k++ )
                  fields[j][k].setEnabled( j == option.getUserChoice() );
  }

  /** initializes a group of buttons with relevant parameters.
   *  Consider dependency, affect only one panel (one set of options). */
  public void initButtonBlock( JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects,
              final Option option, final int retIdx, final JPanel retPanel, final boolean[] compOns )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                 if( option.getUserChoice() != focus )
                     changeFlag = true;
                 option.setUserChoice( focus );
                 if( retIdx == focus ){
                     if( retPanel.isEnabled() )
                         return;
                     retPanel.setEnabled( true );
                     for( int j = 0; j < compOns.length; j++ )
                          retPanel.getComponent(j).setEnabled( compOns[j] );
                 }
                 else{
                     if( !retPanel.isEnabled() )
                         return;
                     for( int j = 0; j < compOns.length; j++ )
                          compOns[ j ] = retPanel.getComponent(j).isEnabled();
                     retPanel.setEnabled( false );
                     for( int j = 0; j < retPanel.getComponentCount(); j++ )
                          retPanel.getComponent(j).setEnabled( false );
                 }
             }
         });
     }
     buttons[option.getUserChoice()].setSelected( true );
  }

  /** initializes a group of buttons with relevant parameters.
   *  Consider dependency, one choice affects several panels (several sets of options). */
  public void initButtonBlock( JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects,
              final Option option, final int retIdx, final JPanel[] retPanels, final boolean[][] compOns )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                 if( option.getUserChoice() != focus )
                     changeFlag = true;
                 option.setUserChoice( focus );
                 if( retIdx == focus ){
                     for( int i = 0; i < retPanels.length; i++ ){
                          if( retPanels[i].isEnabled() )
                              continue;
                          retPanels[i].setEnabled( true );
                          for( int j = 0; j < compOns[i].length; j++ )
                               retPanels[i].getComponent(j).setEnabled( compOns[i][j] );
                     }
                 }
                 else{
                     for( int i = 0; i < retPanels.length; i++ ){
                          if( !retPanels[i].isEnabled() )
                              continue;
                          for( int j = 0; j < compOns[i].length; j++ )
                               compOns[ i ][ j ] = retPanels[i].getComponent(j).isEnabled();
                          retPanels[i].setEnabled( false );
                          for( int j = 0; j < retPanels[i].getComponentCount(); j++ )
                               retPanels[i].getComponent(j).setEnabled( false );
                     }
                 }
             }
         });
     }
     buttons[option.getUserChoice()].setSelected( true );
  }

  /** initializes a group of buttons with relevant parameters.
   *  Consider dependency, several choices affect several panels( several sets of
   *  options ). */
  public void initButtonBlock( JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects,
              final Option option, final JPanel[] retPanels, final boolean[][] compOns )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                 if( option.getUserChoice() != focus )
                     changeFlag = true;
                 option.setUserChoice( focus );
                 if( retPanels[focus].isEnabled() )
                     return;
                 else{
                     retPanels[focus].setEnabled( true );
                     for( int k = 0; k < retPanels[focus].getComponentCount(); k++ )
                          retPanels[focus].getComponent(k).setEnabled( compOns[focus][k] );
                 }
                 for( int j = 0;  j < retPanels.length; j++ )
                      if( j != focus ){
                          if( !retPanels[j].isEnabled() )
                              continue;
                          retPanels[j].setEnabled( false );
                          for( int k = 0; k < retPanels[j].getComponentCount(); k++ )
                               compOns[j][k] = retPanels[j].getComponent(k).isEnabled();
                          for( int k = 0; k < retPanels[j].getComponentCount(); k++ )
                               retPanels[j].getComponent(k).setEnabled( false );
                     }
                }
         });
     }
     buttons[option.getUserChoice()].setSelected( true );
  }

  /** initializes a group of buttons with relevant parameters.
   *  For two-level dependency. */
  public void initButtonBlock( JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
              final JRadioButton[] buttons, String[] buttonTexts, Rectangle[] buttonRects,
              final Option option, final int retIdx, final JPanel retPanel, final boolean[] compOns,
              final Option retOption, final JPanel[] subRetPanels, final boolean[][] subCompOns  )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    ButtonGroup buttonGroup = new ButtonGroup();
    for( int i = 0; i < buttons.length; i++ ){
         initButton( buttons[i], buttonTexts[i], buttonRects[i] );
         buttonGroup.add( buttons[i]);
         panel.add( buttons[i], null );
         final int focus = i;
         buttons[i].addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                 if( option.getUserChoice() != focus )
                     changeFlag = true;
                 option.setUserChoice( focus );
                 if( retIdx == focus ){
                     if( retPanel.isEnabled() )
                         return;
                     retPanel.setEnabled( true );
                     for( int j = 0; j < compOns.length; j++ )
                          retPanel.getComponent(j).setEnabled( compOns[j] );
                     for( int i = 0; i < subRetPanels.length; i++ )
                           if( retOption.getUserChoice() == i ){
                               subRetPanels[i].setEnabled( true );
                               for( int j = 0; j < subCompOns[i].length; j++ )
                                    subRetPanels[i].getComponent(j).setEnabled( subCompOns[i][j] );
                           }
                 }
                 else{
                     if( !retPanel.isEnabled() )
                         return;
                     for( int j = 0; j < compOns.length; j++ )
                          compOns[ j ] = retPanel.getComponent(j).isEnabled();
                     retPanel.setEnabled( false );
                     for( int j = 0; j < retPanel.getComponentCount(); j++ )
                          retPanel.getComponent(j).setEnabled( false );
                     for( int i = 0; i < subRetPanels.length; i++ )
                           if( retOption.getUserChoice() == i ){
                               for( int j = 0; j < subCompOns[i].length; j++ )
                                    subCompOns[i][j] = subRetPanels[i].getComponent(j).isEnabled();
                               subRetPanels[i].setEnabled( false );
                               for( int j = 0; j < subRetPanels[i].getComponentCount(); j++ )
                                    subRetPanels[i].getComponent(j).setEnabled( false );
                           }
                 }
             }
         });
     }
     buttons[option.getUserChoice()].setSelected( true );
  }

  /** initializes a group of checkboxes with relevant parameters. */
  public void initCheckBoxBlock( JPanel panel, JLabel optLabel, String optText, Rectangle optRect,
         final JCheckBox[] boxes, String[] boxTexts, Rectangle[] boxRects, final Option option )
  {
    initLabel( optLabel, optText, optRect );
    panel.add( optLabel );
    for( int i =  0; i < boxes.length; i++ ){
         initCheckBox( boxes[i], boxTexts[i], boxRects[i],
                       option.getOptionItem(0).getParameter(i).getBoolValue() );
         panel.add( boxes[i], null );
         final int focus = i;
         boxes[i].addItemListener( new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
               option.getOptionItem(0).getParameter(focus).setBoolValue(
                       e.getStateChange() == ItemEvent.SELECTED );
            }
         });
    }
  }

  /** initializes buttons in I/O configuration with relevant parameters. */
  public void initDimButton( final SelectionTemplate template, JButton button, final OptionDims optDims )
  {
      button.addActionListener(new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) {
            //template.setEnabled( false );
            //template.setVisible( false );
            optDims.setEnabled( true );
            optDims.setVisible( true );
         }
    });
  }

  /** initializes 'return' button with relevant parameters.  */
  public void initReturnButton( AbstractButton button, Rectangle rect )
  {
     button.setBounds( rect );
     button.addActionListener(new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) {
            setEnabled( false );
            setVisible( false );
            superFrame.setEnabled( true );
            superFrame.setVisible( true );
         }
    });
  }

  /** initializes 'return' button with relevant parameters.  */
  public void initReturnButton( AbstractButton button, String text, Rectangle rect )
  {
     button.setText( text );
     button.setBounds( rect );
     button.addActionListener(new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) {
            setEnabled( false );
            setVisible( false );
            superFrame.setEnabled( true );
            superFrame.setVisible( true );
            ((GuiTemplate)superFrame).setConfirmStatus( changeFlag );
            ((GuiTemplate)superFrame).returnFrom();
         }
    });
  }

  /** initializes 'return' button with relevant parameters.  */
  public void initReturnButton( AbstractButton button, String text, Rectangle rect,
              final int setIdx, final int command )
  {
     button.setText( text );
     button.setBounds( rect );
     button.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) {
            setEnabled( false );
            setVisible( false );
            ((GuiTemplate)superFrame).setConfirmStatus( changeFlag );
            ((GuiAcsNet)superFrame).returnFrom( setIdx, command );
         }
    });
  }

  // LOADING OPTION
  /*
  public void initLoadBlock( JPanel panel, JButton button, Rectangle buttonRect, JLabel fLabel,
              Rectangle fRect, JLabel mLabel, Rectangle mRect, JTextField fField, Rectangle fFieldRect,
              JTextField mField, Rectangle mFieldRect, Option option, int itemIdx )
  {
     initButton( button, "load", buttonRect );
     initLabel( fLabel, "file to load", fRect );
     initLabel( mLabel, "method to call", mRect );
     initField( fField, option.getOptionItem(itemIdx).getParameter(0).getStringValue(), fFieldRect );
     initField( mField, option.getOptionItem(itemIdx).getParameter(1).getStringValue(), mFieldRect );
     panel.add( button );
     panel.add( fLabel );
     panel.add( mLabel );
     panel.add( fField );
     panel.add( mField );
     addLoadListeners( button, fField, mField, option, itemIdx );
  }
  */

  /** initializes user defined OPTION with relevant parameters.  */
  public void initUserDefinedBlock( JPanel panel, JButton[] buttons, Rectangle[] buttonRects,
                                    final String[] texts )
  {
     initButton( buttons[0], "see sample", buttonRects[0] );
     initButton( buttons[1], "explanation", buttonRects[1] );
     panel.add( buttons[0] );
     panel.add( buttons[1] );
     for( int i = 0; i < 2; i++ ){
          final int focus = 1 - i;
          buttons[i].addActionListener( new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  seeFrame.setEnabled( true );
                  seeFrame.setVisible( true );
                  seeFrame.showSample( texts[focus] );
              }
             } );
     }
  }

  /** initializes BLA OPTION with relevant parameters.  */
  public void initBlaBlock( JPanel blaPanel, String optText, final Option option )
  {
     JLabel optLabel_7 = new JLabel();
     final JRadioButton[] blaButtons = new JRadioButton[ 2 ];
     for( int i = 0; i < 2; i++ )
          blaButtons[i] = new JRadioButton();
     JLabel blaParaLabels[][] = new JLabel[2][3];
     for( int i = 0; i < 2; i++ )
       for( int j = 0; j < 3; j++ )
            blaParaLabels[i][j] = new JLabel();
     final JTextField blaParaFields[][] = new JTextField[2][];
     blaParaFields[0] = new JTextField[1];
     for( int i = 0; i < 1; i++ )
          blaParaFields[0][i] = new JTextField();
     blaParaFields[1] = new JTextField[3];
     for( int i = 0; i < 3; i++ )
          blaParaFields[1][i] = new JTextField();
     String[] blaTexts = { "constant base-level activation", "recency-based base-level activation" };
     Rectangle[] blaRects = { new Rectangle(8, 128, 226, 17), new Rectangle(8, 30, 260, 17)  };
     initButtonBlock( blaPanel, optLabel_7, optText, new Rectangle(7, 6, 290, 17),
                      blaButtons, blaTexts, blaRects );
     blaButtons[option.getUserChoice()].setSelected( true );
     for( int i = 0; i < blaButtons.length; i++ )
          blaButtons[i].addActionListener( new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                 for( int i = 0; i < 2; i++ )
                   if( e.getSource() == blaButtons[i] ){
                       if( option.getUserChoice() != i )
                           changeFlag = true;
                       option.setUserChoice( i );
                       if( i == 0 ){
                           blaParaFields[0][0].setEnabled( true );
                           for( int j = 0; j < blaParaFields[1].length; j++ )
                                blaParaFields[1][j].setEnabled( false );
                       }
                       else{
                           blaParaFields[0][0].setEnabled( false );
                           for( int j = 0; j < blaParaFields[1].length; j++ )
                                blaParaFields[1][j].setEnabled( true );
                       }
                   }
               }
          });
     String blaParaLabelTexts[][] = {
            { "Bj =", "formula : Bj = iBj + c * sum( exp( tl,-d ) ) (l = 0",
              "to n, indicates the lth recent use of that item )"}, {"c =", "d =", "iBj ="} };
     double bla = option.getOptionItem(0).getParameter(0).getDoubleValue();
     double c = option.getOptionItem(1).getParameter(0).getDoubleValue();
     double d = option.getOptionItem(1).getParameter(1).getDoubleValue();
     double iBla = option.getOptionItem(1).getParameter(2).getDoubleValue();
     String blaParaFieldTexts[][] = { { String.valueOf(bla) },
            {String.valueOf(c), String.valueOf(d), String.valueOf(iBla)} };
     Rectangle blaParaLabelRects[][] = {
        {new Rectangle(28, 148, 22, 17),new Rectangle(28, 70, 248, 17),new Rectangle(28, 90, 270, 17)},
        {new Rectangle(28, 50, 24, 17),new Rectangle(86, 50, 24, 17),new Rectangle(144, 50, 24, 17)}};
     Rectangle blaParaFieldRects[][] = {
        { new Rectangle(54, 148, 28, 17)}, {new Rectangle(46, 50, 26, 17),
          new Rectangle(104, 50, 26, 17), new Rectangle(172, 50, 26, 17) } };
     for( int i = 0; i < blaParaLabels.length; i++ )
       for( int j = 0; j < blaParaLabels[i].length; j++ ){
            initLabel( blaParaLabels[i][j],blaParaLabelTexts[i][j],blaParaLabelRects[i][j] );
            blaPanel.add( blaParaLabels[i][j] );
       }
     for( int i = 0; i < blaParaFields.length; i++ )
       for( int j = 0; j < blaParaFields[i].length; j++ ){
            initField( blaParaFields[i][j],blaParaFieldTexts[i][j],blaParaFieldRects[i][j] );
            addFieldListener( blaParaFields[i][j], option, i, j );
            blaPanel.add( blaParaFields[i][j] );
       }
     // initialize component status.
     if( option.getUserChoice() == 0 ){
         blaParaFields[0][0].setEnabled( true );
         for( int j = 0; j < blaParaFields[1].length; j++ )
              blaParaFields[1][j].setEnabled( false );
     }
     else{
         blaParaFields[0][0].setEnabled( false );
         for( int j = 0; j < blaParaFields[1].length; j++ )
              blaParaFields[1][j].setEnabled( true );
     }
  }

  /** initializes Utility OPTION with relevant parameters.  */
  public void initUtilityBlock( JPanel utlPanel, String optText, final Option option,
                                String[] utlUDTexts )
  {
     JLabel utlLabel = new JLabel();
     final JRadioButton[] utlButtons = new JRadioButton[3];
     for( int i = 0; i < 3; i++ )
          utlButtons[i] = new JRadioButton();
     JLabel[][] utlLabels = new JLabel[2][];
     utlLabels[0] = new JLabel[3];
     for( int i = 0; i < 3; i++ )
          utlLabels[0][i] = new JLabel();
     utlLabels[1] = new JLabel[5];
     for( int i = 0; i < 5; i++ )
          utlLabels[1][i] = new JLabel();
     final JTextField[][] utlParaFields = new JTextField[2][];
     utlParaFields[0] = new JTextField[3];
     for( int i = 0; i < 3; i++ )
          utlParaFields[0][i] = new JTextField();
     utlParaFields[1] = new JTextField[4];
     for( int i = 0; i < 4; i++ )
          utlParaFields[1][i] = new JTextField();
     final JButton[] utlUDButtons = new JButton[2];
     utlUDButtons[0] = new JButton();
     utlUDButtons[1] = new JButton();
     initLabel( utlLabel, "Rule utility options", new Rectangle(6, 0, 128, 17) );
     utlPanel.add( utlLabel );
     initButton( utlButtons[0], "constant", new Rectangle(6, 130, 120, 17));
     initButton( utlButtons[1], "using function", new Rectangle(6, 24, 120, 17));
     initButton( utlButtons[2], "user defined", new Rectangle(6, 216, 120, 17));
     ButtonGroup buttonGroup = new ButtonGroup();
     for( int i = 0; i < 3; i++ ){
          buttonGroup.add( utlButtons[i] );
          utlPanel.add( utlButtons[i] );
     }
     utlButtons[option.getUserChoice()].setSelected( true );
     for( int i = 0; i < utlButtons.length; i++ )
          utlButtons[i].addActionListener( new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                 for( int i = 0; i < 3; i++ )
                   if( e.getSource() == utlButtons[i] ){
                       if( option.getUserChoice() != i )
                           changeFlag = true;
                       option.setUserChoice( i );
                       if( i == 2 )
                           for( int j = 0; j < 2; j++ )
                                utlUDButtons[j].setEnabled( true );
                       else
                           for( int j = 0; j < utlParaFields[i].length; j++ )
                                utlParaFields[i][j].setEnabled( true );
                   }
                   else{
                       if( i == 2 )
                           for( int j = 0; j < 2; j++ )
                                utlUDButtons[j].setEnabled( false );
                       else
                           for( int j = 0; j < utlParaFields[i].length; j++ )
                                utlParaFields[i][j].setEnabled( false );
                   }
               }
          });
     String[][] utlLabelTexts = {{ "benefit =", "cost =", "v =" }, { "c7 =", "c8 =",
                                   "cost =","v =", "benefit = ( c7 + PM ) / (c8 + PM + NM )"}};
     Rectangle[][] utlLabelRects = {
      { new Rectangle(24, 150, 50, 17),new Rectangle(24, 170, 36, 17),new Rectangle(24, 190, 16, 17)},
      { new Rectangle(44, 64, 24, 17), new Rectangle(120, 64, 24, 17),
        new Rectangle(24, 84, 36, 17), new Rectangle(24, 104, 16, 17),new Rectangle(24, 44, 210, 17)} };
     String[][] utlFieldTexts = {
        { String.valueOf(option.getOptionItem(0).getParameter(0).getDoubleValue()),
          String.valueOf(option.getOptionItem(0).getParameter(1).getDoubleValue()),
          String.valueOf(option.getOptionItem(0).getParameter(2).getDoubleValue())},
        { String.valueOf(option.getOptionItem(1).getParameter(0).getDoubleValue()),
          String.valueOf(option.getOptionItem(1).getParameter(1).getDoubleValue()),
          String.valueOf(option.getOptionItem(1).getParameter(2).getDoubleValue()),
          String.valueOf(option.getOptionItem(1).getParameter(3).getDoubleValue()) }};
     Rectangle[][] utlFieldRects = {
       { new Rectangle(84, 150, 32, 17), new Rectangle(70, 170, 32, 17), new Rectangle(50, 190, 32, 17)},
       { new Rectangle(68, 64, 32, 17), new Rectangle(144, 64, 32, 17),
         new Rectangle(70, 84, 32, 17), new Rectangle(50, 104, 32, 17)} };
     initFieldBlock( utlPanel, utlLabels[0], utlLabelTexts[0], utlLabelRects[0],
                     utlParaFields[0], utlFieldTexts[0], utlFieldRects[0], option, 0 );
     initFieldBlock( utlPanel, utlLabels[1], utlLabelTexts[1], utlLabelRects[1],
                     utlParaFields[1], utlFieldTexts[1], utlFieldRects[1], option, 1 );
     // user defined block
     Rectangle[] utlUDRects = { new Rectangle(24, 240, 160, 17), new Rectangle(24, 260, 160, 17) };
     initUserDefinedBlock( utlPanel, utlUDButtons, utlUDRects, utlUDTexts );
     // initialize component status.
     for( int i = 0; i < 3; i++ )
          if( option.getUserChoice() == i ){
              if( i < 2 )
                  for( int j = 0; j < utlParaFields[i].length; j++ )
                       utlParaFields[i][j].setEnabled( true );
              else
                  for( int j = 0; j < 2; j++ )
                       utlUDButtons[j].setEnabled( true );
          }
          else{
              if( i < 2 )
                  for( int j = 0; j < utlParaFields[i].length; j++ )
                       utlParaFields[i][j].setEnabled( false );
              else
                  for( int j = 0; j < 2; j++ )
                      utlUDButtons[j].setEnabled( false );
         }
  }

  /** initializes a panel and its components with relevant parameters.
   *  For one-level dependency. */
  public void initPanelStatus( Option option, int retIdx, JPanel retPanel, boolean[] compOns )
  {
     for( int i = 0; i < compOns.length; i++ )
          compOns[i] = retPanel.getComponent(i).isEnabled();
     if( option.getUserChoice() == retIdx ){
         retPanel.setEnabled( true );
         for( int j = 0; j < retPanel.getComponentCount(); j++ )
              retPanel.getComponent(j).setEnabled( compOns[j] );
     }
     else{
         retPanel.setEnabled( false );
         for( int j = 0; j < retPanel.getComponentCount(); j++ )
              retPanel.getComponent(j).setEnabled( false );
     }
  }

  /** initializes a panel and its components with relevant parameters.
   *  For two-level dependency. */
  public void initPanelStatus( Option option, int retIdx, JPanel retPanel, boolean[] compOns,
                               Option retOption, JPanel[] subRetPanels, boolean[][] subCompOns )
  {
     for( int i = 0; i < compOns.length; i++ )
          compOns[i] = retPanel.getComponent(i).isEnabled();
     for( int i = 0; i < subRetPanels.length; i++ )
          initPanelStatus( retOption, i, subRetPanels[i], subCompOns[i] );
     if( option.getUserChoice() == retIdx ){
         retPanel.setEnabled( true );
         for( int i = 0; i < retPanel.getComponentCount(); i++ )
              retPanel.getComponent(i).setEnabled( compOns[i] );
     }
     else{
         retPanel.setEnabled( false );
         for( int j = 0; j < retPanel.getComponentCount(); j++ )
              retPanel.getComponent(j).setEnabled( false );
         for( int i = 0; i < subRetPanels.length; i++ )
              if( subRetPanels[i].isEnabled() ){
                  for( int j = 0; j < subRetPanels[i].getComponentCount(); j++ )
                       subRetPanels[i].getComponent(j).setEnabled( false );
              }
     }
  }

  /** Updates a BLA panel with relevant parameters. */
  public void updateBlaPanel( JPanel blaPanel, Option option )
  {
     /*
     Point points[][] = { { new Point(60, 60)},
           {new Point(60, 120), new Point(120, 120), new Point(180, 120)}};
     int choice = option.getUserChoice();
     for( int i = 0; i < points[1-choice].length; i++ )
         ((JTextField)blaPanel.getComponentAt(points[1-choice][i])).setEnabled( false );
     */
     Point points[][] = { { new Point(60, 160)},
           {new Point(60, 60), new Point(120, 60), new Point(180, 60)}};
     int choice = option.getUserChoice();
     for( int i = 0; i < points[1-choice].length; i++ )
         ((JTextField)blaPanel.getComponentAt(points[1-choice][i])).setEnabled( false );
  }

  /** Updates this GUI resulting from the changes from upper level. override this
   *  method if necessary. */
  public void update()
  {
  }

/*************************** ADD LISENTERS ************************************/
  /** Adds the listeners to handle the field GUI component. */
  public void addFieldListener( final JTextField field, final Option option, final int itemIdx,
                                final int paraIdx )
  {
     field.addActionListener( new ActionListener() {
        public void actionPerformed( ActionEvent e ) {
           try{
             final int majorType = option.getOptionItem(itemIdx).getParameter(paraIdx).getMajorType();
             if( majorType == Global.PRIM_TYPE ){
                 int minorType = option.getOptionItem(itemIdx).getParameter(paraIdx).getMinorType();
                 switch( minorType ){
                  case Global.BOOL:
                       final boolean value1 = Boolean.getBoolean( field.getText().trim() );
                       if( option.getOptionItem(itemIdx).getParameter(paraIdx).getBoolValue() != value1 )
                           changeFlag = true;
                       option.getOptionItem(itemIdx).getParameter(paraIdx).setBoolValue( value1 );
                       break;
                  case Global.INT:
                       final int value2 = Integer.parseInt( field.getText().trim() );
                       if( option.getOptionItem(itemIdx).getParameter(paraIdx).getIntValue() != value2 )
                           changeFlag = true;
                       option.getOptionItem(itemIdx).getParameter(paraIdx).setIntValue( value2 );
                       break;
                  case Global.DOUBLE:
                       double value3 = Double.parseDouble( field.getText().trim() );
                       if( option.getOptionItem(itemIdx).getParameter(paraIdx).getDoubleValue() != value3 )
                           changeFlag = true;
                       option.getOptionItem(itemIdx).getParameter(paraIdx).setDoubleValue( value3 );
                       break;
                  case Global.STRING:
                       String value4 = field.getText().trim();
                       if( option.getOptionItem(itemIdx).getParameter(paraIdx).getStringValue() != value4 )
                           changeFlag = true;
                       option.getOptionItem(itemIdx).getParameter(paraIdx).setStringValue( value4 );
                       break;
                 }
             }
             else{
                  String value = field.getText().trim();
                  if( option.getOptionItem(itemIdx).getParameter(paraIdx).getStringValue() != value )
                      changeFlag = true;
                  option.getOptionItem(itemIdx).getParameter(paraIdx).setStringValue( value );
             }
           }
           catch( NumberFormatException e1 ){
              JOptionPane.showMessageDialog( null, "Not a number, reset !",
                                             "Error", JOptionPane.ERROR_MESSAGE );
              //field.setText());
           }
        }
     });
  }

  /** Adds the listeners to handle the field GUI component. */
  public void addFieldListener( final JTextField[] fields, final Option option, final int itemIdx,
                                final int paraIdx )
  {
     for( int idx = 0; idx < fields.length; idx++ ){
          final int i = idx;
          fields[i].addActionListener( new ActionListener() {
               public void actionPerformed( ActionEvent e ) {
                  try{
                      int minorType = option.getOptionItem(itemIdx).getParameter(paraIdx).getMinorType();
                      switch( minorType ){
                       case Global.BOOL:
                            final boolean value1 = Boolean.getBoolean( fields[i].getText().trim() );
                            option.getOptionItem(itemIdx).getParameter(paraIdx).setBoolValue( i, value1 );
                            break;
                       case Global.INT:
                            final int value2 = Integer.parseInt( fields[i].getText().trim() );
                            option.getOptionItem(itemIdx).getParameter(paraIdx).setIntValue( i, value2 );
                            break;
                       case Global.DOUBLE:
                            double value3 = Double.parseDouble( fields[i].getText().trim() );
                            option.getOptionItem(itemIdx).getParameter(paraIdx).setDoubleValue( i, value3 );
                            break;
                       case Global.STRING:
                            String value4 = fields[i].getText().trim();
                            option.getOptionItem(itemIdx).getParameter(paraIdx).setStringValue( i, value4 );
                            break;
                      }
                  }
                  catch( NumberFormatException e1 ){
                     JOptionPane.showMessageDialog( null, "Not a number, reset !",
                                                    "Error", JOptionPane.ERROR_MESSAGE );
                  }
               }
         });
     }
  }

  /** Adds the listeners to handle the 'load' button. */
  public void addLoadListeners( JButton button, final JTextField f1, final JTextField f2,
                                final Option option, final int itemIdx )
  {
     button.addActionListener( new ActionListener() {
       public void actionPerformed( ActionEvent e ) {
         WriteFile wFile = new WriteFile();
         RandomAccessFile file = wFile.openFile();
         String name = wFile.getFullFileName();
         if( file != null ){
             f1.setText( name );
             option.getOptionItem(itemIdx).getParameter(0).setStringValue( name );
         }
         else{
             JOptionPane.showMessageDialog( null, "Please select a file",
                         "WARNING", JOptionPane.WARNING_MESSAGE );
         }
     }
     });
     f1.addActionListener( new ActionListener() {
       public void actionPerformed( ActionEvent e ) {
           option.getOptionItem(itemIdx).getParameter(0).setStringValue( f1.getText().trim() );
       }
     });
     f2.addActionListener( new ActionListener() {
       public void actionPerformed( ActionEvent e ) {
           option.getOptionItem(itemIdx).getParameter(1).setStringValue( f2.getText().trim() );
       }
     });
  }

 /** Adds the listeners to handle the 'load' button. used for reinforcement function
  *  option. */
  public void addLoadListeners( JButton button, final JTextField f1, final JTextField f2,
              final Option[][][][][] option, final int optIdx, final int itemIdx )
  {
     button.addActionListener( new ActionListener() {
       public void actionPerformed( ActionEvent e ) {
         WriteFile wFile = new WriteFile();
         RandomAccessFile file = wFile.openFile();
         String name = wFile.getFullFileName();
         if( file != null ){
             f1.setText( name );
             for( int i = 0; i < option.length; i++ )
              for( int j = 0; j < option[i].length; i++ ){
                   Option opt = option[i][j][Global.ACS_IDN][0][optIdx];
                   opt.getOptionItem(itemIdx).getParameter(0).setStringValue( name );
             }
         }
         else{
             JOptionPane.showMessageDialog( null, "Please select a file",
                         "WARNING", JOptionPane.WARNING_MESSAGE );
         }
     }
     });
     f1.addActionListener( new ActionListener() {
       public void actionPerformed( ActionEvent e ) {
             for( int i = 0; i < option.length; i++ )
              for( int j = 0; j < option[i].length; i++ ){
                   Option opt = option[i][j][Global.ACS_IDN][0][optIdx];
                   opt.getOptionItem(itemIdx).getParameter(0).setStringValue( f1.getText().trim() );
             }
       }
     });
     f2.addActionListener( new ActionListener() {
       public void actionPerformed( ActionEvent e ) {
             for( int i = 0; i < option.length; i++ )
              for( int j = 0; j < option[i].length; i++ ){
                   Option opt = option[i][j][Global.ACS_IDN][0][optIdx];
                   opt.getOptionItem(itemIdx).getParameter(0).setStringValue( f2.getText().trim() );
             }
       }
     });
  }
}
