package clarion.options.acs.net;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to the overall ACS networks by GUI.
 * It contains the options such as eligibility functions, reinforcement functions
 * coordination mode between external network and Goal structure.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of overall ACS networks configuration and it is instantiated in
 * <tt>GuiAcsNets</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionAcsCor extends SelectionTemplate{


  JLabel capLabel = new JLabel();
  // reinforcement function spec
  boolean[] rfCompOns;
  JPanel rfPanel = new JPanel();
  JLabel panelLabel_2 = new JLabel();
  JRadioButton[] rfButtons = new JRadioButton[2]; {
      rfButtons[0] = new JRadioButton();
      rfButtons[1] = new JRadioButton();
  }
  // reinforcement function(all)
  JPanel rfAllPanel = new JPanel();
  JLabel panelLabel_6 = new JLabel();
  JRadioButton[] rfAllButtons = new JRadioButton[2]; {
       rfAllButtons[0] = new JRadioButton();
       rfAllButtons[1] = new JRadioButton();
  }
 JButton[] rfAllUDButtons = new JButton[2]; {
   rfAllUDButtons[0] = new JButton();
   rfAllUDButtons[1] = new JButton();
 }
  // eligibility
  JPanel eligPanel = new JPanel();
  JLabel eligLabel = new JLabel();
  JRadioButton[] eligButtons = new JRadioButton[2]; {
     for( int i = 0; i < 2; i++ )
          eligButtons[i] = new JRadioButton();
  }
  // coordination
  JPanel corPanel = new JPanel();
  JLabel corExLabel = new JLabel();
  JRadioButton[] corExButtons = new JRadioButton[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          corExButtons[i] = new JRadioButton();
  }
  JLabel corExGoalLabel = new JLabel();
  JLabel[] corExGoalLabels = new JLabel[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          corExGoalLabels[i] = new JLabel();
  }
  JTextField[] corExGoalFields = new JTextField[ 2 ]; {
     for( int i = 0; i < 2; i++ )
          corExGoalFields[i] = new JTextField();
  }
  // return button
  JButton returnButton = new JButton();

  public SelectionAcsCor( JFrame superFrame, int width, int height, TransferManager manager )
  {
    super( superFrame, width, height, manager );
    initGui();
  }

  /** initialize the GUI components in this frame. */
  public void initGui()
  {
    final int ELIG = 0, REIN = 1, REIN_ALL = 2, COR_EX = 3, COR_EX_GS = 4;

    options = manager.getOptions( Global.ACS, Global.ACS_NET, Global.ACS_COR, 0, 0, 0 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    setSize( width, height );
    this.getContentPane().setLayout( null );
    // caption.
    initLabel( capLabel, "OPTIONS FOR ALL IDNS AND CORRESPONDING RULE GROUPS", new Font("Dialog",1,14),
               SwingConstants.CENTER, new Rectangle( 100, 0, 500, 26) );
    // Reinforcement function for all network
    initPanel( rfAllPanel, BORDER, null, new Rectangle(10, 120, 360, 120) );
    Rectangle[] rfAllUDRects = { new Rectangle(24, 68, 160, 17), new Rectangle(24, 88, 160, 17) };
    String[] rfAllUDTexts = { TaskClarion.getSample( TaskClarion.ACS, TaskClarion.REIN ),
                              TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.REIN ) };
    initUserDefinedBlock( rfAllPanel, rfAllUDButtons, rfAllUDRects, rfAllUDTexts );
    String[] rfAllTexts = {"default (no reinforcement function)", "user defined"};
    Rectangle[] rfAllRects = {new Rectangle(4, 28, 240, 17),new Rectangle(4, 48, 100, 17)};
    initButtonBlock( rfAllPanel, panelLabel_6, "Reinforcement function for all networks",
          new Rectangle(4, 4, 230, 17), rfAllButtons, rfAllTexts, rfAllRects, options[REIN_ALL],
          1, rfAllUDButtons );
    rfCompOns = new boolean[rfAllPanel.getComponentCount()];
    this.initPanelStatus( options[REIN], Global.REIN_ALL, rfAllPanel, rfCompOns );
    // Reinforcement function specification
    initPanel( rfPanel, BORDER, null, new Rectangle(10, 26, 360, 90) );
    String[] rfTexts = {"One function for all","One function for each network"};
    Rectangle[] rfRects = {new Rectangle(4, 28, 140, 17),new Rectangle(4, 58, 205, 17)};
    initButtonBlock( rfPanel, panelLabel_2, "Reinforcement function specification",
          new Rectangle(4, 4, 240, 17), rfButtons, rfTexts, rfRects, options[REIN], 0,
          rfAllPanel, rfCompOns);
    // eligibility
    initPanel( eligPanel, BORDER, null, new Rectangle(380, 26, 350, 90) );
    String[] eligTexts = { "default (all are eligible)",
                           "user defined  ( to be defined on another screen )" };
    Rectangle[] eligRects = {new Rectangle(8, 28, 200, 17),new Rectangle(8, 58, 340, 17)};
    initButtonBlock( eligPanel, eligLabel, "Eligibility conditions of external action networks",
       new Rectangle(4, 4, 280, 17), eligButtons, eligTexts, eligRects, options[ELIG] );
    // coordinations.
    initPanel( corPanel, BORDER, null, new Rectangle(380, 140, 350, 100) );
    String[] corExTexts = {"perform all", "perform one" };
    Rectangle[] corExRects = {new Rectangle(8, 28, 120, 17),new Rectangle(136, 28, 120, 17)};
    initButtonBlock( corPanel, corExLabel, "Coordinating multilple external action networks",
            new Rectangle(4, 6, 340, 17), corExButtons, corExTexts, corExRects, options[COR_EX]);
    String[] corExGoalLabelTexts = {"P_EXT =", "P_GS =" };
    Rectangle[] corExGoalLabelRects = {new Rectangle(12, 70, 48, 17),new Rectangle(136, 70, 48, 17)};
    String[] corExGoalFieldTexts = {
             String.valueOf(options[COR_EX_GS].getOptionItem(0).getParameter(0).getDoubleValue()),
             String.valueOf(options[COR_EX_GS].getOptionItem(0).getParameter(1).getDoubleValue()) };
    Rectangle[] corExGoalFieldRects = {new Rectangle(70, 70, 32, 17),new Rectangle(180, 70, 32, 17)};
    int[] fieldWidths = { 3, 3 };
    initFieldBlock( corPanel, corExGoalLabel, "Parameters for choosing between external and goal actions",
       new Rectangle(4, 50, 340, 17), corExGoalLabels, corExGoalLabelTexts, corExGoalLabelRects,
       corExGoalFields, corExGoalFieldTexts, corExGoalFieldRects, fieldWidths, options[COR_EX_GS]);
    // return button.
    initReturnButton( returnButton, "RETURN", new Rectangle(630, 260, 100, 28) );
    this.getContentPane().add(capLabel, null);
    this.getContentPane().add(rfPanel, null);
    this.getContentPane().add(rfAllPanel, null);
    this.getContentPane().add(eligPanel, null );
    this.getContentPane().add(corPanel, null );
    this.getContentPane().add(returnButton, null);
 }
}
