package clarion.options.acs.net;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of IDN component in one ACS network by GUI. It contains all
 * of the options relevant to IDN such as I/O dimension information, learning type
 * learning rate and momentum, temperature for action decision making and
 * reinforcement function etc.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUIc components in tis frame.
 * <li> public void update(): update GUI components status resulting from
 * upper level configuration.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of ACS network configuration and it is instantiated in
 * <tt>GuiAcsNet</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionAcsIdn extends SelectionTemplate {
 final int IN_DIM = 0, OUT_DIM = 1, UNITS = 2, LEARN = 3;
 final int L_RATE = 4, MOM = 5, Q_RATE = 6, TEMP = 7, REIN_EACH = 8, MCS = 9;
 //final int ACT_NUM = 10, ACT_REP = 11, RWD_NUM = 12, RWD_DIM = 13, RWD = 14;
 // from other levels
 final int COMB = 9, REIN = 1;
 int netType, netIdx;

 JLabel capLabel = new JLabel();
 // io
 JPanel ioPanel = new JPanel();
 JLabel ioLabel = new JLabel();
 JButton inputDimButton = new JButton();
 JButton outputDimButton = new JButton();
 JLabel hideUnitLabel = new JLabel();
 JTextField hideUnitField = new JTextField();
 // temperature
 boolean[] tempCompOns;
 JPanel tempPanel = new JPanel();
 JLabel blTempLabel = new JLabel();
 JLabel blTempLabel1 = new JLabel();
 JTextField blTempField = new JTextField();
 // learning option
 JPanel learningPanel = new JPanel();
 JLabel learningLabel = new JLabel();
 JRadioButton[] blLearnButtons = new JRadioButton[5]; {
    for( int i = 0; i < 5; i++ )
         blLearnButtons[i] = new JRadioButton();
 }
 // parameters for learning.
 JPanel paramPanel = new JPanel();
 JLabel paramLabel = new JLabel();
 JLabel lrLabel = new JLabel();
 JTextField lrField = new JTextField();
 JLabel momLabel = new JLabel();
 JTextField momField = new JTextField();
 JLabel qdLabel = new JLabel();
 JTextField qdField = new JTextField();
 // reinforcement function(each)
 boolean[] rfCompOns;
 JPanel rfPanel = new JPanel();
 JLabel reinLabel = new JLabel();
 JRadioButton[] rfButtons = new JRadioButton[2]; {
      rfButtons[0] = new JRadioButton();
      rfButtons[1] = new JRadioButton();
 }
 JButton[] rfEachUDButtons = new JButton[2]; {
     rfEachUDButtons[0] = new JButton();
     rfEachUDButtons[1] = new JButton();
 }
 // return button
 JButton returnButton = new JButton();
 OptionDims[] optDims = new OptionDims[2];
 // relative options for dependency
 Option reinOption;
 Option combOption;

 public SelectionAcsIdn( JFrame superFrame, int width, int height, int netType, int netIdx,
                         TransferManager manager )
 {
    super( superFrame, width, height, manager );
    this.netType = netType;
    this.netIdx = netIdx;
    initGui();
 }

 /** initialize the GUIc components in this frame. */
 public void initGui()
 {
    options = manager.getOptions( Global.ACS, Global.ACS_NET, netType, netIdx, Global.ACS_IDN, 0 );
    int CENTER = SwingConstants.CENTER;
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    setSize( width, height );
    this.getContentPane().setLayout(null);
    // caption
    initLabel( capLabel, "OPTIONS FOR EACH IDN", new java.awt.Font("Dialog", 1, 16),
               CENTER, new Rectangle(175, 0, 277, 26) );
    // io
    initPanel( ioPanel, BORDER, null, new Rectangle(10, 26, 350, 90) );
    initLabel( ioLabel, "I/O dimensions and values", new Rectangle(4, 4, 156, 17) );
    initButton( inputDimButton, "dimensions in input", new Rectangle(4, 28, 160, 24));
    initButton( outputDimButton, "dimensions in output", new Rectangle(170, 28, 170, 24));
    if( !options[IN_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_SYS).getBoolValue() ){
        optDims[0] = new OptionDims( this, 420, 780, "IDN INPUT SPECIFICATION", 30, manager,
                     options[IN_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).getIntValues()[0],
                     options[IN_DIM] );
        inputDimButton.setEnabled( true );
    }
    else
        inputDimButton.setEnabled( false );
    if( !options[OUT_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_SYS).getBoolValue() ){
        optDims[1] = new OptionDims( this, 420, 780, "IDN OUTPUT SPECIFICATION", 30, manager,
                     options[OUT_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).getIntValues()[0],
                     options[OUT_DIM] );
        outputDimButton.setEnabled( true );
    }
    else
        outputDimButton.setEnabled( false );
    initDimButton( this, inputDimButton, optDims[0] );
    initDimButton( this, outputDimButton, optDims[1] );
    initLabel( hideUnitLabel, "Number of hidden units", new Rectangle(4, 64, 134, 17));
/********* NEEDS BE UPDATE: three fields indicate input, hidden and ouput unit number ****************/
    initField( hideUnitField, String.valueOf( options[UNITS].getOptionItem(0).getParameter(1).getIntValue()),
               new Rectangle(144, 64, 32, 17), 4, options[UNITS], 0, 1 );
/*****************************************************************************************************/
    // temperature
    initPanel( tempPanel, BORDER, null, new Rectangle(10, 120, 350, 46 ) );
    initLabel( blTempLabel, "Boltzmann temperature for bottom-level action",
               new Rectangle(4, 4, 330, 17));
    initLabel( blTempLabel1, "selection", new Rectangle(4, 26, 60, 17));
    initField( blTempField, String.valueOf( options[TEMP].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(70, 26, 32, 17), 4, options[TEMP], 0, 0 );
    tempPanel.add(blTempLabel, null);
    tempPanel.add(blTempLabel1, null);
    tempPanel.add(blTempField, null);
    tempCompOns = new boolean[tempPanel.getComponentCount()];
    // Bottom level learning options
    initPanel( paramPanel, BORDER, null, new Rectangle(10, 170, 350, 130) );
    String[] blLearnTexts = {"Q-learning with backprogagation",
         "simplified Q-learning with backprogagation", "(supervised learning) backprogagation",
         "fixed", "the learning method to be determined by MCS on the fly" };
    Rectangle[] blLearnRects = { new Rectangle(4, 28, 300, 17), new Rectangle(4, 48, 300, 17),
                new Rectangle(4, 68, 300, 17),new Rectangle(4, 88, 200, 17),new Rectangle(4, 108, 340, 17)};
    initButtonBlock( paramPanel, paramLabel, "Bottom level learning options", new Rectangle(4, 8, 180, 17),
                     blLearnButtons, blLearnTexts, blLearnRects, options[LEARN] );
    // Parameters for BP and Q-learning
    initPanel( learningPanel, BORDER, null, new Rectangle(370, 26, 300, 90));
    initLabel( learningLabel,"Parameters for BP and Q-learning",new Rectangle(4, 8, 210, 17) );
    initLabel( lrLabel, "BP learning rate", new Rectangle(4, 28, 96, 17) );
    initField( lrField, String.valueOf( options[L_RATE].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(100, 28, 32, 17), 4, options[L_RATE], 0, 0 );
    initLabel( momLabel, "BP momentum", new Rectangle(4, 48, 96, 17));
    initField( momField, String.valueOf( options[MOM].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(100, 48, 32, 17), 4, options[MOM], 0, 0 );
    initLabel( qdLabel, "Q value discount rate", new Rectangle(4, 68, 128, 17));
    initField( qdField, String.valueOf( options[Q_RATE].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(130, 68, 32, 17), 4, options[Q_RATE], 0, 0 );
    // Reinforcement function for each network
    initPanel( rfPanel, BORDER, null, new Rectangle(370, 130, 300, 120) );
    Rectangle[] rfEachUDRects = { new Rectangle(24, 68, 120, 20), new Rectangle(24, 90, 120, 20) };
    String[] rfEachUDTexts = { TaskClarion.getSample( TaskClarion.ACS, TaskClarion.REIN ),
                               TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.REIN ) };
    initUserDefinedBlock( rfPanel, rfEachUDButtons, rfEachUDRects, rfEachUDTexts );
    String[] rfTexts = {"default (no reinforcement function)", "user defined"};
    Rectangle[] rfRects = {new Rectangle(4, 28, 240, 17),new Rectangle(4, 48, 100, 17)};
    initButtonBlock( rfPanel, reinLabel, "Reinforcement function for each network", new Rectangle(4, 4, 300, 17),
                     rfButtons, rfTexts, rfRects, options[REIN_EACH], 1, rfEachUDButtons );
    rfPanel.add( reinLabel, null );
    rfCompOns = new boolean[rfPanel.getComponentCount()];
    //reinOption = manager.getOptions( Global.ACS, Global.ACS_NET, Global.ACS_COR, 0, 0, 0, 1 );
    //this.initPanelStatus( reinOption, Global.REIN_EACH, rfPanel, rfCompOns );
    // return button.
    initReturnButton( returnButton, "RETURN", new Rectangle(560, 310, 100, 29) );
    ioPanel.add(ioLabel, null);
    ioPanel.add(inputDimButton, null);
    ioPanel.add(outputDimButton, null);
    ioPanel.add(hideUnitLabel, null);
    ioPanel.add(hideUnitField, null);
    learningPanel.add(learningLabel, null);
    learningPanel.add(lrLabel, null);
    learningPanel.add(lrField, null);
    learningPanel.add(momLabel, null);
    learningPanel.add(momField, null);
    learningPanel.add(qdLabel, null);
    learningPanel.add(qdField, null);
    paramPanel.add(paramLabel, null);
    this.getContentPane().add(capLabel, null);
    this.getContentPane().add(ioPanel, null);
    this.getContentPane().add(tempPanel, null);
    this.getContentPane().add(learningPanel, null);
    this.getContentPane().add(paramPanel, null);
    this.getContentPane().add(rfPanel, null);
    this.getContentPane().add(returnButton, null);
    // init panel status.
    combOption = manager.getOptions( Global.ACS, Global.ACS_NET, netType, netIdx, Global.ACS_BASIC, 0, COMB );
    reinOption = manager.getOptions( Global.ACS, Global.ACS_NET, Global.ACS_COR, 0, 0, 0, REIN );
    // temperature.
    initPanelStatus( combOption, Global.STOCH_COMB_MODE, tempPanel, tempCompOns );
    // reinforcement function
    initPanelStatus( reinOption, Global.REIN_EACH, rfPanel, rfCompOns );
  }

  /** update GUI components status resulting from upper level configuration. */
  public void update()
  {
    // temperature.
    if( combOption.getUserChoice() == Global.STOCH_COMB_MODE ){
        if( !tempPanel.isEnabled() ){
            tempPanel.setEnabled( true );
            for( int j = 0; j < tempCompOns.length; j++ )
                 tempPanel.getComponent(j).setEnabled( tempCompOns[j] );
        }
    }
    else{
        if( tempPanel.isEnabled() ){
            for( int j = 0; j < tempCompOns.length; j++ )
                 tempCompOns[ j ] = tempPanel.getComponent(j).isEnabled();
            tempPanel.setEnabled( false );
            for( int j = 0; j < tempPanel.getComponentCount(); j++ )
                 tempPanel.getComponent(j).setEnabled( false );
        }
    }
    // reinforcement function
    if( reinOption.getUserChoice() == Global.REIN_EACH ){
        if( !rfPanel.isEnabled() ){
            rfPanel.setEnabled( true );
            for( int j = 0; j < rfCompOns.length; j++ )
                 rfPanel.getComponent(j).setEnabled( rfCompOns[j] );
        }
    }
    else{
        if( rfPanel.isEnabled() ){
            for( int j = 0; j < rfCompOns.length; j++ )
                 rfCompOns[ j ] = rfPanel.getComponent(j).isEnabled();
            rfPanel.setEnabled( false );
            for( int j = 0; j < rfPanel.getComponentCount(); j++ )
                 rfPanel.getComponent(j).setEnabled( false );
        }
    }
  }
}
