package clarion.options.acs.net;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the base class
 * for configuration of ACS Rule Type components by GUI such as RER, IRL or FR.
 * It contains all of the opitons those are common for all ACS rule type such as
 * I/O dimension information, decision temperature, partial match, BLA, rule
 * utility etc.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * <li> public void update(): update GUI components status and the relevant options
 *  resulting from upper level.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * The concrete class for implementing the configuration of a concrete ACS rule type
 * component by GUI will extends this class.
 * </ul>
 * @author Xi Zhang
 *
 */


public class SelectionAcsRule extends SelectionTemplate{
 final int IN_DIM = 0, OUT_DIM = 1, TEMP = 2;
 final int REASON = 3, UTL_ON = 4, PM_ON = 5, RBLA_ON = 6, CBLA_ON = 7;
 final int WEIGHT_C = 8, WEIGHT_R = 9, RBLA = 10, CBLA = 11, UTL = 12;
 // from other levels
 final int COMB = 9;
 final int ON = 0, OFF = 1;
 // some GUI constants.
 final int CENTER = SwingConstants.CENTER;
 final Border BORDER = BorderFactory.createLineBorder(Color.black);
 final Color COLOR = SystemColor.activeCaption;
 final Font capFont = new java.awt.Font("Dialog", 1, 20);
 final String[] capTexts = { "", "RER  OPTIONS", "IRL  OPTIONS ( the number of current rule subset = ",
                             "FIXED RULES OPTIONS ( the number of current rule = "  };
 final String[] onOffTexts = { "ON", "OFF" };
 final int IO = 0, TP = 1, RS = 2, REP = 3, WGT_C = 4, WGT_R = 5, CB = 6, RB = 7, UT = 8, RTN = 9;
 final Rectangle[][] compLocs = {
       {
          new Rectangle(10, 30, 320, 70), new Rectangle(10, 110, 320, 40),
          new Rectangle(10, 160, 320, 140), new Rectangle(10, 310, 320, 130),
          new Rectangle(10, 450, 320, 130), new Rectangle(10, 590, 320, 130),
          new Rectangle(340, 30, 310, 176),
          new Rectangle(340, 208, 310, 176), new Rectangle(340, 386, 310, 334),
          new Rectangle(880, 735, 110, 29)
       },
       {
          new Rectangle(4, 24, 320, 70), new Rectangle(4, 98, 320, 40 ),
          new Rectangle(4, 142, 320, 110), new Rectangle(4, 256, 320, 120),
          new Rectangle(4, 380, 320, 120),
          new Rectangle(328, 24, 312, 120), new Rectangle(328, 150, 312, 174),
          new Rectangle(328, 326, 312, 174), new Rectangle(644, 24, 312, 318),
          new Rectangle(846, 520, 110, 29)
       }
       };
 int netType, netIdx, ruleType, setIdx, style;
 // GUI components
 Container rootContainer;
 JLabel capLabel = new JLabel();
 // general
 JPanel genPanel = new JPanel();
 JLabel ioLabel = new JLabel();
 JButton inputDimButton = new JButton();
 JButton outputDimButton = new JButton();
 OptionDims[] optDims = new OptionDims[2];
 // reason.
 JPanel reasonPanel = new JPanel();
 JLabel reasonLabel = new JLabel();
 JLabel thpmLabel = new JLabel();
 JTextField thpmField = new JTextField();
 JRadioButton[] reasonButtons = new JRadioButton[2];{
    for( int i = 0; i < 2; i++ )
         reasonButtons[i] = new JRadioButton();
 }
 // temp
 boolean[] tempCompOns;
 JPanel tempPanel = new JPanel();
 JLabel tlTempLabel = new JLabel();
 JTextField tlTempField = new JTextField();
 // rep
 JPanel repPanel = new JPanel();
 JLabel repLabel = new JLabel();
 JLabel repUtlLabel = new JLabel();
 JRadioButton[] repUtlButtons = new JRadioButton[2];{
   repUtlButtons[0] = new JRadioButton();
   repUtlButtons[1] = new JRadioButton();
 }
 JLabel repPmLabel = new JLabel();
 JRadioButton[] repPmButtons = new JRadioButton[2];{
   repPmButtons[0] = new JRadioButton();
   repPmButtons[1] = new JRadioButton();
 }
 JLabel repRblaLabel = new JLabel();
 JRadioButton[] repRblaButtons = new JRadioButton[2];{
   repRblaButtons[0] = new JRadioButton();
   repRblaButtons[1] = new JRadioButton();
 }
 JLabel repCblaLabel = new JLabel();
 JRadioButton[] repCblaButtons = new JRadioButton[2];{
   repCblaButtons[0] = new JRadioButton();
   repCblaButtons[1] = new JRadioButton();
 }
 // weights of a chunk representing an action rule condition.
 boolean[][] wgtCompOns = new boolean[2][];
 JPanel[] wgtPanels = new JPanel[2];{
    wgtPanels[0] = new JPanel();
    wgtPanels[1] = new JPanel();
 }
 JLabel cwgtLabel = new JLabel();
 JRadioButton[] cwgtButtons = new JRadioButton[2];{
   cwgtButtons[0] = new JRadioButton();
   cwgtButtons[1] = new JRadioButton();
 }
 JButton[] cwgtUDButtons = new JButton[2]; {
   cwgtUDButtons[0] = new JButton();
   cwgtUDButtons[1] = new JButton();
 }
 // weights of an action rule.
 JLabel rwgtLabel = new JLabel();
 JRadioButton[] rwgtButtons = new JRadioButton[2];{
   rwgtButtons[0] = new JRadioButton();
   rwgtButtons[1] = new JRadioButton();
 }
 JButton[] rwgtUDButtons = new JButton[2]; {
   rwgtUDButtons[0] = new JButton();
   rwgtUDButtons[1] = new JButton();
 }
 // rule bla
 boolean[] rblaCompOns;
 JPanel rblaPanel = new JPanel();
 // chunk bla.
 boolean[] cblaCompOns;
 JPanel cblaPanel = new JPanel();
 // utility
 boolean[] utlCompOns;
 JPanel utlPanel = new JPanel();
 // return buttons.
 JButton returnButton = new JButton();
 // relative options for dependency
 Option combOption;

 public SelectionAcsRule( JFrame superFrame, int width, int height, int netType, int netIdx,
                          int ruleType, int setIdx, TransferManager manager )
 {
    super( superFrame, width, height, manager );
    this.netType = netType;
    this.netIdx = netIdx;
    this.ruleType = ruleType;
    this.setIdx = setIdx;
    initGui();
 }

 /** initialize the GUI components. */
 public void initGui()
 {
    //get options.
    options = manager.getOptions( Global.ACS, Global.ACS_NET, netType, netIdx, ruleType, setIdx );
    // set size.
    if( parent != null ){
        setSize( 960, 720 );
        rootContainer = parent;
    }
    else{
        setSize( width, height );
        rootContainer = this.getContentPane();
        rootContainer.setLayout(null);
    }
    // set style.
    style = ( ruleType != Global.FR )?0 : 1;
    // caption.
    String capText = capTexts[ruleType];
    if( ruleType != Global.RER )
        capText += setIdx + " )";
    initLabel( capLabel, capText, capFont, CENTER, new Rectangle( (width-600)/2, 0, 600, 26) );
    // io
    initPanel( genPanel, BORDER, null, compLocs[style][IO] );
    initLabel( ioLabel, "I/O dimensions and values", new Rectangle(8, 8, 160, 17) );
    initButton( inputDimButton, "input dimensions", new Rectangle(16, 24, 132, 24));
    initButton( outputDimButton, "output dimensions", new Rectangle(158, 24, 152, 24));
    if( !options[IN_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_SYS).getBoolValue() ){
        optDims[0] = new OptionDims( this, 420, 780, "ACS RULE-TYPE INPUT SPECIFICATION", 30, manager,
                     options[IN_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).getIntValues()[0],
                     options[IN_DIM] );
        inputDimButton.setEnabled( true );
    }
    else
        inputDimButton.setEnabled( false );
    if( !options[OUT_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_SYS).getBoolValue() ){
        optDims[1] = new OptionDims( this, 420, 780, "ACS RULE-TYPE OUTPUT SPECIFICATION", 30, manager,
                     options[OUT_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).getIntValues()[0],
                     options[OUT_DIM] );
        outputDimButton.setEnabled( true );
    }
    else
        outputDimButton.setEnabled( false );
    initDimButton( this, inputDimButton, optDims[0] );
    initDimButton( this, outputDimButton, optDims[1] );
    // temperature.
    initPanel( tempPanel, BORDER, null, compLocs[style][TP] );
    initLabel( tlTempLabel, "temperature for top-level action selection",new Rectangle(6,10,240,17));
    initField( tlTempField, String.valueOf(options[TEMP].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(250, 10, 32, 17), 5, options[TEMP], 0, 0 );
    tempPanel.add(tlTempLabel, null);
    tempPanel.add(tlTempField, null);
    tempCompOns = new boolean[tempPanel.getComponentCount()];
    // reasoning.
    initPanel( reasonPanel, BORDER, null, compLocs[style][RS] );
    String[] reasonTexts = { "forward chaining", "forward chaining with partial match" };
    Rectangle[] reasonRects = { new Rectangle(4, 30, 126, 17), new Rectangle(4, 54, 250, 17) };
    initLabel( thpmLabel, "partial match threshold", new Rectangle(20, 74, 160, 17));
    initField( thpmField, String.valueOf(options[REASON].getOptionItem(1).getParameter(0).getDoubleValue()),
               new Rectangle(180, 74, 30, 17), 5, options[REASON], 1, 0 );
    initButtonBlock( reasonPanel,reasonLabel, "Options for using action rules in action decision making",
      new Rectangle(4, 8, 314, 17), reasonButtons, reasonTexts, reasonRects, options[REASON], 1, thpmField );
    // weights of a chunk representing an action rule condition.
    initPanel(wgtPanels[0], BORDER, null, compLocs[style][WGT_C]);
    String[] cwgtUDTexts = getUDTexts( 0 );
    Rectangle[] cwgtUDRects = { new Rectangle(24, 72, 160, 20), new Rectangle(24, 96, 160, 20) };
    initUserDefinedBlock( wgtPanels[0], cwgtUDButtons, cwgtUDRects, cwgtUDTexts );
    String[] cwgtTexts = { "default (all weights are equal)", "user defined" };
    Rectangle[] cwgtRects = { new Rectangle(8, 30, 200, 17), new Rectangle(8, 52, 120, 17)};
    initButtonBlock( wgtPanels[0], cwgtLabel, "Weights of action rule condition", new Rectangle(4, 4, 200, 17),
                     cwgtButtons, cwgtTexts, cwgtRects, options[WEIGHT_C], 1, cwgtUDButtons );
    wgtCompOns[0] = new boolean[wgtPanels[0].getComponentCount()];
    super.initPanelStatus( options[PM_ON], ON, wgtPanels[0], wgtCompOns[0] );
    // weights of an action rule.
    initPanel(wgtPanels[1], BORDER, null, compLocs[style][WGT_R]);
    String[] rwgtUDTexts = getUDTexts( 1 );
    Rectangle[] rwgtUDRects = { new Rectangle(24, 72, 160, 20), new Rectangle(24, 96, 160, 20) };
    initUserDefinedBlock( wgtPanels[1], rwgtUDButtons, rwgtUDRects, rwgtUDTexts );
    String[] rwgtTexts = { "default ( weight is 1)", "user defined" };
    Rectangle[] rwgtRects = { new Rectangle(8, 30, 200, 17), new Rectangle(8, 56, 120, 17)};
    initButtonBlock( wgtPanels[1], rwgtLabel, "Weight of action rule", new Rectangle(4, 4, 200, 17),
                     rwgtButtons, rwgtTexts, rwgtRects, options[WEIGHT_R], 1, rwgtUDButtons );
    wgtCompOns[1] = new boolean[wgtPanels[1].getComponentCount()];
    super.initPanelStatus( options[PM_ON], ON, wgtPanels[1], wgtCompOns[1] );
    // chunk bla
    initPanel( cblaPanel, BORDER, null, compLocs[style][CB] );
    initBlaBlock( cblaPanel, "Chunk base-level activation", options[CBLA] );
    cblaCompOns = new boolean[cblaPanel.getComponentCount()];
    super.initPanelStatus( options[CBLA_ON], ON, cblaPanel, cblaCompOns );
    // rule bla
    initPanel( rblaPanel, BORDER, null, compLocs[style][RB] );
    initBlaBlock( rblaPanel, "Rule base-level activation", options[RBLA] );
    rblaCompOns = new boolean[rblaPanel.getComponentCount()];
    super.initPanelStatus( options[RBLA_ON], ON, rblaPanel, rblaCompOns );
    // rule utility
    initPanel( utlPanel, BORDER, null, compLocs[style][UT] );
    String[] utlUDTexts = getUDTexts( 2 );
    initUtilityBlock( utlPanel, "Rule utility options", options[UTL], utlUDTexts );
    utlCompOns = new boolean[utlPanel.getComponentCount()];
    super.initPanelStatus( options[UTL_ON], ON, utlPanel, utlCompOns );
    // representation.
    initPanel( repPanel, BORDER, null, compLocs[style][REP] );
    initLabel( repLabel, "Options for action rule representations", new Rectangle(8, 8, 223, 17));
    Rectangle[] repUtlRects = {new Rectangle(84, 30, 44, 17), new Rectangle(147, 30, 52, 17)};
    initButtonBlock( repPanel, repUtlLabel, "rule utility", new Rectangle(12, 30, 68, 17),
                     repUtlButtons, onOffTexts, repUtlRects, options[UTL_ON], ON, utlPanel, utlCompOns );
    Rectangle[] repPmRects = {new Rectangle(196, 52, 44, 17), new Rectangle(244, 52, 52, 17)};
    initButtonBlock( repPanel, repPmLabel, "partial matching in rule support", new Rectangle(12, 52, 180, 17),
                     repPmButtons, onOffTexts, repPmRects, options[PM_ON], ON, wgtPanels, wgtCompOns );
    Rectangle[] repRblaRects = {new Rectangle(160, 74, 44, 17),new Rectangle(208, 74, 52, 17)};
    initButtonBlock( repPanel, repRblaLabel, "rule base-level activation", new Rectangle(12, 74, 144, 17),
                     repRblaButtons, onOffTexts, repRblaRects, options[RBLA_ON], ON, rblaPanel, rblaCompOns );
    Rectangle[] repCblaRects = {new Rectangle(210, 96, 44, 17),new Rectangle(258, 96, 52, 17)};
    initButtonBlock( repPanel, repCblaLabel, "action chunk base-level activation", new Rectangle(12, 96, 194, 17),
                     repCblaButtons, onOffTexts, repCblaRects, options[CBLA_ON], ON, cblaPanel, cblaCompOns );
    // return button.
    initReturnButton( returnButton, "RETURN", compLocs[style][RTN] );
    // init panel status.
    combOption = manager.getOptions( Global.ACS, Global.ACS_NET, netType, netIdx, Global.ACS_BASIC, 0, COMB );
    initPanelStatus( combOption, Global.STOCH_COMB_MODE, tempPanel, tempCompOns );
    // add components
    genPanel.add(ioLabel, null);
    genPanel.add(inputDimButton, null);
    genPanel.add(outputDimButton, null);
    reasonPanel.add( thpmLabel, null );
    reasonPanel.add( thpmField, null );
    repPanel.add(repLabel, null);
    rootContainer.add(capLabel, null);
    rootContainer.add(genPanel, null);
    rootContainer.add(tempPanel, null);
    rootContainer.add(reasonPanel, null);
    rootContainer.add(repPanel, null);
    rootContainer.add(wgtPanels[0], null);
    rootContainer.add(wgtPanels[1], null);
    rootContainer.add(rblaPanel, null);
    rootContainer.add(utlPanel, null);
    rootContainer.add(cblaPanel, null);
    rootContainer.add(returnButton, null);
 }

 /** update resulting from upper level. */
 public void update()
 {
    // temperature.
    if( combOption.getUserChoice() == Global.STOCH_COMB_MODE ){
        if( tempPanel.isEnabled() )
            return;
        tempPanel.setEnabled( true );
        for( int j = 0; j < tempCompOns.length; j++ )
             tempPanel.getComponent(j).setEnabled( tempCompOns[j] );
    }
    else{
        if( !tempPanel.isEnabled() )
            return;
        for( int j = 0; j < tempCompOns.length; j++ )
             tempCompOns[ j ] = tempPanel.getComponent(j).isEnabled();
        tempPanel.setEnabled( false );
        for( int j = 0; j < tempPanel.getComponentCount(); j++ )
             tempPanel.getComponent(j).setEnabled( false );
    }
 }

 private String[] getUDTexts( int which )
 {
    String[] texts = new String[2];
    switch( which ){
       case 0:
         switch( ruleType ){
           case Global.RER :
                texts[0] = TaskClarion.getSample( TaskClarion.ACS, TaskClarion.RER_WGT_C );
                texts[1] = TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.RER_WGT_C );
                break;
           case Global.IRL :
                texts[0] = TaskClarion.getSample( TaskClarion.ACS, TaskClarion.IRL_WGT_C );
                texts[1] = TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.IRL_WGT_C );
                break;
           case Global.FR :
                texts[0] = TaskClarion.getSample( TaskClarion.ACS, TaskClarion.FR_WGT_C );
                texts[1] = TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.FR_WGT_C );
                break;
         }
         break;
       case 1:
         switch( ruleType ){
           case Global.RER :
                texts[0] = TaskClarion.getSample( TaskClarion.ACS, TaskClarion.RER_WGT_R );
                texts[1] = TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.RER_WGT_R );
                break;
           case Global.IRL :
                texts[0] = TaskClarion.getSample( TaskClarion.ACS, TaskClarion.IRL_WGT_R );
                texts[1] = TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.IRL_WGT_R );
                break;
           case Global.FR :
                texts[0] = TaskClarion.getSample( TaskClarion.ACS, TaskClarion.FR_WGT_R );
                texts[1] = TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.FR_WGT_R );
                break;
         }
         break;
       case 2:
         switch( ruleType ){
           case Global.RER :
                texts[0] = TaskClarion.getSample( TaskClarion.ACS, TaskClarion.RER_UTL );
                texts[1] = TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.RER_UTL );
                break;
           case Global.IRL :
                texts[0] = TaskClarion.getSample( TaskClarion.ACS, TaskClarion.IRL_UTL );
                texts[1] = TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.IRL_UTL );
                break;
           case Global.FR :
                texts[0] = TaskClarion.getSample( TaskClarion.ACS, TaskClarion.FR_UTL );
                texts[1] = TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.FR_UTL );
                break;
         }
         break;
    }
    return texts;
 }
}
