package clarion.options.dims;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to one type of Dimension (INPUT, OUTPUT,
 * GOAL) by GUI. It contains the options such as the number of dimensions,
 * the names of the dimensions and number of values in each dimension.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of Dimension configuration and it is instantiated in
 * <tt>GuiDims</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionDims extends SelectionTemplate{

  int dimNum, maxDimNum, featType;
  String capName;
  JLabel capLabel = new JLabel();
  JPanel panel = new JPanel();
  JLabel dimNumLabel = new JLabel();
  JTextField dimNumField = new JTextField();
  JLabel noteLabel1 = new JLabel();
  JLabel noteLabel2 = new JLabel();
  JLabel noteLabel3 = new JLabel();
  // copy option
  JLabel copyOptLabel = new JLabel();
  JLabel copyOptLabel1 = new JLabel();
  JLabel copySoucLabel = new JLabel();
  JLabel copyDestLabel = new JLabel();
  JLabel copyFromLabel = new JLabel();
  JLabel copyToLabel = new JLabel();
  JLabel copyDestStartLabel = new JLabel();
  JLabel copyPostfixLabel = new JLabel();
  JTextField copyFromField = new JTextField();
  JTextField copyToField = new JTextField();
  JTextField copyDestStartField = new JTextField();
  JTextField copyPostfixField = new JTextField();
  JButton copyButton = new JButton();
  // dimension information.
  JLabel dimLabel = new JLabel();
  JLabel dimNameLabel = new JLabel();
  JLabel dimValNumLabel = new JLabel();
  JLabel[] dimLabels;
  JTextField[] dimNameFields;
  JTextField[] dimValNumFields;
  JButton returnButton = new JButton();

  public SelectionDims( JFrame superFrame, int width, int height, String name,
         int maxDimNum, int featType, TransferManager manager )
  {
    super( superFrame, width, height, manager );
    capName = name;
    this.maxDimNum = maxDimNum;
    this.featType = featType;
    dimLabels = new JLabel[maxDimNum];
    dimNameFields = new JTextField[maxDimNum];
    dimValNumFields = new JTextField[maxDimNum];
    for( int i = 0; i < maxDimNum; i++ ){
         dimLabels[i] = new JLabel();
         dimNameFields[i] = new JTextField();
         dimValNumFields[i] = new JTextField();
    }
    initGui();
  }

  /* initialize the GUI components in this frame. */
  public void initGui()
  {
    final int FEAT_NUMS = 0, FEAT_NAMES = 1, FEAT_VAL_NUMS_START = 2, FEAT_VAL_NAMES_START = 5;

    options = manager.getOptions( Global.DIMS, 0, 0, 0, 0, 0 );
    setSize( 500, 720 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    // caption.
    initLabel( capLabel, capName, new Font("Dialog",1,14), SwingConstants.CENTER,
               new Rectangle((width-300)/2, 2, 300, 25) );
    // panel.
    initPanel( panel, BORDER, null, new Rectangle(10, 30, width-30, height-85) );


    //System.out.println(" the featType = " + featType );


    dimNum = options[FEAT_NUMS].getOptionItem(0).getParameter(featType).getIntValue();
    initLabel( dimNumLabel, "The number of dimension", new Rectangle(10, 4, 170, 17));
    initField( dimNumField, String.valueOf( dimNum ), new Rectangle(180, 4, 32, 17) );
    initLabel( noteLabel1, "Note: the following dimensions specified exactly as provided by external",
               new Rectangle(10, 30, 450, 17));
    initLabel( noteLabel2, "sources. Name each dimension uniquely and specify the value # of each",
               new Rectangle(10, 50, 450, 17));
    initLabel( noteLabel3, "dimension", new Rectangle(10, 70, 100, 17));
    // copy options.
    initLabel( copyOptLabel, "Dimension information copy option", new Rectangle(10, 100, 300, 17) );
    initLabel( copyOptLabel1, "(dimension index: 0 to current number of dimension-1)",
               new Rectangle(10, 120, 400, 17) );
    initLabel( copySoucLabel, "source dimensions : ", new Rectangle(10, 140, 130, 17) );
    initLabel( copyFromLabel, " from ", new Rectangle(140, 140, 40, 17) );
    initField( copyFromField, "0", new Rectangle(180, 140, 40, 17), 4 );
    initLabel( copyToLabel, " to ", new Rectangle(260, 140, 20, 17) );
    initField( copyToField, "0", new Rectangle(280, 140, 40, 17), 4 );
    initLabel( copyDestLabel, "destination location : ", new Rectangle(10, 160, 130, 17) );
    initLabel( copyDestStartLabel, " start ", new Rectangle(140, 160, 40, 17) );
    initField( copyDestStartField, "1", new Rectangle(180, 160, 40, 17), 4 );
    initLabel( copyPostfixLabel, "attached postfix : ", new Rectangle(10, 180, 130, 17) );
    initField( copyPostfixField, "1", new Rectangle(140, 180, 40, 17), 4 );
    initButton( copyButton, "copy dimension information", new Rectangle(10, 200, 300, 20) );
    copyButton.addActionListener( new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) {
            if( JOptionPane.showConfirmDialog( null, "Do you want to copy dimension information?",
                    "WARNING", JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION ) {
                int fromIdx = Integer.parseInt( copyFromField.getText().trim() );
                if( fromIdx < 0 || fromIdx > dimNum-1 ){
                    JOptionPane.showMessageDialog( null, "out of range in 'from' field, reset !",
                                                   "Error", JOptionPane.ERROR_MESSAGE );
                    copyFromField.setText( String.valueOf( 0 ) );
                    return;
                }
                int toIdx = Integer.parseInt( copyToField.getText().trim() );
                if( toIdx < 0 || toIdx > dimNum-1 ){
                    JOptionPane.showMessageDialog( null, "out of range in 'to' field, reset !",
                                                   "Error", JOptionPane.ERROR_MESSAGE );
                    copyToField.setText( String.valueOf( 0 ) );
                    return;
                }
                else if( toIdx < fromIdx ){
                    JOptionPane.showMessageDialog( null, "'from' should be no larger than 'to', reset !",
                                                   "Error", JOptionPane.ERROR_MESSAGE );
                    copyToField.setText( String.valueOf( fromIdx ) );
                    return;
                }
                int startIdx = Integer.parseInt( copyDestStartField.getText().trim() );
                if( startIdx < 0 || startIdx > dimNum-1 ){
                    JOptionPane.showMessageDialog( null, "out of range in 'start' field, reset !",
                                                   "Error", JOptionPane.ERROR_MESSAGE );
                    copyDestStartField.setText( String.valueOf( 1 ) );
                    return;
                }
                int copyLen = toIdx - fromIdx + 1;
                if( startIdx + copyLen > dimNum ){
                    if( JOptionPane.showConfirmDialog( null, "This copy process will overflow,truncate and"
                        + " continue?", "WARNING", JOptionPane.YES_NO_OPTION ) == JOptionPane.NO_OPTION )
                        return;
                    copyLen = dimNum - startIdx;
                }
                for( int i = startIdx + copyLen - 1; i >= startIdx; i-- ){
                     int idx = toIdx + i-(startIdx+copyLen-1);
                     String souName = dimNameFields[idx].getText();
                     String postfix = copyPostfixField.getText();
                     dimNameFields[i].setText( souName+postfix );
                     String name = dimNameFields[i].getText();
                     options[FEAT_NAMES].getOptionItem(0).getParameter(featType).setStringValue(i,name);

                     int num = Integer.parseInt( dimValNumFields[idx].getText() );
                     dimValNumFields[i].setText( String.valueOf(num) );
                     int offset = FEAT_VAL_NUMS_START + featType;
                     int nameOffset = FEAT_VAL_NAMES_START + featType;
                     num = Integer.parseInt( dimValNumFields[i].getText() );
                     if( num < 1 ){
                         JOptionPane.showMessageDialog( null, "out of range, reset !",
                                                        "Error", JOptionPane.ERROR_MESSAGE );
                         dimValNumFields[i].setText( String.valueOf(
                           options[offset].getOptionItem(0).getParameter(0).getIntValues()[i]));
                         return;
                     }
                     options[offset].getOptionItem(0).getParameter(0).setIntValue(i,num);
                     options[nameOffset].getOptionItem(0).getParameter(i).setCount( num );
                 }
            }
         }
    });
    // dimensions information.
    final int START = 240;
    initLabel( dimLabel, "dimensions", new Rectangle(10, START, 150, 17));
    initLabel( dimNameLabel, "name", new Rectangle(140, START, 100, 17));
    initLabel( dimValNumLabel, "value number", new Rectangle(300, START, 80, 17));
    dimNumField.addActionListener( new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                  try{
                    int num = Integer.parseInt( dimNumField.getText().trim() );
                    if( num < 1 || num > maxDimNum ){
                        JOptionPane.showMessageDialog( null, "out of range, reset !",
                                                       "Error", JOptionPane.ERROR_MESSAGE );
                        dimNumField.setText( String.valueOf( dimNum ) );
                        return;
                    }
                    else
                        dimNum = num;
                  }
                  catch( NumberFormatException e1 ){
                     JOptionPane.showMessageDialog( null, "Not a number, reset !",
                                                    "Error", JOptionPane.ERROR_MESSAGE );
                     dimNumField.setText( String.valueOf( dimNum ) );
                     return;
                  }
                  dimNumField.setText( String.valueOf( dimNum ) );
                  options[FEAT_NUMS].getOptionItem(0).getParameter(featType).setIntValue(dimNum);
                  options[FEAT_NAMES].getOptionItem(0).getParameter(featType).setCount(dimNum);
                  options[FEAT_VAL_NUMS_START+featType].getOptionItem(0).getParameter(0).setCount(dimNum);
                  options[FEAT_VAL_NAMES_START+featType].getOptionItem(0).setCount(dimNum);
                  for( int i = 0; i < maxDimNum; i++ ){
                       dimLabels[i].setEnabled( i < dimNum );
                       dimNameFields[i].setEnabled( i < dimNum );
                       dimValNumFields[i].setEnabled( i < dimNum );
                  }
            }
         });
    for( int i = 0; i < maxDimNum; i++ ){
         initLabel( dimLabels[i], "dimension" + i, new Rectangle(10, START + 20+20*i, 100, 17) );
         String name = "dim " + i;
         if ( i < dimNum )
            name = options[FEAT_NAMES].getOptionItem(0).getParameter(featType).getStringValues()[i];
         initField( dimNameFields[i], name, new Rectangle(140, START + 20+20*i, 100, 17) );
         int offset = FEAT_VAL_NUMS_START + featType;
         int dimValNum = 1;
         if( i < dimNum )
             dimValNum = options[offset].getOptionItem(0).getParameter(0).getIntValues()[i];
         initField( dimValNumFields[i], String.valueOf( dimValNum ), new Rectangle(300, START + 20+20*i, 24, 17) );
         dimNameFields[i].setEnabled( i < dimNum );
         dimValNumFields[i].setEnabled( i < dimNum );
         final int focus = i;
         dimNameFields[i].addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent e) {
                 String name = dimNameFields[focus].getText();
                 options[FEAT_NAMES].getOptionItem(0).getParameter(featType).setStringValue(focus,name);
             }
         });
         dimValNumFields[i].addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent e) {
               int offset = FEAT_VAL_NUMS_START + featType;
               int nameOffset = FEAT_VAL_NAMES_START + featType;
               try{
                 int num = Integer.parseInt( dimValNumFields[focus].getText() );
                 if( num < 1 ){
                     JOptionPane.showMessageDialog( null, "out of range, reset !",
                                                    "Error", JOptionPane.ERROR_MESSAGE );
                     dimValNumFields[focus].setText( String.valueOf(
                       options[offset].getOptionItem(0).getParameter(0).getIntValues()[focus]));
                     return;
                 }
                 options[offset].getOptionItem(0).getParameter(0).setIntValue(focus,num);
                 options[nameOffset].getOptionItem(0).getParameter(focus).setCount( num );
               }
               catch( NumberFormatException e1 ){
                   JOptionPane.showMessageDialog( null, "Not a number, reset !",
                                                  "Error", JOptionPane.ERROR_MESSAGE );
                   dimValNumFields[focus].setText( String.valueOf(
                      options[offset].getOptionItem(0).getParameter(0).getIntValues()[focus]));
                   return;
               }
             }
         });
    }
    // return button.
    initReturnButton( returnButton, "RETURN", new Rectangle(width-120, height-40, 100, 20) );
    // add components.
    panel.add( dimNumLabel );
    panel.add( dimNumField );
    for( int i = 0; i < maxDimNum; i++ ){
         panel.add( dimLabels[i] );
         panel.add( dimNameFields[i] );
         panel.add( dimValNumFields[i] );
    }
    panel.add( noteLabel1 );
    panel.add( noteLabel2 );
    panel.add( noteLabel3 );
    panel.add(noteLabel1);
    panel.add(noteLabel2);
    panel.add(copyOptLabel);
    panel.add(copyOptLabel1);
    panel.add(copySoucLabel);
    panel.add(copyDestLabel);
    panel.add(copyFromLabel);
    panel.add(copyToLabel);
    panel.add(copyDestStartLabel);
    panel.add(copyPostfixLabel);
    panel.add(copyFromField);
    panel.add(copyToField);
    panel.add(copyDestStartField);
    panel.add(copyPostfixField);
    panel.add(copyButton);
    panel.add( dimLabel );
    panel.add( dimNameLabel );
    panel.add( dimValNumLabel );
    parent.add(capLabel, null);
    parent.add(panel, null);
    parent.add(returnButton, null);
  }
}
