package clarion.options.nacs;
import java.awt.event.*;
import javax.swing.*;
import clarion.options.*;
import clarion.options.nacs.amn.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to show the top-level screen of configuring NACS subsystem by GUI.
 * It extends the base class <em>GuiTemplate</em>.  It contains the following
 * configuration entries: NACS components selection, GKS, AMNs, EM and AEM.
 *
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public void returnFrom(): update the GUI components when return from other
 * screens.
 * <li> public void addActionListeners( final JFrame mainFrame ): event listeners
 * for the buttons.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to control the process of configuring the NACS subsystem by GUI
 * (on-line configuration).
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>GuiClarion</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class GuiNACS extends GuiTemplate{

  String buttonTexts[] =
  {
     "Options for NACS components",
     "Options for GKS",
     "Options for AMN",
     "Options for EM",
     "Options for AEM",
     //"Options for Coordinating the ACS and NACS",
  };
  SelectionTemplate subSelections[];
  GuiTemplate amns;
  // get basic CLARION info
  int amnNum;

  public GuiNACS( int num )
  {
     super( num, "OPTIONS OF THE NON-ACTION-CENTERED SYSTEM" );
     setButtonTexts( buttonTexts );
     // get basic CLARION info
     amnNum = manager.getAmnNum();
     subSelections = new SelectionTemplate[ 5 ];
     subSelections[0] = new SelectionNacsComp( this, 400, 280, manager );
     subSelections[1] = new SelectionGks( this, 960, 640, manager );
     subSelections[2] = new SelectionEM( this, 350, 650, manager );
     subSelections[3] = new SelectionAem( this, 410, 540, manager );
     //subSelections[4] = new SelectionCor( this, 380, 440, manager );
     amns = new GuiAmn( amnNum );
     amns.init( this );
     returnFrom();
  }

  /** update the GUI components when return from other screens. */
  public void returnFrom()
  {
     if( manager.isGksOn() ){
         optButtons[1].setEnabled( true );
         if( subSelections[1] == null )
             subSelections[1] = new SelectionGks( this, 960, 640, manager );
         ((SelectionGks)subSelections[1]).update();
     }
     else
         optButtons[1].setEnabled( false );
     if( manager.isEmOn() ){
         optButtons[3].setEnabled( true );
         if( subSelections[2] == null )
             subSelections[2] = new SelectionAem( this, 360, 630, manager );
         ((SelectionEM)subSelections[2]).update();
     }
     else
         optButtons[3].setEnabled( false );
     if( manager.isAemOn() ){
         optButtons[4].setEnabled( true );
         if( subSelections[3] == null )
             subSelections[3] = new SelectionAem( this, 360, 420, manager );
         ((SelectionAem)subSelections[3]).update();
     }
     else
         optButtons[4].setEnabled( false );
     ((GuiAmn)amns).updateGui();
  }

  /** event listeners for the buttons. */
  public void addActionListeners( final JFrame mainFrame )
  {
     for( int i = 0; i < 4; i++ ){
          final int idx = i;
          if( idx < 2 )
              optButtons[ idx ].addActionListener(  new  ActionListener(){
                                public void actionPerformed( ActionEvent e ){
                                  activeButtonIdx = idx;
                                  subSelections[idx].setEnabled( true );
                                  subSelections[idx].setVisible( true );
                               }
                       } );
          else
              optButtons[ idx + 1 ].addActionListener(  new  ActionListener(){
                                public void actionPerformed( ActionEvent e ){
                                  activeButtonIdx = idx;
                                  subSelections[idx].setEnabled( true );
                                  subSelections[idx].setVisible( true );
                               }
                       } );
     }
     optButtons[ 2 ].addActionListener(  new  ActionListener(){
                     public void actionPerformed( ActionEvent e ){
                         activeButtonIdx = 2;
                         amns.setEnabled( true );
                         amns.setVisible( true );
                     }
     } );
  }
}
