package clarion.options.nacs;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to Episodic Memory by GUI. It contains
 * the options such as threshold, parameters for using EM to train ACS, rule
 *  and chunk BLA etc.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of NACS configuration and it is instantiated in
 * <tt>GuiNACS</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionEM extends SelectionTemplate{
  JFrame superFrame;
  JLabel capLabel = new JLabel();
  // action-oriented EM bla
  JPanel ablaPanel = new JPanel();
  // non-action-oriented EM bla
  JPanel nblaPanel = new JPanel();
  // em
  JPanel emPanel = new JPanel();
  JLabel emLabel = new JLabel();
  JLabel emthLabel = new JLabel();
  JTextField emthField = new JTextField();
  JLabel emProbLabel1 = new JLabel();
  JLabel emProbLabel2 = new JLabel();
  JTextField emProbField = new JTextField();
  JLabel emNumLabel1 = new JLabel();
  JLabel emNumLabel2 = new JLabel();
  JTextField emNumField = new JTextField();
  JLabel emFreqLabel1 = new JLabel();
  JLabel emFreqLabel2 = new JLabel();
  JTextField emFreqField = new JTextField();
  // return button.
  JButton returnButton = new JButton();

  public SelectionEM( JFrame superFrame, int width, int height, TransferManager manager )
  {
    super( superFrame, width, height, manager );
    initGui();
  }

  /** initialize the GUI components in this frame. */
  public void initGui()
  {
    final int EM_TH = 0, EM_PROB = 1, EM_ITEM = 2, EM_ITER = 3, ABLA = 4, NBLA = 5;

    options = manager.getOptions( Global.NACS, Global.NACS_EM, 0, 0, 0, 0 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    setSize( width, height );
    this.getContentPane().setLayout(null);
    // caption
    initLabel( capLabel,"EM   OPTIONS",new java.awt.Font("Dialog", 1, 16),
               SwingConstants.CENTER, new Rectangle(50, 0, 240, 26));
    // EM parameters
    initPanel( emPanel, BORDER, null, new Rectangle(10, 30, 320, 170));
    initLabel( emLabel,"EM parameters", new Rectangle(4, 4, 96, 17) );
    initLabel( emthLabel, "EM threshold", new Rectangle(4, 28, 80, 17));
    initField( emthField, String.valueOf(options[EM_TH].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(85, 28, 32, 17), 4, options[EM_TH], 0, 0  );
    initLabel( emProbLabel1, "probability of using EM for off-line training of the ACS",
               new Rectangle(4,48,300,17));
    initLabel( emProbLabel2, "at each step", new Rectangle(4, 68, 70, 17) );
    initField( emProbField, String.valueOf(options[EM_PROB].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(80, 68, 32, 17), 4, options[EM_PROB], 0, 0  );
    initLabel( emNumLabel1, "number of items selected for off-line training of ACS", new Rectangle(4, 88, 300, 17));
    initLabel( emNumLabel2, "at each step", new Rectangle(4, 108, 70, 17));
    initField( emNumField, String.valueOf(options[EM_ITEM].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(80, 108, 24, 17), 4, options[EM_ITEM], 0, 0  );
    initLabel( emFreqLabel1, "number of iterations of off-line training of the ACS", new Rectangle(4, 128, 300, 17));
    initLabel( emFreqLabel2, "at each step", new Rectangle(4, 148, 70, 17));
    initField( emFreqField, String.valueOf(options[EM_ITER].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(80, 148, 24, 17), 4, options[EM_ITER], 0, 0  );

    // action-oriented EM
    initPanel( ablaPanel, BORDER, null, new Rectangle(10, 210, 320, 180) );
    initBlaBlock( ablaPanel, "action-oriented EM base-level activation", options[ABLA] );
    // non-action-oriented EM
    initPanel( nblaPanel, BORDER, null, new Rectangle(10, 400, 320, 180) );
    initBlaBlock( nblaPanel, "non-action-oriented EM base-level activation", options[NBLA] );
    // RETTURN BUTTONS.
    initReturnButton( returnButton, "RETURN", new Rectangle(220, 590, 80, 29));
    emPanel.add(emLabel, null);
    emPanel.add(emthLabel, null);
    emPanel.add(emthField, null);
    emPanel.add(emProbLabel1, null);
    emPanel.add(emProbLabel2, null);
    emPanel.add(emProbField, null);
    emPanel.add(emFreqLabel1, null);
    emPanel.add(emFreqLabel2, null);
    emPanel.add(emFreqField, null);
    emPanel.add(emNumLabel1, null);
    emPanel.add(emNumLabel2, null);
    emPanel.add(emNumField, null);
    this.getContentPane().add(capLabel, null);
    this.getContentPane().add(emPanel, null);
    this.getContentPane().add(ablaPanel, null);
    this.getContentPane().add(nblaPanel, null);
    this.getContentPane().add(returnButton, null);
    update();
  }

  public void update()
  {
  }
}
