package clarion.options.nacs;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to GKS, the upper level of NACS subsystem, by GUI.
 * It contains the options such as I/O dimension information, temperature,
 *  similarity measure, rule and chunk BLA, reasoning method and parameters for
 *  knowledge encoding and extraction etc.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of NACS configuration and it is instantiated in
 * <tt>GuiNACS</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionGks extends SelectionTemplate{
 final int IN_DIM = 0, OUT_DIM = 1;
 final int TEMP = 2, ESC = 3, SIM_ON = 4, RBLA_ON = 5, CBLA_ON = 6, WGT_C = 7;
 final int WGT_R = 8, RBLA = 9, CBLA = 10, SBR = 11, SBR_TH = 12, SBR_PARAM = 13;
 final int RENC_ON = 14, CENC_ON = 15, REXT_ON = 16, CEXT_ON = 17;
 final int RENC_PROB = 18, CENC_PROB = 19, RDEN = 20, CDEN = 21;
 final int REXT_TH = 22, CEXT_TH = 23, REXT_PROB = 24, CEXT_PROB = 25;
 final int GKS = 26, ITER = 27, RTH = 28;
 final int ON = 0, OFF = 1;

 JFrame superFrame;
 JLabel capLabel = new JLabel();
 // general
 JPanel genPanel = new JPanel();
 JLabel ioLabel = new JLabel();
 JButton inDimButton = new JButton();
 JButton outDimButton = new JButton();
 JLabel tempLabel = new JLabel();
 JTextField tempField = new JTextField();
 JLabel escLabel = new JLabel();
 JRadioButton[] escButtons = new JRadioButton[2]; {
    for( int i = 0; i < 2; i++ )
         escButtons[i] = new JRadioButton();
 }
 // representation.
 JPanel repPanel = new JPanel();
 JLabel repLabel = new JLabel();
 JLabel repRblaLabel = new JLabel();
 JRadioButton[] repRblaButtons = new JRadioButton[2];{
   repRblaButtons[0] = new JRadioButton();
   repRblaButtons[1] = new JRadioButton();
 }
 JLabel repCblaLabel = new JLabel();
 JRadioButton[] repCblaButtons = new JRadioButton[2];{
   repCblaButtons[0] = new JRadioButton();
   repCblaButtons[1] = new JRadioButton();
 }
 JLabel repSimLabel = new JLabel();
 JRadioButton[] repSimButtons = new JRadioButton[2];{
   repSimButtons[0] = new JRadioButton();
   repSimButtons[1] = new JRadioButton();
 }
 boolean[][] simCompOns = new boolean[3][];
 JPanel[] simPanels = new JPanel[3]; {
   for( int i = 0; i < 3; i++ )
        simPanels[i] = new JPanel();
 }
 // weights of rule condition chunks.
 JLabel cwgtLabel = new JLabel();
 JRadioButton[] cwgtButtons = new JRadioButton[2];{
   cwgtButtons[0] = new JRadioButton();
   cwgtButtons[1] = new JRadioButton();
 }
 JButton[] cwgtUDButtons = new JButton[2]; {
   cwgtUDButtons[0] = new JButton();
   cwgtUDButtons[1] = new JButton();
 }
 // weights of dimensions in a chunk.
 JLabel rwgtLabel = new JLabel();
 JRadioButton[] rwgtButtons = new JRadioButton[2];{
   rwgtButtons[0] = new JRadioButton();
   rwgtButtons[1] = new JRadioButton();
 }
 JButton[] rwgtUDButtons = new JButton[2]; {
   rwgtUDButtons[0] = new JButton();
   rwgtUDButtons[1] = new JButton();
 }
 // similarity
 JLabel simLabel = new JLabel();
 JRadioButton[] simButtons = new JRadioButton[2];{
   simButtons[0] = new JRadioButton();
   simButtons[1] = new JRadioButton();
 }
 JButton[] simUDButtons = new JButton[2]; {
   simUDButtons[0] = new JButton();
   simUDButtons[1] = new JButton();
 }
 JLabel simLabel11 = new JLabel();
 JLabel simLabel1 = new JLabel();
 JLabel simthLabel = new JLabel();
 JTextField simthField = new JTextField();
 // rule bla
 boolean[] rblaCompOns;
 JPanel rblaPanel = new JPanel();
 // chunk bla.
 boolean[] cblaCompOns;
 JPanel cblaPanel = new JPanel();
 /*
 // action-oriented EM bla
 JPanel ablaPanel = new JPanel();
 // non-action-oriented EM bla
 JPanel nblaPanel = new JPanel();
 */
 // learning.
 JPanel learnPanel = new JPanel();
 JLabel learnLabel = new JLabel();
 JLabel ruleExtLabel = new JLabel();
 JRadioButton[] ruleExtButtons = new JRadioButton[2];{
   ruleExtButtons[0] = new JRadioButton();
   ruleExtButtons[1] = new JRadioButton();
 }
 JLabel ruleEncLabel = new JLabel();
 JRadioButton[] ruleEncButtons = new JRadioButton[2];{
   ruleEncButtons[0] = new JRadioButton();
   ruleEncButtons[1] = new JRadioButton();
 }
 JLabel ruleAssimLabel = new JLabel();
 JRadioButton[] ruleAssimButtons = new JRadioButton[2];{
   ruleAssimButtons[0] = new JRadioButton();
   ruleAssimButtons[1] = new JRadioButton();
 }
 JLabel chunkExtLabel = new JLabel();
 JRadioButton[] chunkExtButtons = new JRadioButton[2];{
   chunkExtButtons[0] = new JRadioButton();
   chunkExtButtons[1] = new JRadioButton();
 }
 JLabel chunkEncLabel = new JLabel();
 JRadioButton[] chunkEncButtons = new JRadioButton[2];{
   chunkEncButtons[0] = new JRadioButton();
   chunkEncButtons[1] = new JRadioButton();
 }
 // knowledge encoding
 JPanel encodePanel = new JPanel();
 JLabel encodeLabel = new JLabel();
 JLabel chunkEncProbLabel = new JLabel();
 JTextField chunkEncProbField = new JTextField();
 JLabel ruleEncProbLabel = new JLabel();
 JTextField ruleEncProbField = new JTextField();
 // density
 JPanel denPanel = new JPanel();
 JLabel densityLabel = new JLabel();
 JLabel chunkDenLabel = new JLabel();
 JTextField chunkDenField = new JTextField();
 JLabel ruleDenLabel = new JLabel();
 JTextField ruleDenField = new JTextField();
 // explicit knowledge extraction from AMN
 JPanel extractPanel = new JPanel();
 JLabel extLabel = new JLabel();
 JLabel ruleExtThLabel = new JLabel();
 JLabel chunkExtThLabel = new JLabel();
 JLabel ruleExtProbLabel = new JLabel();
 JLabel chunkExtProbLabel = new JLabel();
 JTextField[] ruleExtFields = new JTextField[2];{
   ruleExtFields[0] = new JTextField();
   ruleExtFields[1] = new JTextField();
 }
 JTextField[] chunkExtFields = new JTextField[2];{
   chunkExtFields[0] = new JTextField();
   chunkExtFields[1] = new JTextField();
 }
 /*
 // app.
 JPanel appPanel = new JPanel();
 JLabel appLabel = new JLabel();
 JLabel appLabel1 = new JLabel();
 JLabel appLabel2 = new JLabel();
 JRadioButton[] appButtons = new JRadioButton[2];{
   appButtons[0] = new JRadioButton();
   appButtons[1] = new JRadioButton();
 }
 */
 // reasoning
 JPanel reasonPanel = new JPanel();
 JLabel reasonLabel = new JLabel();
 JLabel reasonLabel1 = new JLabel();
 JRadioButton[] reasonButtons = new JRadioButton[2]; {
    for( int i = 0; i < 2; i++ )
         reasonButtons[i] = new JRadioButton();
 }
 // iteration
 JLabel iterLabel = new JLabel();
 JRadioButton[] iterButtons = new JRadioButton[1]; {
   for( int i = 0; i < 1; i++ )
        iterButtons[i] = new JRadioButton();
 }
 JTextField iterField = new JTextField();
 // iteration
 JLabel rthLabel = new JLabel();
 JTextField rthField = new JTextField();

 // return buttons.
 JButton returnButton = new JButton();
 OptionDims[] optDims = new OptionDims[2];

 public SelectionGks( JFrame superFrame, int width, int height, TransferManager manager )
 {
    super( superFrame, width, height, manager );
    initGui();
 }

 /** initialize the GUI components in this frame. */
 public void initGui()
 {
    options = manager.getOptions( Global.NACS, Global.NACS_GKS, 0, 0, 0, 0 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    setSize( width, height );
    //setSize( 960, 720 );
    this.getContentPane().setLayout(null);
    // caption.
    initLabel( capLabel,"G K S     O P T I O N S", new java.awt.Font("Dialog", 1, 20),
               SwingConstants.CENTER, new Rectangle(338, 1, 277, 27));
    // general
    initPanel( genPanel,BORDER, null, new Rectangle(6, 30, 310, 130));
    initLabel( ioLabel,"GKS general options",new Rectangle(4, 8, 125, 17) );
    initButton( inDimButton, "dimensions in input", new Rectangle( 40, 30, 180, 20));
    initButton( outDimButton, "dimensions in output", new Rectangle( 40, 55, 180, 20));
    // frames for dimension selection
    optDims[0] = new OptionDims( this, 420, 780, "IDN INPUT SPECIFICATION", 30, manager,
        options[IN_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).getIntValues()[0], options[IN_DIM] );
    optDims[1] = new OptionDims( this, 420, 780, "IDN OUTPUT SPECIFICATION", 30, manager,
        options[OUT_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).getIntValues()[0], options[OUT_DIM] );
    initDimButton( this, inDimButton, optDims[0] );
    initDimButton( this, outDimButton, optDims[1] );
    initLabel( tempLabel, "temperature for chunk selection", new Rectangle(4, 84, 200, 17));
    initField( tempField, String.valueOf(options[TEMP].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(210, 84, 30, 17), 5, options[TEMP], 0, 0 );
    Rectangle[] escRects = {new Rectangle(173, 104, 54, 17),new Rectangle(233, 104, 52, 17)};
    initButtonBlock( genPanel, escLabel, "experience-specific chunks", new Rectangle(4, 104, 159, 17),
                     escButtons, onOffTexts, escRects, options[ESC] );
    // weights of rule condition chunks
    initPanel(simPanels[0], BORDER, null, new Rectangle(6, 300, 310, 130));
    String[] cwgtUDTexts = { TaskClarion.getSample( TaskClarion.NACS, TaskClarion.ASSOC_WGT_C ),
                          TaskClarion.getExplanation( TaskClarion.NACS, TaskClarion.ASSOC_WGT_C ) };
    Rectangle[] cwgtUDRects = { new Rectangle(24, 76, 160, 17), new Rectangle(24, 98, 160, 17) };
    initUserDefinedBlock( simPanels[0], cwgtUDButtons, cwgtUDRects, cwgtUDTexts );
    String[] cwgtTexts = { "default (all weights are equal)", "user defined" };
    Rectangle[] cwgtRects = { new Rectangle(8, 28, 200, 17), new Rectangle(8, 54, 120, 17)};
    initButtonBlock( simPanels[0], cwgtLabel, "Weights of associative rule conditions", new Rectangle(4, 4, 240, 17),
                     cwgtButtons, cwgtTexts, cwgtRects, options[WGT_C], 1, cwgtUDButtons );
    simCompOns[0] = new boolean[simPanels[0].getComponentCount()];
    super.initPanelStatus( options[SIM_ON], ON, simPanels[0], simCompOns[0] );
    // weights of dimensions in a chunk
    initPanel(simPanels[1], BORDER, null, new Rectangle(6, 450, 310, 130));
    String[] rwgtUDTexts = { TaskClarion.getSample( TaskClarion.NACS, TaskClarion.ASSOC_WGT_R ),
                          TaskClarion.getExplanation( TaskClarion.NACS, TaskClarion.ASSOC_WGT_R ) };
    Rectangle[] rwgtUDRects = { new Rectangle(24, 76, 160, 17), new Rectangle(24, 98, 160, 17) };
    initUserDefinedBlock( simPanels[1], rwgtUDButtons, rwgtUDRects, rwgtUDTexts );
    String[] rwgtTexts = { "default ( all weights  are 1 )", "user defined" };
    Rectangle[] rwgtRects = { new Rectangle(8, 28, 200, 17), new Rectangle(8, 54, 120, 17)};
    initButtonBlock( simPanels[1], rwgtLabel, "Weights of dimensions in a chunk", new Rectangle(4, 4, 240, 17),
                     rwgtButtons, rwgtTexts, rwgtRects, options[WGT_R], 1, rwgtUDButtons );
    simCompOns[1] = new boolean[simPanels[1].getComponentCount()];
    super.initPanelStatus( options[SIM_ON], ON, simPanels[1], simCompOns[1] );
    // similarity.
    initPanel( simPanels[2], BORDER, null, new Rectangle(320, 400, 290, 180));
    initLabel( simLabel1, "Options for similarity-based reasoning", new Rectangle(4, 8, 240, 17));
    String[] simUDTexts = { TaskClarion.getSample( TaskClarion.NACS, TaskClarion.SIM ),
                            TaskClarion.getExplanation( TaskClarion.NACS, TaskClarion.SIM ) };
    Rectangle[] simUDRects = { new Rectangle(24, 104, 160, 17), new Rectangle(24, 124, 160, 17) };
    initUserDefinedBlock( simPanels[2], simUDButtons, simUDRects, simUDTexts );
    String[] simTexts = {"default", "user defined"};
    Rectangle[] simRects = {new Rectangle(12, 48, 78, 17),new Rectangle(12, 84, 100, 17)};
    initButtonBlock( simPanels[2], simLabel, "similarity measure", new Rectangle(8, 28, 114, 17),
                     simButtons, simTexts, simRects, options[SBR], 1, simUDButtons );
    initLabel( simLabel11,"S(c1,c2) = dim # of (c1 && c2) / f(dim # of c2)",new Rectangle(28, 64, 256, 17));
    initLabel( simthLabel, "similarity threshold", new Rectangle(12, 148, 114, 17));
    initField( simthField, String.valueOf(options[SBR_TH].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(129, 148, 37, 17), 4, options[SBR_TH], 0, 0 );
    simPanels[2].add(simLabel, null);
    simPanels[2].add(simLabel1, null);
    simPanels[2].add(simLabel11, null);
    simPanels[2].add(simthLabel, null);
    simPanels[2].add(simthField, null);
    simCompOns[2] = new boolean[simPanels[2].getComponentCount()];
    super.initPanelStatus( options[SIM_ON], ON, simPanels[2], simCompOns[2] );
    /*
    initLabel( simbalLabel, "balancing constants", new Rectangle(12, 172, 128, 17));
    initLabel( simbalLabel1, "c14 =", new Rectangle(140, 172, 32, 17) );
    initField( simbalField1, String.valueOf(options[SBR_PARAM].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(172, 172, 24, 17), 4, options[SBR_PARAM], 0, 0 );
    initLabel( simbalLabel2, "c15 =", new Rectangle(220, 172, 32, 17));
    initField( simbalField2, String.valueOf(options[SBR_PARAM].getOptionItem(0).getParameter(1).getDoubleValue()),
               new Rectangle(252, 172, 24, 17), 4, options[SBR_PARAM], 0, 1 );
    */
    // rule bla.
    initPanel( rblaPanel, BORDER, null, new Rectangle(320, 30, 290, 180) );
    initBlaBlock( rblaPanel, "Options for associative rule base-level activation", options[RBLA] );
    rblaCompOns = new boolean[rblaPanel.getComponentCount()];
    super.initPanelStatus( options[RBLA_ON], ON, rblaPanel, rblaCompOns );
    // chunk bla.
    initPanel( cblaPanel, BORDER, null, new Rectangle(320, 215, 290, 180) );
    initBlaBlock( cblaPanel, "NACS chunk base-level activation", options[CBLA] );
    cblaCompOns = new boolean[cblaPanel.getComponentCount()];
    super.initPanelStatus( options[CBLA_ON], ON, cblaPanel, cblaCompOns  );
    // representation
    initPanel( repPanel, BORDER, null, new Rectangle(6, 190, 310, 90) );
    initLabel( repLabel, "Options for associative rule representations", new Rectangle(8, 6, 300, 17));
    Rectangle[] repRblaRects = {new Rectangle(156, 28, 44, 17),new Rectangle(208, 28, 52, 17)};
    initButtonBlock( repPanel, repRblaLabel, "rule base-level activation", new Rectangle(12, 28, 143, 17),
                     repRblaButtons, onOffTexts, repRblaRects, options[RBLA_ON], ON, rblaPanel, rblaCompOns );
    Rectangle[] repCblaRects = {new Rectangle(176, 48, 44, 17),new Rectangle(228, 48, 52, 17)};
    initButtonBlock( repPanel, repCblaLabel, "chunk base-level activation", new Rectangle(12, 48, 160, 17),
                     repCblaButtons, onOffTexts, repCblaRects, options[CBLA_ON], ON, cblaPanel, cblaCompOns );
    Rectangle[] repSimRects = {new Rectangle(100, 68, 44, 17),new Rectangle(166, 68, 52, 17)};
    initButtonBlock( repPanel, repSimLabel, "similarity", new Rectangle(12, 68, 80, 17),
                     repSimButtons, onOffTexts, repSimRects, options[SIM_ON], ON, simPanels, simCompOns );
    /*
    // action-oriented EM
    initPanel( ablaPanel, BORDER, null, new Rectangle(320, 510, 290, 190) );
    initBlaBlock( ablaPanel, "action-oriented EM base-level activation", options[ABLA] );
    // non-action-oriented EM
    initPanel( nblaPanel, BORDER, null, new Rectangle(614, 30, 330, 180) );
    initBlaBlock( nblaPanel, "non-action-oriented EM base-level activation", options[NBLA] );
    */
    // NACS learning.
    initPanel( learnPanel, BORDER, null, new Rectangle(614, 30, 330, 110) );
    initLabel( learnLabel, "Options for non-action centered learning",new Rectangle(4, 6, 230, 17) );
    Rectangle[] ruleEncRects = {new Rectangle(174, 30, 44, 17), new Rectangle(218, 30, 52, 17)};
    initButtonBlock( learnPanel, ruleEncLabel, "encoding associative rules", new Rectangle(8, 30, 155, 17),
                     ruleEncButtons, onOffTexts, ruleEncRects, options[RENC_ON] );
    Rectangle[] chunkEncRects = {new Rectangle(123, 50, 44, 17), new Rectangle(168, 50, 52, 17)};
    initButtonBlock( learnPanel, chunkEncLabel, "encoding chunks", new Rectangle(8, 50, 102, 17),
                     chunkEncButtons, onOffTexts, chunkEncRects, options[CENC_ON] );
    Rectangle[] ruleExtRects = {new Rectangle(186, 70, 44, 17), new Rectangle(230, 70, 52, 17)};
    initButtonBlock( learnPanel, ruleExtLabel, "associative rule extraction", new Rectangle(8, 70, 176, 17),
                     ruleExtButtons, onOffTexts, ruleExtRects, options[REXT_ON], ON, ruleExtFields );
    Rectangle[] chunkExtRects = {new Rectangle(206, 90, 44, 17), new Rectangle(250, 90, 52, 17)};
    initButtonBlock( learnPanel, chunkExtLabel, "chunk extraction from the AMN", new Rectangle(8, 90, 176, 17),
                     chunkExtButtons, onOffTexts, chunkExtRects, options[CEXT_ON], ON, chunkExtFields );
    // knowledge encoding
    initPanel( encodePanel, BORDER, null, new Rectangle(614, 145, 330, 70));
    initLabel( encodeLabel, "Probability parameters for explicit knowledge encoding", new Rectangle(4, 6, 320, 17) );
    initLabel( chunkEncProbLabel, "For chunks", new Rectangle(8, 28, 72, 17));
    initField( chunkEncProbField, String.valueOf(options[CENC_PROB].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(80, 28, 32, 17), 4, options[CENC_PROB], 0, 0  );
    initLabel( ruleEncProbLabel, "For associative rules", new Rectangle(8, 48, 124, 17));
    initField( ruleEncProbField, String.valueOf(options[RENC_PROB].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(130, 48, 32, 17), 4, options[RENC_PROB], 0, 0 );
    // density
    initPanel( denPanel, BORDER, null, new Rectangle(614, 220, 330, 70));
    initLabel( densityLabel,"Density parameters",new Rectangle(4, 6, 122, 17) );
    initLabel( chunkDenLabel, "For chunks", new Rectangle(8, 28, 72, 17) );
    initField( chunkDenField, String.valueOf(options[CDEN].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(80, 28, 40, 17), 5, options[CDEN], 0, 0 );
    initLabel( ruleDenLabel, "For associative rules", new Rectangle(8, 48, 124, 17) );
    initField( ruleDenField, String.valueOf(options[RDEN].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(130, 48, 40, 17), 5, options[RDEN], 0, 0 );
    // explicit knowledge extraction from AMN
    initPanel( extractPanel, BORDER,null, new Rectangle(614, 295, 330, 110));
    initLabel( extLabel, "Parameters for explicit knowledge extraction from AMN",new Rectangle(4, 6, 320, 17) );
    initLabel( ruleExtThLabel, "extraction threshold for associative rules", new Rectangle(8, 29, 240, 17));
    initField( ruleExtFields[0], String.valueOf(options[REXT_TH].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(248, 28, 32, 17), 4, options[REXT_TH], 0, 0  );
    initLabel( chunkExtThLabel, "extraction threshold for chunks", new Rectangle(8, 48, 184, 17));
    initField( chunkExtFields[0], String.valueOf(options[CEXT_TH].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(192, 48, 32, 17), 4, options[CEXT_TH], 0, 0 );
    initLabel( ruleExtProbLabel,"extraction probability for associative rules",new Rectangle(8, 68, 250, 17));
    initField( ruleExtFields[1], String.valueOf(options[REXT_PROB].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(258, 68, 32, 17), 4, options[REXT_PROB], 0, 0 );
    initLabel( chunkExtProbLabel, "extraction probability for chunks", new Rectangle(8, 88, 190, 17));
    initField( chunkExtFields[1], String.valueOf(options[CEXT_PROB].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(200, 88, 32, 17), 4, options[CEXT_PROB], 0, 0 );
    /*
    // app
    initPanel( appPanel, BORDER, null, new Rectangle(614, 730, 330, 70));
    String[] appTexts = {"minimum activation","average activation"};
    Rectangle[] appRects = {new Rectangle(6, 44, 133, 17),new Rectangle( 166, 44, 123, 17)};
    initButtonBlock( appPanel, appLabel, "chunk extraction from the AMN", new Rectangle(8, 114, 169, 17),
                     appButtons, appTexts, appRects, options[TH_MEASURE] );
    initLabel( appLabel1, "Options for application of the extraction theshold", new Rectangle(4, 8, 280, 17) );
    initLabel( appLabel2, "for associative rules and for chunks", new Rectangle(4, 25, 256, 17) );
    */
   // reasoning
   initPanel( reasonPanel, BORDER, null, new Rectangle(614, 410, 330, 130));
   initLabel( reasonLabel, "Use of GKS",new Rectangle(8, 4, 200, 17));
   String[] reasonTexts = { "forward chaining", "similarity-based forward reasoning" };
   Rectangle[] reasonRects = { new Rectangle(16, 48, 140, 17), new Rectangle(16, 68, 240, 17)  };
   initButtonBlock( reasonPanel, reasonLabel1, "GKS reasoning method", new Rectangle(12, 28, 140, 17),
                    reasonButtons, reasonTexts, reasonRects, options[GKS] );
   // # of iterations.
   initLabel( iterLabel, "number of iterations of reasoning", new Rectangle(12, 90, 200, 17) );
   initField( iterField, String.valueOf(options[ITER].getOptionItem(0).getParameter(0).getIntValue()),
              new Rectangle(210, 90, 32, 17) );
   // reasoning threshold.
   initLabel( rthLabel, "reasoning threshold", new Rectangle(12, 110, 180, 17));
   initField( rthField, String.valueOf(options[RTH].getOptionItem(0).getParameter(0).getDoubleValue()),
              new Rectangle(150, 110, 32, 17) );

    // return buttons.
    initReturnButton( returnButton, "RETURN", new Rectangle(834, 550, 110, 29));
    genPanel.add(ioLabel, null);
    genPanel.add(inDimButton, null);
    genPanel.add(outDimButton, null);
    genPanel.add(tempLabel, null);
    genPanel.add(tempField, null);
    repPanel.add(repLabel, null);
    learnPanel.add(learnLabel, null);
    encodePanel.add(encodeLabel, null);
    encodePanel.add(chunkEncProbLabel, null);
    encodePanel.add(ruleEncProbLabel, null);
    encodePanel.add(chunkEncProbField, null);
    encodePanel.add(ruleEncProbField, null);
    denPanel.add(densityLabel, null);
    denPanel.add(ruleDenLabel, null);
    denPanel.add(chunkDenLabel, null);
    denPanel.add(ruleDenField, null);
    denPanel.add(chunkDenField, null);
    extractPanel.add(extLabel, null);
    extractPanel.add(ruleExtThLabel, null);
    extractPanel.add(ruleExtFields[0], null);
    extractPanel.add(chunkExtThLabel, null);
    extractPanel.add(chunkExtFields[0], null);
    extractPanel.add(ruleExtProbLabel, null);
    extractPanel.add(ruleExtFields[1], null);
    extractPanel.add(chunkExtProbLabel, null);
    extractPanel.add(chunkExtFields[1], null);
    //appPanel.add(appLabel1, null);
    //appPanel.add(appLabel2, null);
    reasonPanel.add(reasonLabel, null);
    reasonPanel.add(reasonLabel1, null);
    reasonPanel.add(iterLabel, null);
    reasonPanel.add(iterField, null);
    reasonPanel.add(rthLabel, null);
    reasonPanel.add(rthField, null);

    /*
    parent.add(capLabel, null);
    parent.add(genPanel, null);
    parent.add(repPanel, null);
    parent.add(simPanels[0], null);
    parent.add(simPanels[1], null);
    parent.add(simPanels[2], null);
    parent.add(cblaPanel, null);
    parent.add(rblaPanel, null);
    parent.add(learnPanel, null);
    parent.add(encodePanel, null);
    parent.add(denPanel, null);
    parent.add(extractPanel, null);
    //parent.add(appPanel, null);
    parent.add(returnButton, null);
    */
    this.getContentPane().add(capLabel, null);
    this.getContentPane().add(genPanel, null);
    this.getContentPane().add(repPanel, null);
    this.getContentPane().add(simPanels[0], null);
    this.getContentPane().add(simPanels[1], null);
    this.getContentPane().add(simPanels[2], null);
    this.getContentPane().add(cblaPanel, null);
    this.getContentPane().add(rblaPanel, null);
    this.getContentPane().add(learnPanel, null);
    this.getContentPane().add(encodePanel, null);
    this.getContentPane().add(denPanel, null);
    this.getContentPane().add(extractPanel, null);
    //this.getContentPane().add(appPanel, null);
    this.getContentPane().add(reasonPanel, null);
    this.getContentPane().add(returnButton, null);
    update();
 }

 public void update()
 {
    // EM.
    if( manager.isEmOn() ){
        /*
        ablaPanel.setEnabled( true );
        for( int i = 0; i < ablaPanel.getComponentCount(); i++ )
             ablaPanel.getComponent(i).setEnabled( true );
        super.updateBlaPanel( ablaPanel, options[ABLA] );
        nblaPanel.setEnabled( true );
        for( int i = 0; i < nblaPanel.getComponentCount(); i++ )
             nblaPanel.getComponent(i).setEnabled( true );
        super.updateBlaPanel( nblaPanel, options[NBLA] );
        */
    }
    else{
        /*
        for( int i = 0; i < ablaPanel.getComponentCount(); i++ )
             ablaPanel.getComponent(i).setEnabled( false );
        nblaPanel.setEnabled( false );
        for( int i = 0; i < nblaPanel.getComponentCount(); i++ )
             nblaPanel.getComponent(i).setEnabled( false );
        */
    }
 }
}
