package clarion.options.supervise;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import clarion.system.*;
import clarion.options.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to show the top-level screen of configuring Supervisory subsystem
 * by GUI. It extends the base class <em>GuiTemplate</em>.  It contains the following
 * configuration entries:  Supervisory Subsystem components selection, Motivational
 * Subsystem and Meta-Cognitive Subsystem.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public void returnFrom(): update the GUI components when return from other
 * screens.
 * <li> public void addActionListeners( final JFrame mainFrame ) :  event listeners
 * for the buttons.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to control the process of configuring the Supervisory subsystem by
 * GUI (on-line configuration).
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>GuiClarion</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class GuiSupervisory extends GuiTemplate{

  //final int SUB_OPT_NUMS[] = {  2, 2, 29 };
  String buttonTexts[] = {
     "Options for Supervisory Subsystem components",
     "Options for Motivational Subsystem",
     "Options for Meta-Cognitive Subsystem",
  };
  SelectionTemplate subSelections[];

  public GuiSupervisory( int num )
  {
     super( num, "OPTIONS OF THE SUPERVISORY SUBSYSTEMS" );
     setButtonTexts( buttonTexts );
     subSelections = new SelectionTemplate[3];
     subSelections[0] = new SelectionSuperComp( this, 400, 240, manager );
     subSelections[1] = new SelectionMS( this, 750, 300, manager );
     subSelections[2] = new SelectionMCS( this, 1020, 1080, manager );
  }

  /** update the GUI components when return from other screens. */
  public void returnFrom()
  {
     if( manager.isMsOn() ){
         optButtons[1].setEnabled( true );
         if( subSelections[1] == null )
             subSelections[1] = new SelectionMS( this, 750, 300, manager );
         ((SelectionMS)subSelections[1]).update();
     }
     else
         optButtons[1].setEnabled( false );
     if( manager.isMcsOn() ){
         optButtons[2].setEnabled( true );
         if( subSelections[2] == null )
             subSelections[2] = new SelectionMCS( this, 1020, 1080, manager );
         ((SelectionMCS)subSelections[2]).update();
     }
     else
         optButtons[2].setEnabled( false );
  }

  /** event listeners for the buttons. */
  public void addActionListeners( final JFrame mainFrame )
  {
     for( int i = 0; i < OPT_NUM; i++ ){
          final int idx = i;
          optButtons[ idx ].addActionListener(  new  ActionListener(){
                            public void actionPerformed( ActionEvent e ){
                               activeButtonIdx = idx;
                               subSelections[idx].setEnabled( true );
                               subSelections[idx].setVisible( true );
                            }
                    } );
     }
  }
}
