package clarion.options.supervise;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to Meta-Cognitive Subsystem in Supervisory
 * subsystem by GUI. It contains the options such as behavioral aims, information
 * filtering, information acquisition, information utilization, outcome selection,
 * cognitive mode and parameters for ACS and NACS configuration.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of Supervisory subsystem configuration and it is instantiated
 * in <tt>GuiSupervisory</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionMCS extends SelectionTemplate{
 final int REIN_FUNC = 0, GS_DRIVE = 1;
 final int ACS_IN_DIM = 2, NACS_IN_DIM = 3, ACS_OUT_DIM = 4, NACS_OUT_DIM = 5;
 final int ACS_IN_VAL = 6, NACS_IN_VAL = 7, ACS_OUT_VAL = 8, NACS_OUT_VAL = 9;
 final int ACS_LEARN = 10, NACS_LEARN = 11, ACS_REASON = 12, NACS_REASON = 13;
 final int ACS_CL = 14, ACS_PARAM = 15, NACS_PARAM = 16, ACS_REGU = 17, GS_TIME = 18;
 final int ACS_PERFORM = 19, NACS_PERFORM = 20, ACS_CONC_NUM = 21, NACS_CONC_NUM = 22;
 final int ACS_GAUGE = 23, NACS_GAUGE = 24, ACS_EPI_NUM = 25, NACS_EPI_NUM = 26;
 final int EPI_DEF = 27, ACS_INTEG = 28, NACS_INTEG = 29;
 final int ON = 0, OFF = 1;

 JFrame superFrame;
 JLabel capLabel = new JLabel();
 // reinforcement function
 JPanel funcPanel = new JPanel();
 JLabel funcLabel = new JLabel();
 JRadioButton[] funcButtons = new JRadioButton[2];{
   funcButtons[0] = new JRadioButton();
   funcButtons[1] = new JRadioButton();
 }
 JButton[] funcUDButtons = new JButton[2]; {
   funcUDButtons[0] = new JButton();
   funcUDButtons[1] = new JButton();
 }
 // goal setting
 JPanel goalPanel = new JPanel();
 JLabel goalLabel = new JLabel();
 JRadioButton[] goalButtons = new JRadioButton[2];{
   goalButtons[0] = new JRadioButton();
   goalButtons[1] = new JRadioButton();
 }
 JButton[] goalUDButtons = new JButton[2]; {
   goalUDButtons[0] = new JButton();
   goalUDButtons[1] = new JButton();
 }
 // input dimension selection
 JPanel inputDimPanel = new JPanel();
 JLabel acsInputDimLabel = new JLabel();
 JRadioButton[] acsInputDimButtons = new JRadioButton[2];{
   acsInputDimButtons[0] = new JRadioButton();
   acsInputDimButtons[1] = new JRadioButton();
 }
 JButton[] acsInputDimUDButtons = new JButton[2]; {
   acsInputDimUDButtons[0] = new JButton();
   acsInputDimUDButtons[1] = new JButton();
 }
 JLabel nacsInputDimLabel = new JLabel();
 JRadioButton[] nacsInputDimButtons = new JRadioButton[2];{
   nacsInputDimButtons[0] = new JRadioButton();
   nacsInputDimButtons[1] = new JRadioButton();
 }
 JButton[] nacsInputDimUDButtons = new JButton[2]; {
   nacsInputDimUDButtons[0] = new JButton();
   nacsInputDimUDButtons[1] = new JButton();
 }
 // output dimension selection
 JPanel outputDimPanel = new JPanel();
 JLabel acsOutputDimLabel = new JLabel();
 JRadioButton[] acsOutputDimButtons = new JRadioButton[2];{
   acsOutputDimButtons[0] = new JRadioButton();
   acsOutputDimButtons[1] = new JRadioButton();
 }
 JButton[] acsOutputDimUDButtons = new JButton[2]; {
   acsOutputDimUDButtons[0] = new JButton();
   acsOutputDimUDButtons[1] = new JButton();
 }
 JLabel nacsOutputDimLabel = new JLabel();
 JRadioButton[] nacsOutputDimButtons = new JRadioButton[2];{
   nacsOutputDimButtons[0] = new JRadioButton();
   nacsOutputDimButtons[1] = new JRadioButton();
 }
 JButton[] nacsOutputDimUDButtons = new JButton[2]; {
   nacsOutputDimUDButtons[0] = new JButton();
   nacsOutputDimUDButtons[1] = new JButton();
 }
 // input value selection
 JPanel inputValPanel = new JPanel();
 JLabel acsInputValLabel = new JLabel();
 JRadioButton[] acsInputValButtons = new JRadioButton[2];{
   acsInputValButtons[0] = new JRadioButton();
   acsInputValButtons[1] = new JRadioButton();
 }
 JButton[] acsInputValUDButtons = new JButton[2]; {
   acsInputValUDButtons[0] = new JButton();
   acsInputValUDButtons[1] = new JButton();
 }
 JLabel nacsInputValLabel = new JLabel();
 JRadioButton[] nacsInputValButtons = new JRadioButton[2];{
   nacsInputValButtons[0] = new JRadioButton();
   nacsInputValButtons[1] = new JRadioButton();
 }
 JButton[] nacsInputValUDButtons = new JButton[2]; {
   nacsInputValUDButtons[0] = new JButton();
   nacsInputValUDButtons[1] = new JButton();
 }
 // output value selection
 JPanel outputValPanel = new JPanel();
 JLabel acsOutputValLabel = new JLabel();
 JRadioButton[] acsOutputValButtons = new JRadioButton[2];{
   acsOutputValButtons[0] = new JRadioButton();
   acsOutputValButtons[1] = new JRadioButton();
 }
 JButton[] acsOutputValUDButtons = new JButton[2]; {
   acsOutputValUDButtons[0] = new JButton();
   acsOutputValUDButtons[1] = new JButton();
 }
 JLabel nacsOutputValLabel = new JLabel();
 JRadioButton[] nacsOutputValButtons = new JRadioButton[2];{
   nacsOutputValButtons[0] = new JRadioButton();
   nacsOutputValButtons[1] = new JRadioButton();
 }
 JButton[] nacsOutputValUDButtons = new JButton[2]; {
   nacsOutputValUDButtons[0] = new JButton();
   nacsOutputValUDButtons[1] = new JButton();
 }
 // selection of learning methods
 JPanel learningPanel = new JPanel();
 JLabel acsLearningLabel = new JLabel();
 JRadioButton[] acsLearningButtons = new JRadioButton[2];{
   acsLearningButtons[0] = new JRadioButton();
   acsLearningButtons[1] = new JRadioButton();
 }
 JButton[] acsLearningUDButtons = new JButton[2]; {
   acsLearningUDButtons[0] = new JButton();
   acsLearningUDButtons[1] = new JButton();
 }
 JLabel nacsLearningLabel = new JLabel();
 JRadioButton[] nacsLearningButtons = new JRadioButton[2];{
   nacsLearningButtons[0] = new JRadioButton();
   nacsLearningButtons[1] = new JRadioButton();
 }
 JButton[] nacsLearningUDButtons = new JButton[2]; {
   nacsLearningUDButtons[0] = new JButton();
   nacsLearningUDButtons[1] = new JButton();
 }
 // selection of reasoning methods
 JPanel reasoningPanel = new JPanel();
 JLabel acsReasoningLabel = new JLabel();
 JRadioButton[] acsReasoningButtons = new JRadioButton[2];{
   acsReasoningButtons[0] = new JRadioButton();
   acsReasoningButtons[1] = new JRadioButton();
 }
 JButton[] acsReasoningUDButtons = new JButton[2]; {
   acsReasoningUDButtons[0] = new JButton();
   acsReasoningUDButtons[1] = new JButton();
 }
 JLabel nacsReasoningLabel = new JLabel();
 JRadioButton[] nacsReasoningButtons = new JRadioButton[2];{
   nacsReasoningButtons[0] = new JRadioButton();
   nacsReasoningButtons[1] = new JRadioButton();
 }
 JButton[] nacsReasoningUDButtons = new JButton[2]; {
   nacsReasoningUDButtons[0] = new JButton();
   nacsReasoningUDButtons[1] = new JButton();
 }
 // cross-level integration
 JPanel clIntegrationPanel = new JPanel();
 JLabel acsCLIntegrationLabel = new JLabel();
 JRadioButton[] acsCLIntegrationButtons = new JRadioButton[2];{
   acsCLIntegrationButtons[0] = new JRadioButton();
   acsCLIntegrationButtons[1] = new JRadioButton();
 }
 JButton[] acsCLIntegrationUDButtons = new JButton[2]; {
   acsCLIntegrationUDButtons[0] = new JButton();
   acsCLIntegrationUDButtons[1] = new JButton();
 }
 // setting of parameters
 JPanel paramSettingPanel = new JPanel();
 JLabel acsParamSettingLabel = new JLabel();
 JRadioButton[] acsParamSettingButtons = new JRadioButton[2];{
   acsParamSettingButtons[0] = new JRadioButton();
   acsParamSettingButtons[1] = new JRadioButton();
 }
 JButton[] acsParamSettingUDButtons = new JButton[2]; {
   acsParamSettingUDButtons[0] = new JButton();
   acsParamSettingUDButtons[1] = new JButton();
 }
 JLabel nacsParamSettingLabel = new JLabel();
 JRadioButton[] nacsParamSettingButtons = new JRadioButton[2];{
   nacsParamSettingButtons[0] = new JRadioButton();
   nacsParamSettingButtons[1] = new JRadioButton();
 }
 JButton[] nacsParamSettingUDButtons = new JButton[2]; {
   nacsParamSettingUDButtons[0] = new JButton();
   nacsParamSettingUDButtons[1] = new JButton();
 }
 // regulation of cross-level integration in the ACS.
 JPanel clRegulationPanel = new JPanel();
 JLabel acsCLRegulationLabel = new JLabel();
 JRadioButton[] acsCLRegulationButtons = new JRadioButton[2];{
   acsCLRegulationButtons[0] = new JRadioButton();
   acsCLRegulationButtons[1] = new JRadioButton();
 }
 // timing of goal setting.
 JPanel goalTimingPanel = new JPanel();
 JLabel goalTimingLabel = new JLabel();
 JRadioButton[] goalTimingButtons = new JRadioButton[2];{
   goalTimingButtons[0] = new JRadioButton();
   goalTimingButtons[1] = new JRadioButton();
 }
 JLabel thLabel1 = new JLabel();
 JLabel thLabel2 = new JLabel();
 JTextField thField1 = new JTextField();
 JTextField thField2 = new JTextField();
 // measures of performance
 JPanel performancePanel = new JPanel();
 JLabel acsPerformanceLabel = new JLabel();
 JRadioButton[] acsPerformanceButtons = new JRadioButton[2];{
   acsPerformanceButtons[0] = new JRadioButton();
   acsPerformanceButtons[1] = new JRadioButton();
 }
 JButton[] acsPerformanceUDButtons = new JButton[2]; {
   acsPerformanceUDButtons[0] = new JButton();
   acsPerformanceUDButtons[1] = new JButton();
 }
 JLabel nacsPerformanceLabel = new JLabel();
 JRadioButton[] nacsPerformanceButtons = new JRadioButton[2];{
   nacsPerformanceButtons[0] = new JRadioButton();
   nacsPerformanceButtons[1] = new JRadioButton();
 }
 JButton[] nacsPerformanceUDButtons = new JButton[2]; {
   nacsPerformanceUDButtons[0] = new JButton();
   nacsPerformanceUDButtons[1] = new JButton();
 }
 // numbers of conclusions monitored
 JPanel conclusionPanel = new JPanel();
 JLabel conclusionLabel = new JLabel();
 JLabel acsTLConcLabel1 = new JLabel();
 JLabel nacsTLConcLabel1 = new JLabel();
 JLabel acsBLConcLabel1 = new JLabel();
 JLabel nacsBLConcLabel1 = new JLabel();
 JLabel acsTLConcLabel2 = new JLabel();
 JLabel nacsTLConcLabel2 = new JLabel();
 JLabel acsBLConcLabel2 = new JLabel();
 JLabel nacsBLConcLabel2 = new JLabel();
 JTextField acsTLConcField = new JTextField();
 JTextField nacsTLConcField = new JTextField();
 JTextField acsBLConcField = new JTextField();
 JTextField nacsBLConcField = new JTextField();
 // measures of gauging Learning
 JPanel gaugingPanel = new JPanel();
 JLabel acsGaugingLabel = new JLabel();
 JRadioButton[] acsGaugingButtons = new JRadioButton[2];{
   acsGaugingButtons[0] = new JRadioButton();
   acsGaugingButtons[1] = new JRadioButton();
 }
 JButton[] acsGaugingUDButtons = new JButton[2]; {
   acsGaugingUDButtons[0] = new JButton();
   acsGaugingUDButtons[1] = new JButton();
 }
 JLabel nacsGaugingLabel = new JLabel();
 JRadioButton[] nacsGaugingButtons = new JRadioButton[2];{
   nacsGaugingButtons[0] = new JRadioButton();
   nacsGaugingButtons[1] = new JRadioButton();
 }
 JButton[] nacsGaugingUDButtons = new JButton[2]; {
   nacsGaugingUDButtons[0] = new JButton();
   nacsGaugingUDButtons[1] = new JButton();
 }
 // numbers of episodes monitored
 JPanel episodePanel = new JPanel();
 JLabel episodeLabel = new JLabel();
 JLabel acsTLEpiLabel1 = new JLabel();
 JLabel nacsTLEpiLabel1 = new JLabel();
 JLabel acsBLEpiLabel1 = new JLabel();
 JLabel nacsBLEpiLabel1 = new JLabel();
 JLabel acsTLEpiLabel2 = new JLabel();
 JLabel nacsTLEpiLabel2 = new JLabel();
 JLabel acsBLEpiLabel2 = new JLabel();
 JLabel nacsBLEpiLabel2 = new JLabel();
 JTextField acsTLEpiField = new JTextField();
 JTextField nacsTLEpiField = new JTextField();
 JTextField acsBLEpiField = new JTextField();
 JTextField nacsBLEpiField = new JTextField();
 // definition of episodes
 JPanel epiDefPanel = new JPanel();
 JLabel epiDefLabel = new JLabel();
 JRadioButton[] epiDefButtons = new JRadioButton[2];{
   epiDefButtons[0] = new JRadioButton();
   epiDefButtons[1] = new JRadioButton();
 }
 JButton[] epiDefUDButtons = new JButton[2]; {
   epiDefUDButtons[0] = new JButton();
   epiDefUDButtons[1] = new JButton();
 }
 JLabel stepNumLabel = new JLabel();
 JTextField stepNumField = new JTextField();
 // cross-level integration in the MCS.
 JPanel mcsIntegrationPanel = new JPanel();
 JLabel mcsIntegrationLabel = new JLabel();
 JLabel acsTLMcsIntegrationLabel = new JLabel();
 JLabel nacsTLMcsIntegrationLabel = new JLabel();
 JLabel acsBLMcsIntegrationLabel = new JLabel();
 JLabel nacsBLMcsIntegrationLabel = new JLabel();
 JTextField acsTLMcsIntegrationField = new JTextField();
 JTextField nacsTLMcsIntegrationField = new JTextField();
 JTextField acsBLMcsIntegrationField = new JTextField();
 JTextField nacsBLMcsIntegrationField = new JTextField();
 // return buttons.
 JButton returnButton = new JButton();
 OptionDims[] optDims = new OptionDims[2];

 public SelectionMCS( JFrame superFrame, int width, int height, TransferManager manager )
 {
    super( superFrame, width, height, manager );
    initGui();
 }

 /** initialize the GUI components in this frame. */
 public void initGui()
 {
    options = manager.getOptions( Global.SUPER, Global.SUPER_MCS, 0, 0, 0, 0 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    setSize( 960, 720 );
    // caption.
    initLabel( capLabel,"META-COGNITIVE   SUBSYSTEM   OPTIONS", new java.awt.Font("Dialog", 1, 18),
               SwingConstants.CENTER, new Rectangle(250, 0, 450, 26));
    // reinforcement function.
    initPanel( funcPanel, BORDER, null, new Rectangle(10, 30, 320, 90) );
    String[] funcUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.REIN_FUNC ),
                             TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.REIN_FUNC )
                           };
    Rectangle[] funcUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( funcPanel, funcUDButtons, funcUDRects, funcUDTexts );
    String[] funcTexts = onOffTexts;
    Rectangle[] funcRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( funcPanel, funcLabel, "reinforcement function", new Rectangle(4, 4, 200, 17),
                     funcButtons, funcTexts, funcRects, options[REIN_FUNC], 0, funcUDButtons );
    // goal setting based on drives.
    initPanel( goalPanel, BORDER, null, new Rectangle(10, 130, 320, 90) );
    String[] goalUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.GOAL_SETTING ),
                             TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.GOAL_SETTING )
                           };
    Rectangle[] goalUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( goalPanel, goalUDButtons, goalUDRects, goalUDTexts );
    String[] goalTexts = onOffTexts;
    Rectangle[] goalRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( goalPanel, goalLabel, "goal setting based on drives", new Rectangle(4, 4, 200, 17),
                     goalButtons, goalTexts, goalRects, options[GS_DRIVE], 0, goalUDButtons );
    // input dimension selection.
    initPanel( inputDimPanel, BORDER, null, new Rectangle(10, 230, 320, 190) );
    String[] acsInputDimUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_IN_DIM ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_IN_DIM )
                           };
    Rectangle[] acsInputDimUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( inputDimPanel, acsInputDimUDButtons, acsInputDimUDRects, acsInputDimUDTexts );
    String[] acsInputDimTexts = onOffTexts;
    Rectangle[] acsInputDimRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( inputDimPanel, acsInputDimLabel, "ACS input dimension selection",
        new Rectangle(4, 4, 200, 17), acsInputDimButtons, acsInputDimTexts, acsInputDimRects,
        options[ACS_IN_DIM], 0, acsInputDimUDButtons );
    String[] nacsInputDimUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.NACS_IN_DIM ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.NACS_IN_DIM )
                           };
    Rectangle[] nacsInputDimUDRects = { new Rectangle(24, 144, 120, 17), new Rectangle(150, 144, 120, 17) };
    initUserDefinedBlock( inputDimPanel, nacsInputDimUDButtons, nacsInputDimUDRects, nacsInputDimUDTexts );
    String[] nacsInputDimTexts = onOffTexts;
    Rectangle[] nacsInputDimRects = { new Rectangle(8, 124, 200, 17), new Rectangle(8, 170, 120, 17)};
    initButtonBlock( inputDimPanel, nacsInputDimLabel, "NACS input dimension selection",
        new Rectangle(4, 104, 200, 17), nacsInputDimButtons, nacsInputDimTexts, nacsInputDimRects,
        options[NACS_IN_DIM], 0, nacsInputDimUDButtons);
    // output dimension selection.
    initPanel( outputDimPanel, BORDER, null, new Rectangle(10, 430, 320, 190) );
    String[] acsOutputDimUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_IN_DIM ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_IN_DIM )
                           };
    Rectangle[] acsOutputDimUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( outputDimPanel, acsOutputDimUDButtons, acsOutputDimUDRects, acsOutputDimUDTexts );
    String[] acsOutputDimTexts = onOffTexts;
    Rectangle[] acsOutputDimRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( outputDimPanel, acsOutputDimLabel, "ACS output dimension selection",
        new Rectangle(4, 4, 200, 17), acsOutputDimButtons, acsOutputDimTexts, acsOutputDimRects,
        options[ACS_OUT_DIM], 0, acsOutputDimUDButtons );
    String[] nacsOutputDimUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.NACS_IN_DIM ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.NACS_IN_DIM )
                           };
    Rectangle[] nacsOutputDimUDRects = { new Rectangle(24, 144, 120, 17), new Rectangle(150, 144, 120, 17) };
    initUserDefinedBlock( outputDimPanel, nacsOutputDimUDButtons, nacsOutputDimUDRects, nacsOutputDimUDTexts );
    String[] nacsOutputDimTexts = onOffTexts;
    Rectangle[] nacsOutputDimRects = { new Rectangle(8, 124, 200, 17), new Rectangle(8, 170, 120, 17)};
    initButtonBlock( outputDimPanel, nacsOutputDimLabel, "NACS output dimension selection",
        new Rectangle(4, 104, 200, 17), nacsOutputDimButtons, nacsOutputDimTexts, nacsOutputDimRects,
        options[NACS_OUT_DIM], 0, nacsOutputDimUDButtons);

    // input value selection.
    initPanel( inputValPanel, BORDER, null, new Rectangle(10, 630, 320, 190) );
    String[] acsInputValUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_IN_VAL ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_IN_VAL )
                           };
    Rectangle[] acsInputValUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( inputValPanel, acsInputValUDButtons, acsInputValUDRects, acsInputValUDTexts );
    String[] acsInputValTexts = onOffTexts;
    Rectangle[] acsInputValRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( inputValPanel, acsInputValLabel, "ACS input value selection",
        new Rectangle(4, 4, 200, 17), acsInputValButtons, acsInputValTexts, acsInputValRects,
        options[ACS_IN_VAL], 0, acsInputValUDButtons );
    String[] nacsInputValUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.NACS_IN_VAL ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.NACS_IN_VAL )
                           };
    Rectangle[] nacsInputValUDRects = { new Rectangle(24, 144, 120, 17), new Rectangle(150, 144, 120, 17) };
    initUserDefinedBlock( inputValPanel, nacsInputValUDButtons, nacsInputValUDRects, nacsInputValUDTexts );
    String[] nacsInputValTexts = onOffTexts;
    Rectangle[] nacsInputValRects = { new Rectangle(8, 124, 200, 17), new Rectangle(8, 170, 120, 17)};
    initButtonBlock( inputValPanel, nacsInputValLabel, "NACS input value selection",
        new Rectangle(4, 104, 200, 17), nacsInputValButtons, nacsInputValTexts, nacsInputValRects,
        options[NACS_IN_VAL], 0, nacsInputValUDButtons);
    // output value selection.
    initPanel( outputValPanel, BORDER, null, new Rectangle(10, 830, 320, 190) );
    String[] acsOutputValUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_IN_VAL ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_IN_VAL )
                           };
    Rectangle[] acsOutputValUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( outputValPanel, acsOutputValUDButtons, acsOutputValUDRects, acsOutputValUDTexts );
    String[] acsOutputValTexts = onOffTexts;
    Rectangle[] acsOutputValRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( outputValPanel, acsOutputValLabel, "ACS output value selection",
        new Rectangle(4, 4, 200, 17), acsOutputValButtons, acsOutputValTexts, acsOutputValRects,
        options[ACS_OUT_VAL], 0, acsOutputValUDButtons );
    String[] nacsOutputValUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.NACS_IN_VAL ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.NACS_IN_VAL )
                           };
    Rectangle[] nacsOutputValUDRects = { new Rectangle(24, 144, 120, 17), new Rectangle(150, 144, 120, 17) };
    initUserDefinedBlock( outputValPanel, nacsOutputValUDButtons, nacsOutputValUDRects, nacsOutputValUDTexts );
    String[] nacsOutputValTexts = onOffTexts;
    Rectangle[] nacsOutputValRects = { new Rectangle(8, 124, 200, 17), new Rectangle(8, 170, 120, 17)};
    initButtonBlock( outputValPanel, nacsOutputValLabel, "NACS output value selection",
        new Rectangle(4, 104, 200, 17), nacsOutputValButtons, nacsOutputValTexts, nacsOutputValRects,
        options[NACS_OUT_VAL], 0, nacsOutputValUDButtons);

    // selection of Learning methods.
    initPanel( learningPanel, BORDER, null, new Rectangle(340, 30, 320, 190) );
    String[] acsLearningUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_LEARN ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_LEARN )
                           };
    Rectangle[] acsLearningUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( learningPanel, acsLearningUDButtons, acsLearningUDRects, acsLearningUDTexts );
    String[] acsLearningTexts = onOffTexts;
    Rectangle[] acsLearningRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( learningPanel, acsLearningLabel, "ACS learning method",
        new Rectangle(4, 4, 200, 17), acsLearningButtons, acsLearningTexts, acsLearningRects,
        options[ACS_LEARN], 0, acsLearningUDButtons );
    String[] nacsLearningUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.NACS_LEARN ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.NACS_LEARN )
                           };
    Rectangle[] nacsLearningUDRects = { new Rectangle(24, 144, 120, 17), new Rectangle(150, 144, 120, 17) };
    initUserDefinedBlock( learningPanel, nacsLearningUDButtons, nacsLearningUDRects, nacsLearningUDTexts );
    String[] nacsLearningTexts = onOffTexts;
    Rectangle[] nacsLearningRects = { new Rectangle(8, 124, 200, 17), new Rectangle(8, 170, 120, 17)};
    initButtonBlock( learningPanel, nacsLearningLabel, "NACS learning method",
        new Rectangle(4, 104, 200, 17), nacsLearningButtons, nacsLearningTexts, nacsLearningRects,
        options[NACS_LEARN], 0, nacsLearningUDButtons);
    // selection of reasoning methods.
    initPanel( reasoningPanel, BORDER, null, new Rectangle(340, 230, 320, 190) );
    String[] acsReasoningUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_REASON ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_REASON )
                           };
    Rectangle[] acsReasoningUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( reasoningPanel, acsReasoningUDButtons, acsReasoningUDRects, acsReasoningUDTexts );
    String[] acsReasoningTexts = onOffTexts;
    Rectangle[] acsReasoningRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( reasoningPanel, acsReasoningLabel, "ACS Reasoning method",
        new Rectangle(4, 4, 200, 17), acsReasoningButtons, acsReasoningTexts, acsReasoningRects,
        options[ACS_REASON], 0, acsReasoningUDButtons );
    String[] nacsReasoningUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.NACS_REASON ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.NACS_REASON )
                           };
    Rectangle[] nacsReasoningUDRects = { new Rectangle(24, 144, 120, 17), new Rectangle(150, 144, 120, 17) };
    initUserDefinedBlock( reasoningPanel, nacsReasoningUDButtons, nacsReasoningUDRects, nacsReasoningUDTexts );
    String[] nacsReasoningTexts = onOffTexts;
    Rectangle[] nacsReasoningRects = { new Rectangle(8, 124, 200, 17), new Rectangle(8, 170, 120, 17)};
    initButtonBlock( reasoningPanel, nacsReasoningLabel, "NACS Reasoning method",
        new Rectangle(4, 104, 200, 17), nacsReasoningButtons, nacsReasoningTexts, nacsReasoningRects,
        options[NACS_REASON], 0, nacsReasoningUDButtons);
    // setting of parameter
    initPanel( paramSettingPanel, BORDER, null, new Rectangle(340, 430, 320, 190) );
    String[] acsParamSettingUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_PARAM ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_PARAM )
                           };
    Rectangle[] acsParamSettingUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( paramSettingPanel, acsParamSettingUDButtons, acsParamSettingUDRects, acsParamSettingUDTexts );
    String[] acsParamSettingTexts = onOffTexts;
    Rectangle[] acsParamSettingRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( paramSettingPanel, acsParamSettingLabel, "ACS Parameter setting ",
        new Rectangle(4, 4, 200, 17), acsParamSettingButtons, acsParamSettingTexts, acsParamSettingRects,
        options[ACS_PARAM], 0, acsParamSettingUDButtons );
    String[] nacsParamSettingUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.NACS_PARAM ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.NACS_PARAM )
                           };
    Rectangle[] nacsParamSettingUDRects = { new Rectangle(24, 144, 120, 17), new Rectangle(150, 144, 120, 17) };
    initUserDefinedBlock( paramSettingPanel, nacsParamSettingUDButtons, nacsParamSettingUDRects, nacsParamSettingUDTexts );
    String[] nacsParamSettingTexts = onOffTexts;
    Rectangle[] nacsParamSettingRects = { new Rectangle(8, 124, 200, 17), new Rectangle(8, 170, 120, 17)};
    initButtonBlock( paramSettingPanel, nacsParamSettingLabel, "NACS Parameter Setting ",
        new Rectangle(4, 104, 200, 17), nacsParamSettingButtons, nacsParamSettingTexts, nacsParamSettingRects,
        options[NACS_PARAM], 0, nacsParamSettingUDButtons);
    // cross-level integration.
    initPanel( clIntegrationPanel, BORDER, null, new Rectangle(340, 630, 320, 100) );
    String[] acsCLIntegrationUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_CL ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_CL )
                           };
    Rectangle[] acsCLIntegrationUDRects = { new Rectangle(24, 44, 120, 17), new Rectangle(150, 44, 120, 17) };
    initUserDefinedBlock( clIntegrationPanel, acsCLIntegrationUDButtons, acsCLIntegrationUDRects, acsCLIntegrationUDTexts );
    String[] acsCLIntegrationTexts = onOffTexts;
    Rectangle[] acsCLIntegrationRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 70, 120, 17)};
    initButtonBlock( clIntegrationPanel, acsCLIntegrationLabel, "ACS cross-level integration",
        new Rectangle(4, 4, 200, 17), acsCLIntegrationButtons, acsCLIntegrationTexts, acsCLIntegrationRects,
        options[ACS_CL], 0, acsCLIntegrationUDButtons );
    // regulation of cross-level integration in ACS.
    initPanel( clRegulationPanel, BORDER, null, new Rectangle(340, 740, 320, 80) );
    String[] acsCLRegulationTexts = { "through P parameters directly", "through beta parameters indirectly"};
    Rectangle[] acsCLRegulationRects = { new Rectangle(8, 28, 300, 17), new Rectangle(8, 48, 300, 17)};
    initButtonBlock( clRegulationPanel, acsCLRegulationLabel, "regulation of cross-level integration in ACS",
        new Rectangle(4, 4, 300, 17), acsCLRegulationButtons, acsCLRegulationTexts, acsCLRegulationRects,
        options[ACS_REGU] );
    // Timing of goal setting
    initPanel( goalTimingPanel, BORDER, null, new Rectangle(340, 830, 320, 190) );
    String[] goalTimingTexts = { "drive threshold", "drive difference threshold" };
    Rectangle[] goalTimingRects = { new Rectangle(8, 28, 200, 17), new Rectangle(8, 88, 200, 17)};
    initButtonBlock( goalTimingPanel, goalTimingLabel, "timing of goal setting",
       new Rectangle(4, 4, 200, 17), goalTimingButtons, goalTimingTexts, goalTimingRects, options[GS_TIME]);
    initLabel( thLabel1, "threshold_d1", new Rectangle(24, 48, 80, 17));
    initField( thField1, String.valueOf(options[GS_TIME].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(104, 48, 32, 17), 4, options[GS_TIME], 0, 0 );
    initLabel( thLabel2, "threshold_d2", new Rectangle(24, 108, 80, 17));
    initField( thField2, String.valueOf(options[GS_TIME].getOptionItem(1).getParameter(0).getDoubleValue()),
               new Rectangle(104, 108, 32, 17), 4, options[GS_TIME], 1, 0 );
    goalTimingPanel.add( thLabel1 );
    goalTimingPanel.add( thLabel2 );
    goalTimingPanel.add( thField1 );
    goalTimingPanel.add( thField2 );
    // measures of performance.
    initPanel( performancePanel, BORDER, null, new Rectangle(670, 30, 320, 190) );
    String[] acsPerformanceUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_PERFORM ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_PERFORM )
                           };
    Rectangle[] acsPerformanceUDRects = { new Rectangle(24, 70, 120, 17), new Rectangle(150, 70, 120, 17) };
    initUserDefinedBlock( performancePanel, acsPerformanceUDButtons, acsPerformanceUDRects, acsPerformanceUDTexts );
    String[] acsPerformanceTexts = { "default: rs", "user defines" };
    Rectangle[] acsPerformanceRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 50, 120, 17)};
    initButtonBlock( performancePanel, acsPerformanceLabel, "ACS performance measure",
        new Rectangle(4, 4, 200, 17), acsPerformanceButtons, acsPerformanceTexts, acsPerformanceRects,
        options[ACS_PERFORM], 1, acsPerformanceUDButtons );
    String[] nacsPerformanceUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.NACS_PERFORM ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.NACS_PERFORM )
                           };
    Rectangle[] nacsPerformanceUDRects = { new Rectangle(24, 170, 120, 17), new Rectangle(150, 170, 120, 17) };
    initUserDefinedBlock( performancePanel, nacsPerformanceUDButtons, nacsPerformanceUDRects, nacsPerformanceUDTexts );
    String[] nacsPerformanceTexts = { "default: rs", "user defines" };
    Rectangle[] nacsPerformanceRects = { new Rectangle(8, 124, 200, 17), new Rectangle(8, 150, 120, 17)};
    initButtonBlock( performancePanel, nacsPerformanceLabel, "NACS performance measure",
        new Rectangle(4, 104, 200, 17), nacsPerformanceButtons, nacsPerformanceTexts, nacsPerformanceRects,
        options[NACS_PERFORM], 1, nacsPerformanceUDButtons);

    // measures for gauging learning
    initPanel( gaugingPanel, BORDER, null, new Rectangle(670, 230, 320, 190) );
    String[] acsGaugingUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.ACS_GAUGE ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.ACS_GAUGE )
                           };
    Rectangle[] acsGaugingUDRects = { new Rectangle(24, 70, 120, 17), new Rectangle(150, 70, 120, 17) };
    initUserDefinedBlock( gaugingPanel, acsGaugingUDButtons, acsGaugingUDRects, acsGaugingUDTexts );
    String[] acsGaugingTexts = { "default: sr", "user defines" };
    Rectangle[] acsGaugingRects = { new Rectangle(8, 24, 200, 17), new Rectangle(8, 50, 120, 17)};
    initButtonBlock( gaugingPanel, acsGaugingLabel, "ACS gauging learning measure",
        new Rectangle(4, 4, 200, 17), acsGaugingButtons, acsGaugingTexts, acsGaugingRects,
        options[ACS_GAUGE], 1, acsGaugingUDButtons );
    String[] nacsGaugingUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.NACS_GAUGE ),
                                 TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.NACS_GAUGE )
                           };
    Rectangle[] nacsGaugingUDRects = { new Rectangle(24, 170, 120, 17), new Rectangle(150, 170, 120, 17) };
    initUserDefinedBlock( gaugingPanel, nacsGaugingUDButtons, nacsGaugingUDRects, nacsGaugingUDTexts );
    String[] nacsGaugingTexts = { "default: sr", "user defines" };
    Rectangle[] nacsGaugingRects = { new Rectangle(8, 124, 200, 17), new Rectangle(8, 150, 120, 17)};
    initButtonBlock( gaugingPanel, nacsGaugingLabel, "NACS gauging learning measure",
        new Rectangle(4, 104, 200, 17), nacsGaugingButtons, nacsGaugingTexts, nacsGaugingRects,
        options[NACS_GAUGE], 1, nacsGaugingUDButtons);
    // numbers of conclusion monitored.
    initPanel( conclusionPanel, BORDER, null, new Rectangle(670, 430, 320, 190) );
    initLabel( conclusionLabel, "numbers of conclusion monitored", new Rectangle(4, 4, 280, 17));
    initLabel( acsTLConcLabel1, "number of top conclusions monitored in the top level",
              new Rectangle(4, 28, 310, 17));
    initLabel( acsTLConcLabel2, "of ACS", new Rectangle(4, 48, 50, 17));
    initField( acsTLConcField,
               String.valueOf(options[ACS_CONC_NUM].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(54, 48, 32, 17), 4, options[ACS_CONC_NUM], 0, 0 );
    initLabel( nacsTLConcLabel1, "number of top conclusions monitored in the top level",
               new Rectangle(4, 68, 310, 17));
    initLabel( nacsTLConcLabel2, "of NACS", new Rectangle(4, 88, 60, 17));
    initField( nacsTLConcField,
               String.valueOf(options[NACS_CONC_NUM].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(64, 88, 32, 17), 4, options[NACS_CONC_NUM], 0, 0 );
    initLabel( acsBLConcLabel1, "number of top conclusions monitored in the bottom",
               new Rectangle(4, 108, 310, 17));
    initLabel( acsBLConcLabel2, "level of ACS", new Rectangle(4, 128, 80, 17) );
    initField( acsBLConcField,
               String.valueOf(options[ACS_CONC_NUM].getOptionItem(0).getParameter(1).getIntValue()),
               new Rectangle(84, 128, 32, 17), 4, options[ACS_CONC_NUM], 0, 1 );
    initLabel( nacsBLConcLabel1, "number of top conclusions monitored in the bottom",
               new Rectangle(4, 148, 310, 17));
    initLabel( nacsBLConcLabel2, "level of NACS", new Rectangle(4, 168, 80, 17));
    initField( nacsBLConcField,
               String.valueOf(options[NACS_CONC_NUM].getOptionItem(0).getParameter(1).getIntValue()),
               new Rectangle(84, 168, 32, 17), 4, options[NACS_CONC_NUM], 0, 1 );
    conclusionPanel.add( conclusionLabel );
    conclusionPanel.add( acsTLConcLabel1 );
    conclusionPanel.add( acsBLConcLabel1 );
    conclusionPanel.add( nacsTLConcLabel1 );
    conclusionPanel.add( nacsBLConcLabel1 );
    conclusionPanel.add( acsTLConcLabel2 );
    conclusionPanel.add( acsBLConcLabel2 );
    conclusionPanel.add( nacsTLConcLabel2 );
    conclusionPanel.add( nacsBLConcLabel2 );
    conclusionPanel.add( acsTLConcField );
    conclusionPanel.add( acsBLConcField );
    conclusionPanel.add( nacsTLConcField );
    conclusionPanel.add( nacsBLConcField );
    // numbers of episode monitored.
    initPanel( episodePanel, BORDER, null, new Rectangle(670, 630, 320, 190) );
    initLabel( episodeLabel, "numbers of episode monitored", new Rectangle(4, 4, 280, 17));
    initLabel( acsTLEpiLabel1, "number of previous episodes monitored in the top",
               new Rectangle(4, 28, 310, 17));
    initLabel( acsTLEpiLabel2, "level of ACS", new Rectangle(4, 48, 80, 17));
    initField( acsTLEpiField,
               String.valueOf(options[ACS_EPI_NUM].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(84, 48, 32, 17), 4, options[ACS_EPI_NUM], 0, 0 );
    initLabel( nacsTLEpiLabel1, "number of previous episodes monitored in the top",
               new Rectangle(4, 68, 310, 17));
    initLabel( nacsTLEpiLabel2, "level of NACS", new Rectangle(4, 88, 80, 17));
    initField( nacsTLEpiField,
               String.valueOf(options[NACS_EPI_NUM].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(84, 88, 32, 17), 4, options[NACS_EPI_NUM], 0, 0 );
    initLabel( acsBLEpiLabel1, "number of previous episodes monitored in the bottom",
               new Rectangle(4, 108, 310, 17));
    initLabel( acsBLEpiLabel2, "level of ACS", new Rectangle(4, 128, 80, 17));
    initField( acsBLEpiField,
               String.valueOf(options[ACS_EPI_NUM].getOptionItem(0).getParameter(1).getIntValue()),
               new Rectangle(84, 128, 32, 17), 4, options[ACS_EPI_NUM], 0, 1 );
    initLabel( nacsBLEpiLabel1, "number of previous episodes monitored in the bottom",
               new Rectangle(4, 148, 310, 17));
    initLabel( nacsBLEpiLabel2, "level of NACS", new Rectangle(4, 168, 80, 17));
    initField( nacsBLEpiField,
               String.valueOf(options[NACS_EPI_NUM].getOptionItem(0).getParameter(1).getIntValue()),
               new Rectangle(84, 168, 32, 17), 4, options[NACS_EPI_NUM], 0, 1 );
    episodePanel.add( episodeLabel );
    episodePanel.add( acsTLEpiLabel1 );
    episodePanel.add( acsBLEpiLabel1 );
    episodePanel.add( nacsTLEpiLabel1 );
    episodePanel.add( nacsBLEpiLabel1 );
    episodePanel.add( acsTLEpiLabel2 );
    episodePanel.add( acsBLEpiLabel2 );
    episodePanel.add( nacsTLEpiLabel2 );
    episodePanel.add( nacsBLEpiLabel2 );
    episodePanel.add( acsTLEpiField );
    episodePanel.add( acsBLEpiField );
    episodePanel.add( nacsTLEpiField );
    episodePanel.add( nacsBLEpiField );
    // definition of episodes.
    initPanel( epiDefPanel, BORDER, null, new Rectangle(670, 830, 320, 110) );
    String[] epiDefTexts = { "by number of steps", "by the termination of one iteration of hte task" };
    Rectangle[] epiDefRects = { new Rectangle(8, 28, 160, 17), new Rectangle(8, 78, 310, 17)};
    initButtonBlock( epiDefPanel, epiDefLabel, "definition of episodes", new Rectangle(4, 4, 200, 17),
                     epiDefButtons, epiDefTexts, epiDefRects, options[EPI_DEF] );
    initLabel( stepNumLabel, "the number of steps", new Rectangle(24, 48, 140, 17));
    initField( stepNumField, String.valueOf(options[EPI_DEF].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(150, 48, 32, 17), 4, options[EPI_DEF], 0, 0 );
    epiDefPanel.add( stepNumLabel );
    epiDefPanel.add( stepNumField );
    // cross-level integration in the MCS.
    initPanel( mcsIntegrationPanel, BORDER, null, new Rectangle(670, 950, 320, 70) );
    initLabel( mcsIntegrationLabel, "cross-level integration in the MCS", new Rectangle(4, 4, 280, 17));
    initLabel( acsTLMcsIntegrationLabel, "P_TL (ACS) = ", new Rectangle(12, 28, 100, 17));
    initField( acsTLMcsIntegrationField,
               String.valueOf(options[ACS_INTEG].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(100, 28, 32, 17), 4, options[ACS_INTEG], 0, 0 );
    initLabel( acsBLMcsIntegrationLabel, "P_BL (ACS) = ",   new Rectangle(160, 28, 100, 17));
    initField( acsBLMcsIntegrationField,
               String.valueOf(options[ACS_INTEG].getOptionItem(0).getParameter(1).getDoubleValue()),
               new Rectangle(250, 28, 32, 17), 4, options[ACS_INTEG], 0, 1 );
    initLabel( nacsTLMcsIntegrationLabel, "P_TL (NACS) = ",  new Rectangle(12, 48, 100, 17));
    initField( nacsTLMcsIntegrationField,
               String.valueOf(options[NACS_INTEG].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(100, 48, 32, 17), 4, options[NACS_INTEG], 0, 0 );
    initLabel( nacsBLMcsIntegrationLabel, "P_BL (NACS) = ",  new Rectangle(160, 48, 100, 17));
    initField( nacsBLMcsIntegrationField,
               String.valueOf(options[NACS_INTEG].getOptionItem(0).getParameter(1).getDoubleValue()),
               new Rectangle(250, 48, 32, 17), 4, options[NACS_INTEG], 0, 1 );
    mcsIntegrationPanel.add( mcsIntegrationLabel );
    mcsIntegrationPanel.add( acsTLMcsIntegrationLabel );
    mcsIntegrationPanel.add( acsBLMcsIntegrationLabel );
    mcsIntegrationPanel.add( nacsTLMcsIntegrationLabel );
    mcsIntegrationPanel.add( nacsBLMcsIntegrationLabel );
    mcsIntegrationPanel.add( acsTLMcsIntegrationField );
    mcsIntegrationPanel.add( acsBLMcsIntegrationField );
    mcsIntegrationPanel.add( nacsTLMcsIntegrationField );
    mcsIntegrationPanel.add( nacsBLMcsIntegrationField );
    // return buttons.
    initReturnButton( returnButton, "RETURN", new Rectangle(840, 1040, 110, 29));
    // add components.
    parent.add(capLabel, null);
    parent.add(funcPanel, null);
    parent.add(goalPanel, null);
    parent.add(inputDimPanel, null);
    parent.add(outputDimPanel, null);
    parent.add(inputValPanel, null);
    parent.add(outputValPanel, null);
    parent.add(learningPanel, null);
    parent.add(reasoningPanel, null);
    parent.add(clIntegrationPanel, null);
    parent.add(paramSettingPanel, null);
    parent.add(clRegulationPanel, null);
    parent.add(goalTimingPanel, null);
    parent.add(performancePanel, null);
    parent.add(conclusionPanel, null);
    parent.add(gaugingPanel, null);
    parent.add(episodePanel, null);
    parent.add(epiDefPanel, null);
    parent.add(mcsIntegrationPanel, null);
    parent.add(returnButton, null);

    update();
 }

 public void update()
 {
 }
}
