package clarion.options.transfer.subsys;
import java.io.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the process of transferring Response Time configuration
*  data between GUI and <tt>Global</tt> object or persistent storage.
 * <li> important methods <br>
 * <ol>
 * <li> public void load(): implement the process of loading options from persistent
 * storage.
 * <li> public void save(): implement the process of save the options to persistent
 * storage or to the associated <tt>Global</tt> object.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model. <br>
 * This class is part of data transfer subsystem and is not related directly to
 * CLARION model.
 * <li> Usage of this class. <br>
 * It extends the base <tt>Transfer</t>> class and is instantiated in the class
 * <em>TransferManager</em>.
 * </ul>
 * @author Xi Zhang
 *
 **/

public class RtTransfer extends Transfer {

  /** constructor. instantiated with the associated <tt>TransferManager</tt> object
   * and <tt>Global</tt> object. */
  public RtTransfer( TransferManager manager, Global global )
  {
     super(manager, global );
  }

  /** implement the process of loading options from persistent storage. */
  public void load()
  {
     loadedFileName = "rt.txt";
     String path = global.getSettingDir() + "/DEFAULT/subjectGroup0/rt/";
     sysOptions = new Option[1][1][1][1][1][];
     sysOptions[0][0][0][0][0] = loadOptions( path );
     subsysPath = this.searchFile( "rt" );
     userOptions = new Option[1][1][1][1][1][];
     userOptions[0][0][0][0][0] = loadOptions( subsysPath );
  }

  /** implement the process of save the options to persistent storage or to the
   * associated <tt>Global</tt> object. */
  public void save()
  {
     if( saveCode == SAVE_TO_GLOBAL ){
         initGlobalRT();
         return;
     }
     // create RT setting directory.
     String path = global.getSettingDir() + "/" + global.getTaskName()
                   + "/" + global.getTaskSettingName() + "/rt";
     new File( path ).mkdir();
     savedFileName = "/rt.txt";
     try{
        new File( path + savedFileName ).createNewFile();
     }
     catch( Exception e ){
        System.out.println( e );
     }
     // save RT setting.
     saveOptions( path, userOptions[0][0][0][0][0] );
  }

  /**************** init Global Response Time ******************/
  /** initialize the variables relevant to Response Time calculation in the associated
   * <tt>Global</tt> object. */
  private void initGlobalRT()
  {
     final int OVER = 0, OVER_BL = 1, OVER_TL = 2, PT_BL = 3, DT_BL = 4, AT_BL = 5;
     final int PT_TL = 6, DT_TL = 7, AT_TL = 8, ASSOC = 9, CHUNK = 10, AMN_PASS = 11;
     Option[] opts = userOptions[0][0][0][0][0];
     //global.setOverallFormula( String value );
     //global.setBLFormula( String value );
     //global.setTLFormula( String value );
     global.setPT_BL(opts[PT_BL].getOptionItem(0).getParameter(0).getIntValue());
     global.setDT_BL(opts[DT_BL].getOptionItem(0).getParameter(0).getIntValue());
     global.setAT_BL(opts[AT_BL].getOptionItem(0).getParameter(0).getIntValue());
     global.setPT_TL(opts[PT_TL].getOptionItem(0).getParameter(0).getIntValue());
     global.setDT_TL_actionTime(opts[DT_TL].getOptionItem(0).getParameter(0).getIntValue());
     global.setDT_TL_t0(opts[DT_TL].getOptionItem(0).getParameter(1).getDoubleValue());
     global.setDT_TL_t1(opts[DT_TL].getOptionItem(0).getParameter(2).getDoubleValue());
     global.setAT_TL(opts[AT_TL].getOptionItem(0).getParameter(0).getIntValue());
     //global.setAssoAppTimeFormula( String value );
     global.setAssocAppTime_t4(opts[ASSOC].getOptionItem(0).getParameter(0).getDoubleValue());
     global.setAssocAppTime_t5(opts[ASSOC].getOptionItem(0).getParameter(1).getDoubleValue());
     //global.setChunkRetTimeFormula( String value );
     global.setChunkRetTime_t2(opts[CHUNK].getOptionItem(0).getParameter(0).getDoubleValue());
     global.setChunkRetTime_t3(opts[CHUNK].getOptionItem(0).getParameter(1).getDoubleValue());
     global.setAmnOnePassTime(opts[AMN_PASS].getOptionItem(0).getParameter(0).getIntValue());
  }

}
