package clarion.options.transfer.subsys;
import java.io.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the process of transferring selection of CLARION
* subsystems configuration data between GUI and <tt>Global</tt> object or persistent
* storage.
 * <li> important methods <br>
 * <ol>
 * <li> public void load(): implement the process of loading options from persistent
 * storage.
 * <li> public void save(): implement the process of save the options to persistent
 * storage or to the associated <tt>Global</tt> object.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model. <br>
 * This class is part of data transfer subsystem and is not related directly to
 * CLARION model.
 * <li> Usage of this class. <br>
 * It extends the base <tt>Transfer</t>> class and is instantiated in the class
 * <em>TransferManager</em>.
 * </ul>
 * @author Xi Zhang
 *
 **/

public class SubsysTransfer extends Transfer {

  /** constructor. instantiated with the associated <tt>TransferManager</tt> object
   * and <tt>Global</tt> object. */
  public SubsysTransfer( TransferManager manager, Global global )
  {
     super( manager, global );
  }

  /** Check if NACS subsystem is selected or not. */
  public boolean isNacsOn()
  {
     return userOptions[0][0][0][0][0][0].getUserChoice() == ON;
  }

  /** Check if Supervisory subsystem is selected or not. */
  public boolean isSupervisoryOn()
  {
     return userOptions[0][0][0][0][0][1].getUserChoice() == ON;
  }

  /** Check if Response Time is selected or not. */
  public boolean isRtOn()
  {
     return userOptions[0][0][0][0][0][2].getUserChoice() == ON;
  }

  /** implement the process of loading options from persistent storage. */
  public void load()
  {
     loadedFileName = "subsys.txt";
     String path = global.getSettingDir() + "/DEFAULT/subjectGroup0/";
     sysOptions = new Option[1][1][1][1][1][];
     sysOptions[0][0][0][0][0] = loadOptions( path );
     path = global.getSettingDir() + "/" + global.getTaskName()
                   + "/" + global.getTaskSettingName() + "/";
     userOptions = new Option[1][1][1][1][1][];
     userOptions[0][0][0][0][0] = loadOptions( path );
  }

  /** implement the process of save the options to persistent storage or to the
   * associated <tt>Global</tt> object. */
  public void save()
  {
     if( saveCode == SAVE_TO_GLOBAL ){
         initGlobalSubsys();
         return;
     }
     String path = global.getSettingDir() + "/" + global.getTaskName()
                   + "/" + global.getTaskSettingName() + "/";
     savedFileName = "subsys.txt";
     // save subsys setting.
     saveOptions( path, userOptions[0][0][0][0][0] );
  }

/************************** initialize global variables ***********************/
  /** initialize the variables relevant to selection of CLARION subsystems in
   * the associated <tt>Global</tt> object. */
  private void initGlobalSubsys()
  {
    final int SUB_NACS = 0, SUB_SUPER = 1, SUB_RT = 2;
    Option[] opts = userOptions[0][0][0][0][0];
    global.setNacsOn( opts[SUB_NACS].getUserChoice() == ON );
    global.setSupervisorySubsysOn( opts[SUB_SUPER].getUserChoice() == ON );
    global.setRtOn( opts[SUB_RT].getUserChoice() == ON );
  }

}
