package clarion.system;
import java.lang.Class;
import clarion.common.*;
import clarion.acs.*;
import clarion.nacs.*;
import clarion.ms.*;
import clarion.mcs.*;
import clarion.acs.gs.*;
import clarion.acs.wm.*;
import clarion.tools.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to manage all of the parameters or variables (data structures) globally
 * used in the CLARION system. To know the meaning of the variables, please see the appendix
 * of the tutorial on Options and Parameters.
 *
 * Note: For user-defined features, the system auppose the feature order of each kind
 * determines the location in the corresponding I/O data structures.
 *
 * <li> summary of the important methods. <br>
 * Most methods of this class are Get and Set methods. The left is some initializeation routines
 * for ACS networks, NACS networks and ACS action rules.
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is very important to the CLARION system and it manages all of the variables and
 * communicates with all of the components of CLARIN model. By communicating with <tt>this</tt>
 * class, all of the components of CLARION model can get the requested parameters/variables
 * information.
 * <li> Usage of <tt>this</tt> class. <br>
 * <tt>this</tt> Class is called by all of the components of CLARION model to get the requested
 * parameters/variables information. And it is instantiated inside a <tt>TaskAgent</tt> object.
 * </ul>
 * @author Xi Zhang
 */
public class Global{

 /** class loader. */
 ClarionClassLoader classLoader;
 /*============================ global constants. =============================*/
 /****************** constants for parameter types ****************************/
 final public static int PRIM_TYPE = 0, ARRAY_TYPE = 1, BOOL = 0, INT = 1, DOUBLE = 2, STRING = 3;
 /** delimiter */
 final public static String VALUE_DELIM = ",;", TABLE_STR = "  ";
 /****************** constants for option settings ****************************/
 final public static int MAX_TASK_NUM = 20, MAX_SETTING_NUM = 20, MAX_RUNNING_AGENT_NUM = 100;
 final public static int SAVE_TO_GLOBAL = 0, SAVE_TO_FILES = 1, CLARION_OPTIONS = 0, TASK_OPTIONS = 1;
 final public static int DIM_SPEC_SYS = 0, DIM_SPEC_TYPE = 1, DIM_SPEC_INDEX = 2;
 /************ constants for user interface.*********************/
 final public static int TOTOAL_SUBSYS_NUM = 8, CLARION_SUBSYS_NUM = 6, TASK_SUBSYS_NUM = 2,
 ACS = 0, NACS = 1, SUPER = 2, RT = 3, INTERFACE = 4, DIMS = 5, TS = 6, SUBSYS = 7,
 // indices for major ACS options.
 ACS_COMP_NUM = 4, ACS_COMP = 0, ACS_NET = 1, ACS_WM = 2, ACS_GS = 3,
 ACS_NET_OPT_NUM = 4, EX_NET = 0, WM_NET = 1, GS_NET = 2, ACS_COR = 3,
 ACS_NET_COMP_NUM = 5, ACS_IDN = 0, ACS_RER = 1, ACS_IRL = 2, ACS_FR = 3, ACS_BASIC = 4,
 MAX_EX_NET_NUM = 4, MAX_RULE_NUMS[] = { 1, 1, 15, 15 },
 // indices for major NACS options.
 NACS_COMP_NUM = 6, NACS_COMP = 0, NACS_GKS = 1, NACS_AMN = 2, NACS_EM = 3, NACS_AEM = 4, NACS_COR = 5,
 MAX_AMN_NUM = 2, SUPER_COMP_NUM = 5, SUPER_COMP = 0, SUPER_MS = 1, SUPER_MCS = 2;
 // indices for major supervisory options.
 // indices for major RT options.
 /** constants for VCG group ( ACS action format ). */
 final public static int SINGLE_ACTIVE_VALUED = 0, MULTI_ACTIVE_VALUED = 1, ELEM_INACTIVE = 0, ELEM_ACTIVE = 1;
 /** constants for the format of ACS action controlling NACS. */
 final public static int D_TYPE = 0, D_ACT = 1, D_STAGE = 2, D_INFER = 3, D_REPORT = 4,
 D_CUE = 5, D_OUTPUT = 6, D_TIME = 7, D_WM = 8, DO_NOTHING = 0, RETRIEVAL_ONE = 1, RETRIEVAL_ALL = 2;
 /** constants for feature types. */
 final public static int FEAT_TYPE_NUM = 4, FT_SYS = 0, FT_INPUT = 1, FT_OUTPUT = 2, FT_GS = 3;
 /** constants for output dimension numbers. */
 final public static int ACT_DIM_NUM_WM = 2, ACT_DIM_NUM_NACS_CONTROL = 18;
 /** SYSTEM-defined features. */
 final public static int NACS_CONTROL_DIM_START = 5;
 final public static int FEAT_SYS_NUM = 23,
 FT_SYS_GS_ACT = 0, FT_SYS_WM_FLAG = 1, FT_SYS_WM_ACT = 2, FT_SYS_WM_ACT_PARA = 3,
 FT_SYS_REWARD = 4, FT_SYS_NACS_ACT = 5, FT_SYS_NACS_LEVEL = 6, FT_SYS_NACS_ENCODE = 7,
 FT_SYS_NACS_ASSIM = 8, FT_SYS_NACS_INFER = 9, FT_SYS_NACS_SET = 10,
 FT_SYS_NACS_IN_IN = 11, FT_SYS_NACS_IN_OUT = 12, FT_SYS_NACS_OUT_IN = 13,
 FT_SYS_NACS_OUT_OUT = 14, FT_SYS_NACS_GKS_RSN_MTHD = 15, FT_SYS_GKS_RSN_ITER = 16,
 FT_SYS_NACS_AMN_PASS = 17, FT_SYS_NACS_AMNS = 18, FT_SYS_CHK_TYPES = 19,
 FT_SYS_NACS_CHKS = 20, FT_SYS_NACS_STRENGTH = 21, FT_SYS_NACS_COMB_MODE = 22;
 final public static String GS_ID_DIM_NAME = "Goal ID";
 // SINGLE_ACTIVE_VALUED = 0 or MULTI_ACTIVE_VALUED = 1
 final public static int GKS_ITER_UNLIMIT_IDX = 10;
 public short[] FT_SYS_VALUE_PROPERTY =
                             {0, 1, 0, 1, 0,  0, 0, 0, 0, 0, 0,  1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0 };
 public short[] FT_SYS_LENGTHS =
                             {3, 0, 4, 0, 1,  3, 3, 3, 3, 3, 2,  0, 0, 0, 0, 2, 11, 10, 0, 5, 0, 11, 4 };
 public String[] FT_SYS_NAMES = {
    "goal structure action", "working memory flag", "working memory action", "working memory action parameter", "reward",
    "activate NACS or report", "choose level", "encode externally given knowledge", "assimilate explicit knowledge",
    "do inference or retrieval", "set chunk strength", "format of input to NACS (using input to ACS",
    "format of input to NACS (using output from ACS", "format of output to NACS (using input to ACS",
    "format of output to NACS (using output from ACS", "GKS reasoning method", "number of iteration of GKS resoning",
    "number of AMN passes for each iteration", "involved AMNs",  "type of chunk to store into retrieval buffer",
    "the chunk to set strength of", "strength level to set at", "combination method"
 };
 public String[][] FT_SYS_VALUE_NAMES = { {"do nothing","set","reset",}, {},
   {"do nothing", "set", "reset", "reset_all"}, {}, {"reward value 1"},
   {"not activate NACS and not report", "activate NACS", "report to eternal destination from the retrieval buffer of NACS"},
   {"choose GKS", "choose AMN", "choose both"},
   {"do nothing", "system-default encodes as associative rules", "task-specific encodes as associative rules" },
   {"do nothing", "system-default assimilates", "task-specific assimilates"},
   {"do nothing", "retrieve one chunk from NACS above a threshold by boltzmann distribution", "retrieve all chunks from NACS above a threshold"},
   {"do nothing", "set strength"}, {},{},{},{}, {"forward reasoning", "forward chaining with similarity-based reasoning"},
   {"# of iterations is 1", "# of iterations is 2","# of iterations is 3","# of iterations is 4","# of iterations is 5",
    "# of iterations is 6","# of iterations is 7","# of iterations is 8","# of iterations is 9", "# of iterations is 10",
    "# of iterations is unlimited"},
   {"# of passes is 1","# of passes is 2", "# of passes is 3", "# of passes is 4", "# of passes is 5", "# of passes is 6",
    "# of passes is 7", "# of passes is 8", "# of passes is 9", "# of passes is 10"},  {},
   {"all chunk types", "state-related chunks", "state-related chunk not involved in the input to NACS", "action-related actions",
    "state-related chunk not involved in the input to NACS"}, {},
   {"0.0", "0.1", "0.2", "0.3", "0.4", "0.5", "0.6", "0.7", "0.8", "0.9", "1.0"},
   {"do nothing", "system default", "task-specific bottom-up rectification", "task-specific top-down guidance"}
 };
 /** constants for format of WM ITEM. */
 final public static int GENERIC_COMP_NUM = 8,
 GENERIC_EX_INPUT = 0, GENERIC_GS_INPUT = 1, GENERIC_EX_ACT = 2,
 GENERIC_NACS_ACT = 3, GENERIC_GS_ACT = 4, GENERIC_WM_ACT = 5,
 GENERIC_WM_FLAG = 6, GENERIC_USER_SPEC = 7;
 /** constants for ACS action types. */
 final public static int ACTION_NORMAL = 0, ACTION_CONTROL = 1;
 // constants for minor options in each major option.
 final public static int AEM = 1;
 final public static int AAM = 0, HAM = 1;
 final public static int EMPTY = -1, CONSTANT_OPTION = 0, VARIED_OPTION = 1;
 final public static int OUTPUT_TYPE_NUM = 5, ACS_OUTPUT_TYPE_NUM = 4;
 final public static int EX_TYPE = 0, WM_TYPE = 1, GS_TYPE = 2, CONTROL_TYPE = 3, MCS_TYPE = 4;
 final public static int GS_STACK = 0, GS_LIST = 1, GS_NON = 2;
 final public static int INPUT = 0, HIDDEN = 1, OUTPUT = 2;
 final public static int ACS_KNOW_TYPE_NUM = 4;
 final public static int BL = 0, TL = 1, RER = 1, IRL = 2, FR = 3;
 final public static int ORDINAL = 0, NOMINAL = 1;
 final public static int QL = 0, SQL = 1, BP = 2, FIX = 3, BL_BY_MCS = 4;
 final public static int REIN_ALL = 0, REIN_EACH = 1;
 final public static int FORWARD_CHAIN = 0, FORWARD_CHAIN_WITH_SBR = 1;
 final public static int GENERALIZATION = 0, SPECIFICATION = 1;
 final public static int NO_GEN_SPEC = 0, SPEC_TO_GEN = 1, GEN_TO_SPEC = 2, DBL_DIRECT = 3;
 final public static int POS_MATCH_TYPE = 0, NEG_MATCH_TYPE = 1;
 final public static int GS_CONTI_VAL = 0, GS_NCONT_VAL = 1, GS_ONE_ALL_VALUE = 2;
 final public static int NO_MERGE = 0, MERGE_PER_ONE = 1, MERGE_PER_TEN = 3;
 final public static int UTL_BENE = 0, UTL_COST = 1, UTL_V = 2, UTL_C7 = 3, UTL_C8 = 4;
 final public static int BLA_C = 0, BLA_D = 1, BLA_I = 2;
 final public static int ACTION_BY_TL = 0, ACTION_BY_BOTH = 1;
 final public static int FALSE = 0, TRUE = 1;
 final public static int COMB_BY_MCS = 0, COMB_NBY_MCS = 1;
 final public static int FIX_COMB_MODE = 0, VAR_COMB_MODE = 1, CT_COMB_MODE = 2;
 final public static int STOCH_COMB_MODE = 0, BU_COMB_MODE = 1, TG_COMB_MODE = 2;
 final public static int ALL_CHK_TYPES = -1, CHK_STATE_DIM = 0, CHK_ACTION_DIM = 1,
                         CHK_REWARD = 2, CHK_STEP = 3, CHK_STATE = 4, CHK_ACTION = 5,
                         CHK_ACTION_RULE = 6, CHK_CUE = 7, CHK_RETRIEVAL = 8, CHK_COMB = 9,
                         CHK_ASSOC = 10, CHK_GOAL = 11, CHK_WM_FLAG = 12;
 final public static String[] CHUNK_TYPE_LABELS ={
                         "STATE DIM", "ACTION DIM", "REWARD", "STEP",
                         "STATE", "ACTION", "ACTION RULE", "CUE",
                         "RETRIEVAL", "COMB", "ASSOC", "GOAL", "FLAG"
                     };
 final public static int STEP_COMP_NUM = 4;
 final public static int CUR_STATE = 0, CUR_ACTION = 1, NEXT_STATE = 2, REWARD = 3;
 final public static int COR_EXT_ALL = 0, COR_EXT_ONE = 1;
 final public static int RULE_BASED = 0, SIMI_BASED = 1;
 final public static int ONE_PASS = 0, MULTI_PASS = 1;
 final public static int ADR_REASON = 0, ADM_REASON = 1;
 final public static int NO_DISPLAY = 0, DISP_INPUT = 1, DISP_BOTH = 2;
 final public static int EVERY_STEP = 1, EVERY_X_STEP = 2;
 /*-------------------------  CLARION SETTINGS -------------------------------*/
  Clarion clarion;
  boolean nacsOn;
  boolean superOn;
  boolean rtOn;
 /*---------------------------------------------------------------------------*/
/*---------------------------FEATURES INFORMATION.----------------------------*/
 /** FT_SYS, FT_INPUT, FT_OUTPUT, FT_GS. */
 int     featureTypeNum = Global.FEAT_TYPE_NUM;
 /** feature # in each type.  */
 short[] featureNums;  {
    featureNums = new short[ featureTypeNum ];
    featureNums[Global.FT_SYS] = FEAT_SYS_NUM;
 }
 /** the length of each feature. */
 short[] featureLengths[]; {
    featureLengths = new short[ featureTypeNum ][];
    featureLengths[Global.FT_SYS] = FT_SYS_LENGTHS;
 }
 /** names of features. */
 String  featureNames[][]; {
    featureNames = new String[ featureTypeNum ][];
    featureNames[Global.FT_SYS] = FT_SYS_NAMES;
 }
 /** names of feature values. */
 String  featureValueNames[][][]; {
    featureValueNames = new String[ featureTypeNum ][][];
    featureValueNames[Global.FT_SYS] = FT_SYS_VALUE_NAMES;
 }
 short[] featureMultiValueProperties[]; {
   featureMultiValueProperties = new short[Global.FEAT_TYPE_NUM][];
   featureMultiValueProperties[Global.FT_SYS] = FT_SYS_VALUE_PROPERTY;
 }
 /** multiple-active-value properties: order: EX_TYPE, WM_TYPE, GS_TYPE, CONTROL_TYPE, MCS_TYPE */
 short[] outputMultiValueProperties[]; {
   outputMultiValueProperties = new short[OUTPUT_TYPE_NUM][];
   outputMultiValueProperties[Global.WM_TYPE] = new short[ACT_DIM_NUM_WM];
   // action dim.
   outputMultiValueProperties[Global.WM_TYPE][0] = Global.SINGLE_ACTIVE_VALUED;
   // action parameter dim.
   outputMultiValueProperties[Global.WM_TYPE][1] = Global.MULTI_ACTIVE_VALUED;
   outputMultiValueProperties[Global.CONTROL_TYPE] = new short[ACT_DIM_NUM_NACS_CONTROL];
   for( int i = 0; i < ACT_DIM_NUM_NACS_CONTROL; i++ )
        outputMultiValueProperties[Global.CONTROL_TYPE][i] = FT_SYS_VALUE_PROPERTY[i+NACS_CONTROL_DIM_START];
 };
 /*---------------------------------------------------------------------------*/
 /*=============== SETTINGS ABOUT ACTION-CENTERED SUBSYSTEM ==================*/
 /******************** SETTINGS ABOUT ACS COMPONENTS **************************/
  /** #s of all ACS networks including EX, WM, GS if exist. */
  int     acsNetNum;
  /** #s of all External ACS networks. */
  int     exNetNum;
  /** the WM net index.  */
  int     wmNetIdx;
  /** the GS net index.  */
  int     gsNetIdx;
  /** goal stack option. */
  int     gsOption;
  /** working memory option. */
  boolean wmOn;
  /** Goal Structure IDN on-off option. */
  boolean gsIdnOn;
  /** Working Memory IDN on-off option. */
  boolean wmIdnOn;
  /** Input dimension characters. */
  short[] inputDimCharacters;
  /** #s of each type of output in the overall outputs: EX, WM, GS, NACS-CONTROL. */
  short[] acsPhysicOutputNums; {
    acsPhysicOutputNums = new short[Global.ACS_OUTPUT_TYPE_NUM];
  }
  /** D-V info of each output dim in terms of VCGs: EX_TYPE, WM_TYPE, GS_TYPE.  */
  short[] acsPhysicOutputDimVals[]; {
    acsPhysicOutputDimVals = new short[Global.ACS_OUTPUT_TYPE_NUM-1][];
  }
  /** D-V info of each output dim: NACS control action. */
  short[] nacsControlOutputDimVals; {
    nacsControlOutputDimVals = new short[Global.ACT_DIM_NUM_NACS_CONTROL];
  }
  /** D-V info of each output dim in terms of VCGs:  NACS control action. */
  short[] nacsControlPhysicOutputDimVals; {
    nacsControlPhysicOutputDimVals = new short[Global.ACT_DIM_NUM_NACS_CONTROL];
  }
  /** D-V info of each output dim in terms of VCGs:  NACS control action. */
  short[][] nacsControlFeatureInfo; {
    nacsControlFeatureInfo = new short[Global.ACT_DIM_NUM_NACS_CONTROL][2];
        for(int i = 0; i < nacsControlFeatureInfo.length; i++ ){
             nacsControlFeatureInfo[i][0] = Global.FT_SYS;
             nacsControlFeatureInfo[i][1] = (short)(Global.FT_SYS_NACS_ACT + i);
        }
  }
 /************************* SETTINGS ABOUT IDN *******************************/
  /** D-V info of each input dim of all IDNs of ACS. */
  short[] idnInputDimVals[];
  /** D-V info of each output dim of all IDNs of ACS. */
  short[] idnOutputDimVals[];
  /** Feature info of each input dim of all IDNs of ACS. */
  short[] idnInputFeatureInfo[][];
  /** Feature info of each output dim of all IDNs of ACS. */
  short[] idnOutputFeatureInfo[][];
  /** #s of hidden unit of all ACS IDN network. */
  short[] idnHiddenNums;
  /** Eligible Conditions of all external IDNs.  */
  String  exIdnEligibleConditions[];
  /** BL learning options of all IDNs. */
  short[] idnBLLearningOptions;
  /** Frequencies of Assmilation of all IDNs. */
  short[] idnBlAssimFreqs[];
  /** BP Learning Rates of all IDNs. */
  double  idnBPLRs[];
  /** BP Momentums of all IDNs. */
  double  idnBPMoms[];
  /** Discount Rates of all IDNs for Q-Learing. */
  double  idnQDiscountRates[];
  /** Temperatures for action selection of all IDNs. */
  double  idnActionSelTemps[];
  /** Temperatures for overall action selection (within one ACS network) of all ACS networks. */
  double  overallActionSelTemps[];
  /** Reinforcement Funciton options of each IDN. */
  short   idnRLFuncOpts[];
  /** Reinforcement Funciton option for all IDNs. */
  String  overallIdnRLFunction;
  /** Reinforcement Funciton of each IDN. */
  String  idnRLFunctions[];
  /** On-off options of Reinforcement Funciton is by MCS or not. */
  boolean idnRLFuncByMCSs[];
 /************************** SETTINGS ABOUT ARS *******************************/
  /** D-V info of each input dim of each rule component of all ACS networks. */
  short[] arInputDimVals[][];
  /** D-V info of each output dim of each rule component of all ACS networks. */
  short[] arOutputDimVals[][];
  /** Feature info of each input dim of each rule component of all ACS networks. */
  short[] arInputFeatureInfo[][][];
  /** Feature info of each output dim of each rule component of all ACS networks. */
  short[] arOutputFeatureInfo[][][];
  /** Temperatures for action selection of each rule component of all ACS networks. */
  double  arActionSelTemps[][];
  /** Reasoning Method of each rule component of all ACS networks. */
  short[] arReasoningMethods[];
 /********************* SETTINGS ABOUT RULE LEARNING **************************/
  /** # of ACS knowledge types: DIN, RER, IRL and FR. */
  int     acsKnowledgeNum;
  /** On-Off options for RER use in ACS networks. */
  boolean rerOns[];
  /** On-Off options for IRL use in ACS networks. */
  boolean irlOns[];
  /** On-Off options for FR use in ACS networks. */
  boolean frOns[];
  /** On-Off options for External Instructions in ACS networks. */
  boolean extInstructOns[];
  /** On-Off options for Imitative Learning in ACS networks. */
  boolean ilOns[];
  /** On-Off options for condition and conclusion of FRs is varied or not. */
  boolean frActionVariedOns[][];
  /** Maximum # of RER rules in one ACS network. */
  int     maxRerRuleNum;
  /** The representations of all FRs of all ACS networks. */
  String  frSets[][];
  /** The condition of all FRs of all ACS networks. */
  short[] frConditions[][][];
  /** The conclusion of all FRs of all ACS networks. */
  short[] frActions[][][][];
  /** the #s of FR rules of all ACS networks. */
  short[] frNums;
  /** the #s of IRL rule sets of all ACS networks. */
  short[] irlSetNums;
  String  irlSets[][][];
  /** the #s of IRL rules in each rule set of all ACS networks. */
  short[] irlRuleNums[];
  double  irlRuleParams[][][][];
  /** the On-Off flags of IRL generalization. */
  boolean irlGenOns[];
  /** the On-Off flags of IRL specialization. */
  boolean irlSpecOns[];
  /** IRL density parameters of all ACS networks. */
  double  d_is[];
  /** RER density parameters of all ACS networks. */
  double  d_rs[];
  /** RER extraction probabilities of all ACS networks. */
  double  p_res[];
  /** RER generalization/specialization directions of all ACS networks. */
  short[] rerGenSpecModes;
  /** IRL generalization/specialization directions of all ACS networks. */
  short[] irlGenSpecModes;
  String  rerPosCriterions[];
  String  irlPosCriterions[];
  /** RER positivity thresholds of all ACS networks. */
  double  threshold_RERs[];
  /** IRL positivity thresholds of all ACS networks. */
  double  threshold_IRLs[];
  String  rerIGMeasures[];
  String  irlIGMeasures[];
  /** RER IG Parameters of all ACS networks. */
  double  c1s[];
  /** RER IG Parameters of all ACS networks. */
  double  c2s[];
  /** IRL IG Parameters of all ACS networks. */
  double  c5s[];
  /** IRL IG Parameters of all ACS networks. */
  double  c6s[];
  /** Partial match thresholds of all ACS networks. */
  double  threshold_PMs[];
  /** Thresholds for RER generalization of all ACS networks. */
  double  threshold1s[];
  /** Thresholds for RER specification of all ACS networks. */
  double  threshold2s[];
  /** Thresholds for IRL generalization of all ACS networks. */
  double  threshold6s[];
  /** Thresholds for IRL specification of all ACS networks. */
  double  threshold7s[];
  /** Thresholds for IRL deletion of all ACS networks. */
  double  threshold4s[];
  /** RER rule generalization/specification style of all ACS networks. */
  short[] rerGsValueRanges;
  /** IRL rule generalization/specification style of all ACS networks. */
  short[] irlGsValueRanges;
  /** # of steps for RER rule to merge of all ACS networks. */
  short[] ruleMergeIntervals;
 /******************* SETTINGS ABOUT RULE REPRESENTATIN ***********************/
  /** On-Off opitons for rule utility use in each rule set of each rule type of
   *  all ACS networks. */
  boolean ruleUtilityOns[][][];
  /** On-Off opitons for Partial Match use in each rule set of each rule type of
   *  all ACS networks. */
  boolean rulePartMatchOns[][][];
  /** On-Off opitons for rule BLA use in each rule set of each rule type of all
   *  ACS networks. */
  boolean ruleBlaOns[][][];
  /** On-Off opitons for action chunk BLA use in each rule set of each rule type
   *  of all ACS networks. */
  boolean actionChunkBlaOns[][][];
  /** Rule Condition Weight mode for each rule set of each rule type of all ACS networks. */
  short   ruleConditionWeightModes[][][];
  /** Rule Weight mode for each rule set of each rule type of all ACS networks. */
  short   ruleWeightModes[][][];
  /** Rule BLA options for each rule set of each rule type of all ACS networks. */
  short   actionRuleBlaOptions[][][];
  /** Rule BLA for each rule set of each rule type of all ACS networks. */
  double  actionRuleBlas[][][];
  /** Rule BLA parameters for each rule set of each rule type of all ACS networks. */
  double  actionRuleBlaParas[][][][];
  /** Rule Utility options for each rule set of each rule type of all ACS networks. */
  short   ruleUtilityOptions[][][];
  /** Rule Utility parameters for each rule set of each rule type of all ACS networks. */
  double  ruleUtilityParas[][][][];
  /** ACS chunk BLA options for each rule set of each rule type of all ACS networks. */
  short   acsChunkBlaOptions[][][];
  /** ACS chunk BLA for each rule set of each rule type of all ACS networks. */
  double  acsChunkBlas[][][];
  /** ACS chunk BLA parameters for each rule set of each rule type of all ACS networks. */
  double  acsChunkBlaParas[][][][];
 /********************* SETTINGS ABOUT GOAL STRUCTURE *************************/
  /** Goal Structure associated with this Global. */
  GoalStructure gs;
  String  goalFeatLabels[];
  String  goalChunkLabels[];
  short[] goals[][];
  /** size of a goal chunk. */
  int     size_gc;
  /** D-V info of goal parameter dims. */
  short[] goalParaDimVals;
  /** # of goal items in this goal structure. */
  int     size_g;
  /** threshold for a goal item to participate the compettition of the goal list. */
  double  threshold_GS;
  int     goalDecisionMode;
  /** Goal BLA option. */
  int     goalBlaOption;
  /** Goal BLA. */
  double  goalBla;
  /** Goal BLA parameters. */
  double  goalBlaParas[];
  /** Parameter for coordination of external vs. goal vs. wm */
  double  P_EXT;
  /** Parameter for coordination of external vs. goal vs. wm */
  double  P_GS;
  /** item dim info.  */
  short[] gsItemDimInfo[];
  /** length of GS item with sum of all dim lengths including GOAL dim. */
  int     gsItemLength;
  /** length of GS as input to IDN. */
  int     gsInputLength;
 /********************* SETTINGS ABOUT WORKING MEMORY *************************/
  /** working memeory associated with this global. */
  WM      wm;
  /** # of WM items. */
  int     size_wm;
  /** threshold for an WM item to be able to be input to IDN. */
  double  threshold_WM;
  /** On-off options for WM flags. */
  boolean wmFlagsOn;
  /** # of WM flags. */
  int     wmFlagNum;
  int     wmActionDecisionMode;
  int     wmActionFlagDecisionMode;
  /** WM BLA option. */
  int     wmBlaOption;
  /** WM BLA. */
  double  wmBla;
  /** WM BLA Parameters. */
  double  wmBlaParas[];
  /** item # in each WM section.  */
  short[] wmItemNums;
  /** item dim info in a WM section.  */
  short[] wmItemDimInfos[][];
  /** length of each WM item with sum of all dim lengths. */
  short[] wmItemLengths;
  /** length of WM as input to IDN. */
  int     wmInputLength;
 /***************** SETTINGS ABOUT COORDINATING ARS AND IDN *******************/
  boolean overrideByIdnOn;
  int     combineDeterMode;
  /** Combination modes of all ACS networks.
   *  element 0: STOCH, BU, TG; element 1: FIX, VAR, CT. */
  short   acsCrossCombModes[][];
  /** Combination parameters of all ACS networks. */
  double  acsCrossCombParas[][];
  /** Combination parameters of all ACS networks used (as constants) to calculate
   *  the probability of each knowledge type in action decision making. */
  double  c3s[][];
  /** Combination parameters of all ACS networks used (as constants) to calculate
   *  the probability of each knowledge type in action decision making. */
  double  c4s[][];
  /** the mode of coordination of external and goal networks. */
  int     corMultiExtMode;
 /*===========================================================================*/
 /*============= SETTINGS ABOUT NON-ACTION-CENTERED SUBSYSTEM ================*/
 /******************** SETTINGS ABOUT NACS COMPONENTS *************************/
  /** On-Off option for GKS use in NACS. */
  boolean gksOnFlag;
  /** On-Off option for Episodic Memory use in NACS. */
  boolean emOnFlag;
  /** On-Off option for Abstract Episodic Memory use in NACS. */
  boolean aemOnFlag;
 /******************** SETTINGS ABOUT GKS REPRESENATIONS **********************/
  /** D-V info of each input dim of GKS. */
  short[] gksInputDimVals;
  /** D-V info of each output dim of GKS. */
  short[] gksOutputDimVals;
  /** Feature info of each input dim of GKS. */
  short[] gksInputFeatureInfo[];
  /** Feature info of each output dim of GKS. */
  short[] gksOutputFeatureInfo[];
  /** Maximum # of chunks in GKS. */
  int     gksMaxChunkNum = 200;
  /** The termperature for GKS output selection. */
  double  gksOutputTemperature;
  /** On-Off option for use of Experince-specific chunk. */
  boolean esChunkOn;
  /** On-Off option for use of similarity. */
  boolean similarityOn;
  /** On-Off option for use of Associative rule BLA. */
  boolean assocBlaOn;
  /** On-Off option for use of Associative chunk BLA. */
  boolean nacsChunkBlaOn;
  /** Condition weight mode of Associative rules. */
  int     assocConditionWeightMode;
  /** Dimension weight mode of GKS chunk. */
  int     assocDimensionWeightMode;
  /** associative rule BLA option. */
  int     assocBlaOption;
  /** associative rule BLA. */
  double  assocBla;
  /** associative rule BLA parameters. */
  double  assocBlaParas[];
  /** GKS chunk BLA option. */
  int     chunkBlaOption;
  /** GKS chunk BLA. */
  double  chunkBla;
  /** GKS chunk BLA parameters. */
  double  chunkBlaParas[];
  /** EM BLA options for both action-oriented and non-action-oriented EM. */
  short[] emBlaOptions;
  /** EM BLAs for both action-oriented and non-action-oriented EM. */
  double  emBlas[];
  /** EM BLA parameters for both action-oriented and non-action-oriented EM. */
  double  emBlaParas[][];
  /** threshold for an EM item to keep or remove from EM. */
  double  threshold_EM;
  /** maximal # of EM items. */
  int     emCapacity = 50;
  /** probability of using EM for off-line training of the ACS at each step. */
  double  emTrainAcsProb;
  /** number of items selected for off-line training of the ACS at each step. */
  int     c9;
  /** number of iterations of off-line training of the ACS at each step. */
  int     c10;
  /** the strength threshold to discard a conclusion or not when using similarity
   *  measure. */
  double  threshold_sim;
  /** RBR parameter for calculating conclusion strength when using mixture of RBR
   *  and SBR. */
  double  c14;
  /** SBR parameter for calculating conclusion strength when using mixture of RBR
   *  and SBR. */
  double  c15;
 /*********************** SETTINGS ABOUT GKS LEARNINGS ************************/
  /** On-Off option of associative rule extraction from the AMNs. */
  boolean assocExtractOn;
  /** On-Off option of encoding associative rules dictated by ACS. */
  boolean assocEncodeExternOn;
  /** On-Off option of encoding chunks dictated by ACS. */
  boolean chunkEncodeExternOn;
  /** On-Off option of chunk extraction from the AMNs. */
  boolean chunkFromAmnOn;
  /** the chunk encoding probability. */
  double  p_c;
  /** the associative rule encoding probability. */
  double  p_a;
  /** the density parameter for NACS chunks. */
  double  d_c;
  /** the density parameter for associative rules. */
  double  d_a;
  /** the extraction threshold for associative rules. */
  double  threshold_ae;
  /** the extraction threshold for chunks. */
  double  threshold_ce;
  /** the extraction probability for associative rules. */
  double  p_ae;
  /** the extraction probability for chunks. */
  double  p_ce;
  /** activation measure. */
  int     activationMeasure;
 /******************** SETTINGS ABOUT AMN REPRESENATIONS **********************/
 /*-------- AMN SETTINGS --------*/
  /** # of AMN networks. */
  int     amnNum;
  /** the input chunk type of an AMN. */
  short[] amnInputChunkTypes;
  /** the output chunk type of an AMN. */
  short[] amnOutputChunkTypes;
  /** D-V info of each input dim of all AMN networks. */
  short[] amnInputDimVals[];
  /** D-V info of each output dim of all AMN networks. */
  short[] amnOutputDimVals[];
  /** Feature info of each input dim of all AMN networks. */
  short[] amnInputFeatureInfo[][];
  /** Feature info of each output dim of all AMN networks. */
  short[] amnOutputFeatureInfo[][];
  /** # of hidden units of each BP network of all AMN networks. */
  short[] amnHiddenNums;
  /** On-Off option of one pass vs. multiple pass. */
  boolean amnMultiPassOns[];
  /** # of passes in one GKS iteration. */
  short   amnPassNums[];
  /** On-Off opiton of allowing multiple output values in a dim.  */
  boolean amnMultiFlagOn;
 /*-------- AAM SETTINGS --------*/
  /** the probablity of AMN training at each step. */
  double  amnTrainedByEmProb;
  /** the # of items from the EM to use for training AMN at each step. */
  int     amnTrainedByEmItemNum;
  /** the # of training iterations for the chosen training set at each step. */
  int     amnTrainedByEmIterNum;
  /** the type of items from the EM to use for training AMN at each step. */
  int     amnTrainedByEmItemType;
 /*-------- AMN BP SETTINGS --------*/
  /* the BP learning rate of each AMN BL netwrok. */
  double  amnLRs[];
  /* the BP momentum of each AMN BL netwrok. */
  double  amnMoms[];
 /*-------- AEM SETTINGS --------*/
  /** Maximal # of states that can be coded in AEM. */
  int     aemStateNumBound;
  /** the number limit of potential next states. */
  int     aemPotentialNextStateNum = 5;
  /** Maximal # of action that can be coded in AEM. */
  int     aemActionNumBound;
  /** increment for quantization of reinforcement. */
  double  aemReinIncrement;
  /** the lower bound for quantization of reinforcement. */
  double  aemReinRangeLowerBound;
  /** the upper bound for quantization of reinforcement. */
  double  aemReinRangeUpperBound;
  /** probability of training AEM at each step. */
  double  aemTrainedByEmProb;
  /** the # of action-oriented EM items to use for training the AEM. */
  int     aemTrainedByEmItemNum;
  /** # of times each selected item is prsented to AEM at each step. */
  int     aemTrainedByEmIterNum;
  /** the BP learning rate for training the AEM. */
  double  aemTrainedByEmLR;
  /** the BP momentum for training the AEM. */
  double  aemTrainedByEmMom;
  /** the probability of using the AEM to training the ACS at each step. */
  double  aemTrainAcsProb;
  /** # of AEM samples to use for training the ACS at each step. */
  int     aemTrainAcsSampleNum;
  /** # of times each selected item is prsented to ACS for training the ACS. */
  int     aemTrainAcsIterNum;
  /** the temperature for AEM  output selection. */
  double  aemTrainAcsOutputTemp;
 /**************** SETTINGS ABOUT COORDINATING ACS and NACS *******************/
  /** action-directed reasoning or reasoning for action decision making. */
  int     corAcsNacsType;
  /** GKS reasoning method: forward-chaining or forward-chaining with SBR. */
  int     gksReasoningMethod;
  /** # of iterations of GKS reasoning in one round. */
  int     gksReasonIterNum;
  /** On-Off option of GKS reasoning by MCS. */
  boolean gksReasonByMCSOn;
  /** the reasoning threshold that determines whether a conclusion is acceptale or not. */
  double  threshold_r;
 /*===========================================================================*/
 /*================== SETTINGS ABOUT SUPERVISORY SUBSYSTEM ===================*/
 boolean msOn;
 boolean mcsOn;
 int c20, c21, c22, c23, c24, c25, c26, c27, c28, c29, c30, c31;
 MCS mcs;
 MS ms;
 /*===========================================================================*/
 /*============== SETTINGS ABOUT RESPONSE TIME DETERMINATION =================*/
  //int     maxUsageNum;
  String  overallFormula;
  String  blFormula;
  String  tlFormula;
  /** Perceptual Time of Bottom Level */
  int     PT_BL;
  /** Decision Time of Bottom Level */
  int     DT_BL;
  /** Activation Time of Bottom Level */
  int     AT_BL;
  /** Perceptual Time of Top Level */
  int     PT_TL;
  /** Operation time as part of Decision Time of Top Level */
  int     DT_TL_actionTime;
  /** constant as part of calculating Decision Time of Top Level */
  double  t0;
  /** constant as part of calculating Decision Time of Top Level */
  double  t1;
  /** Actuation Time of Top Level */
  int     AT_TL;
  String  assocAppTimeFormula;
  /** constant as part of calculating associative rule application time. */
  double  t4;
  /** constant as part of calculating associative rule application time. */
  double  t5;
  String  chunkRetTimeFormula;
  /** constant as part of calculating chunk retrieval time. */
  double  t2;
  /** constant as part of calculating chunk retrieval time. */
  double  t3;
  /** one-pass mapping time for the AMNs. */
  int     t_AMN;
 /*===========================================================================*/
 /*========================= SETTINGS ABOUT INTERFACE ========================*/
  int ruleDisplayMode;
  int ruleDisplayFreq;
 /*===========================================================================*/
 /*---------------------------------------------------------------------------*/
 /*---------------------- TASK-SPECIFIC SETTINGS -----------------------------*/
 /*---------------------------------------------------------------------------*/
  int       episodeLength = 10;       // the length of each episode.
  boolean   learningTestOn;           // testing the learning process flag.
  boolean[] inputFormat;
  boolean[] outputFormat;
  /** identify agent type.  */
  int       agentGroupId;
  /** the task agent associated with this Global object. */
  TaskAgent taskAgent;
  /** ACS subsystem. */
  ACS       acs;
  /** NACS subsystem. */
  NACS      nacs;
 /*---------------------------------------------------------------------------*/
 /*---------------------- SYSTEM VARIABLE SETTINGS ---------------------------*/
 /*---------------------------------------------------------------------------*/
  String taskSettingName, dataName;
  /** the variables for directory information of the software system. */
  static String rootDir = System.getProperty( "user.dir" );
  static String settingDir = "taskSetting";
  static String dataDir = "dataFiles";
  static String classDir = "bin";
  static String docDir = "documentation";
  static String srcDir = "sourceCode";
  /** the task being simulated. implementing some task specific functions, it should
  override clarion.system.Task.java. */
  static Task  task;
  /** the TaskClarion class associated with this Global object to implment the
   *  task-spedific CLARION options. */
  static Class taskClarionClass;
  /** the TaskClarion object associated with this Global object to implment the
   *  task-spedific CLARION options. */
  TaskClarion taskClarion;
  /** the variables for directory information of the task being simulated. */
  static String taskDir = classDir + "/tasksimulators/newsimu/";
  static String taskPkgName = "tasksimulators.newsimu.";
  static String taskClsName = "TaskPC";
  static String taskName = "PC";
  /** configuration with GUI or without GUI.  */
  static boolean guiFlag;

 /*************************** CONSTRUCTER *************************************/
 public Global(){}

 /***************************** GET METHODS ***********************************/
 /*--------------------- FEATURES INFORMATION. -------------------------------*/
 /** Get the numbers of features of all dimension types.
  * @return the numbers of features of all dimension types. */
 public short[] getFeatureNums(){  return featureNums; }
 /** Get the number of dimension types.
  * @return the number of dimension types. */
 public int getFeatureTypeNum(){ return featureTypeNum; }
 /** Get the length of each feature of all dimension types.
  * @return the length of each feature of all dimension types.. */
 public short[][] getFeatureLengths(){ return featureLengths; }
 public short[] getFeatureLengths( int typeIdx ){ return featureLengths[typeIdx]; }
 public int getFeatureLength( int typeIdx, int featIdx )
 { return featureLengths[typeIdx][featIdx]; }
 /** Get the name of each feature of all dimension types.
  * @return the length of each feature of all dimension types.. */
 public String[][] getFeatureNames(){ return featureNames; }
 public String[] getFeatureNames( int typeIdx )
 { return featureNames[ typeIdx ]; }
 public String getFeatureNames( int typeIdx, int featIdx )
 { return featureNames[typeIdx][featIdx]; }
 /** Get the value names of each feature of all dimension types.
  * @return the length of each feature of all dimension types.. */
 public String[][][] getFeatureValueNames(){ return featureValueNames; }
 public String[][] getFeatureValueNames( int typeIdx )
 { return featureValueNames[typeIdx]; }
 public String[] getFeatureValueNames(int typeIdx, int featIdx )
 { return featureValueNames[typeIdx][featIdx]; }
 public String getFeatureValueNames(int typeIdx, int featIdx, int valIdx )
 { return featureValueNames[typeIdx][featIdx][valIdx]; }
 /** Get the Multiple-Active-Values property of of each feature of all dimension types.
 * @return the length of each feature of all dimension types.. */
 public short[] getFeatureMultiValueProperty( int featType )
 { return featureMultiValueProperties[featType]; }
 // actionType: EX_TYPE, WM_TYPE, GS_TYPE, CONTROL_TYPE, MCS_TYPE
 public short[] getOutputMultiValueProperty( int actionType )
 { return outputMultiValueProperties[actionType]; }
 /******************** SETTINGS ABOUT SYSTEM COMPONENTS ***********************/
 public Clarion getClarion(){ return clarion; }
 public boolean isNacsOn(){ return nacsOn; }
 public boolean isSupervisorySubsysOn(){ return superOn; }
 public boolean isRtOn(){ return rtOn; }
 /******************** SETTINGS ABOUT ACS COMPONENTS **************************/
  public int getAcsNetNum() {
   return (acsNetNum = (isWmOn()&&isGsOn())?exNetNum+2 : (isWmOn()||isGsOn()?exNetNum+1 : exNetNum)); }
  public int getExNetNum(){ return exNetNum; }
  public int getGsOption() { return gsOption; }
  public boolean isGsOn() { return gsOption != GS_NON; }
  public boolean isWmOn() { return wmOn; }
  public int getWmNetIdx(){ return (wmOn)? exNetNum : EMPTY; }
  public int getGsNetIdx(){ return (gsOption!=EMPTY)?(wmOn?exNetNum+1:exNetNum):EMPTY; }
  public boolean isExNet(int netIdx){ return netIdx<exNetNum;}
  public boolean isWmNet(int netIdx){ return netIdx==getWmNetIdx(); }
  public boolean isGsNet(int netIdx){ return netIdx==getGsNetIdx(); }
  public boolean isIdnOn( int netIdx ){
     if( netIdx < exNetNum ) return true;
     return( netIdx==exNetNum )?((wmOn)?wmIdnOn:gsIdnOn):gsIdnOn;
  }
  public boolean isGsIdnOn() { return gsIdnOn; }
  public boolean isWmIdnOn() { return wmIdnOn; }
  /**************** SETTINGS ABOUT ACTION-CENTERED SUBSYSTEM *******************/
  public int getInputDimCharacter( int idx ){ return inputDimCharacters[ idx ]; }
  public int getAcsEntirePhysicOutputNum(){ return CommonMethods.getSum( acsPhysicOutputNums ); }
  public short[] getAcsPhysicOutputNums(){ return acsPhysicOutputNums; }
 /************************** SETTINGS ABOUT IDN *******************************/
  public int getIdnInputDimNum( int netIdx ){ return idnInputDimVals[ netIdx ].length; }
  public int getIdnOutputDimNum( int netIdx ){ return idnOutputDimVals[ netIdx ].length; }
  public int getAcsPhysicOutputDimNum( int netIdx )
  { int netType = isExNet(netIdx) ? Global.EX_TYPE :(isWmNet(netIdx) ? Global.WM_TYPE : Global.GS_TYPE );
    return acsPhysicOutputDimVals[ netType ].length;
  }
  public short[] getIdnInputDimVals( int netIdx ){ return idnInputDimVals[ netIdx ]; }
  public short[] getIdnOutputDimVals( int netIdx ){ return idnOutputDimVals[ netIdx ]; }
  public short[] getNacsControlOutputDimVals(){ return nacsControlOutputDimVals; }
  public short[] getAcsPhysicOutputDimVals( int netIdx )
  { int netType = isExNet(netIdx) ? Global.EX_TYPE :(isWmNet(netIdx) ? Global.WM_TYPE : Global.GS_TYPE );
    return acsPhysicOutputDimVals[ netType ];
  }
  public short[] getNacsControlPhysicOutputDimVals(){ return nacsControlPhysicOutputDimVals; }
  // give the info of each input dim's type and order in that type.
  public short[][] getIdnInputFeatureInfo( int netIdx ){ return idnInputFeatureInfo[ netIdx ]; }
  // give the info of each output dim's type and order in that type.
  public short[][] getIdnOutputFeatureInfo( int netIdx ){ return idnOutputFeatureInfo[ netIdx ]; }
  public short[][] getNacsControlFeatureInfo (){return nacsControlFeatureInfo;}
  public int getIdnInputNum( int netIdx ){ return CommonMethods.getSum( idnInputDimVals[netIdx]); }
  public int getIdnOutputNum( int netIdx ){ return CommonMethods.getSum( idnOutputDimVals[netIdx]); }
  public int getIdnHiddenNum( int netIdx ){ return idnHiddenNums[ netIdx ]; }
  public int getNacsControlOutputNum(){ return CommonMethods.getSum( nacsControlOutputDimVals ); }
  public int getAcsPhysicOutputNum( int netIdx )
  { int netType = isExNet(netIdx) ? Global.EX_TYPE :(isWmNet(netIdx) ? Global.WM_TYPE : Global.GS_TYPE );
    return CommonMethods.getSum( acsPhysicOutputDimVals[netType] );
  }
  public int getNacsControlPhysicOutputNum(){ return CommonMethods.getSum( nacsControlPhysicOutputDimVals ); }
  public String getExIdnEligibleCondiitons( int netIdx ){ return exIdnEligibleConditions[ netIdx ]; }
  public int getIdnLearningOption( int netIdx ){ return idnBLLearningOptions[ netIdx ]; }
  public short[] getIdnAssimilationFreqs( int netIdx ){ return idnBlAssimFreqs[ netIdx ]; }
  public double getIdnBPLearningRate( int netIdx ){ return idnBPLRs[ netIdx ]; }
  public double getIdnBPMomentum( int netIdx ){ return idnBPMoms[ netIdx ]; }
  public double getIdnQDiscountRate( int netIdx ){ return idnQDiscountRates[ netIdx ]; }
  public double getIdnActionSelectionTemp( int netIdx ){ return idnActionSelTemps[ netIdx ]; }
  public double getOverallActionSelectionTemp( int netIdx ){ return overallActionSelTemps[ netIdx ]; }
  public int getIdnRLFunctionOption( int netIdx ){ return idnRLFuncOpts[ netIdx ]; }
  public String getOverallIdnRLFunction(){ return overallIdnRLFunction; }
  public String getIdnRLFunctions( int netIdx ){ return idnRLFunctions[ netIdx ]; }
  public boolean isIdnRLFuncByMCS( int netIdx ){ return idnRLFuncByMCSs[ netIdx ]; }
 /************************** SETTINGS ABOUT ARS *******************************/
  public int getARInputDimNum( int netIdx, int ruleType )
  { return arInputDimVals[ netIdx ][ ruleType ].length; }
  public int getAROutputDimNum( int netIdx, int ruleType )
  { return arOutputDimVals[ netIdx ][ ruleType ].length; }
  public short[] getARInputDimVals( int netIdx, int ruleType )
  { return arInputDimVals[ netIdx ][ ruleType ]; }
  public short[] getAROutputDimVals( int netIdx, int ruleType )
  { return arOutputDimVals[netIdx][ruleType]; }
  /** give the info of each input dim's type and order in that type. the order in that
   *  type determines the location in the corresponding I/O data structure. */
  public short[][] getARInputFeatureInfo( int netIdx, int ruleType )
  { return arInputFeatureInfo[ netIdx ][ ruleType ]; }
  /** give the info of each output dim's type and order in that type. the order in that
   *  type determines the location in the corresponding I/O data structure. */
  public short[][] getAROutputFeatureInfo( int netIdx, int ruleType )
  { return arOutputFeatureInfo[ netIdx ][ ruleType ]; }
  public int getARInputNum( int netIdx, int ruleType )
  { return CommonMethods.getSum( arInputDimVals[ netIdx ][ ruleType ] ); }
  public int getAROutputNum( int netIdx, int ruleType )
  { return CommonMethods.getSum( arOutputDimVals[ netIdx ][ ruleType ] ); }
  public double getARActionSelectionTemp( int netIdx, int ruleType )
  {return arActionSelTemps[netIdx][ruleType];}
  public int getARReasoningMethod( int netIdx, int ruleType )
  {return arReasoningMethods[netIdx][ruleType];}
 /********************* SETTINGS ABOUT RULE LEARNING **************************/
  public int getAcsKnowledgeNum(){ return acsKnowledgeNum; }
  public boolean isRerOn( int netIdx ){ return rerOns[ netIdx ]; }
  public boolean isIrlOn( int netIdx ){ return irlOns[ netIdx ]; }
  public boolean isExtInstructOn( int netIdx ){ return extInstructOns[ netIdx ]; }
  public boolean isILOn( int netIdx ){ return ilOns[ netIdx ]; }
  public boolean isFrOn( int netIdx ){ return frOns[ netIdx ]; }
  public int getMaxRerRuleNum(){ return maxRerRuleNum; }
  public boolean isFrActionVariedOn( int netIdx, int setIdx )
  {  return frActionVariedOns[ netIdx ][ setIdx ]; }
  public String[] getFixRuleSet( int netIdx ){ return frSets[ netIdx ];}
  public int getFixRuleNum( int netIdx ){return frNums[ netIdx ];}
  public short[][] getFixRuleConditions(int netIdx,int ruleIdx)
  { return frConditions[netIdx][ruleIdx]; }
  public short[][][] getFixRuleActions( int netIdx, int ruleIdx )
  { return frActions[ netIdx ][ ruleIdx ]; }
  public int getIrlSetNum( int netIdx ){ return irlSetNums[ netIdx ]; }
  public String[] getIrlSet( int netIdx, int setIdx ){ return irlSets[ netIdx ][ setIdx ]; }
  public short[] getIrlRuleNums( int netIdx ){ return irlRuleNums[ netIdx ]; }
  public double[][][] getIrlRuleParameters( int netIdx ){ return irlRuleParams[ netIdx ]; }
  public double[][] getIrlRuleParameters( int netIdx, int setIdx )
  {return irlRuleParams[netIdx][setIdx];}
  public double[] getIrlRuleParameters(int netIdx,int setIdx,int ruleIdx)
  {return irlRuleParams[netIdx][setIdx][ruleIdx];}
  public double getRerDensity( int netIdx ){ return d_rs[ netIdx ];}
  public double getIrlDensity( int netIdx ){ return d_is[ netIdx ];}
  public double[] getRerExtractionProbs(){ return p_res; }
  public double getRerExtractionProb( int netIdx ){ return p_res[ netIdx ]; }
  public int getRerGenSpecMode( int netIdx ){ return rerGenSpecModes[ netIdx ]; }
  public int getIrlGenSpecMode( int netIdx ){ return irlGenSpecModes[ netIdx ]; }
  public String getRerPosCriterion( int netIdx ){ return rerPosCriterions[ netIdx ]; }
  public String getIrlPosCriterion( int netIdx ){ return irlPosCriterions[ netIdx ]; }
  public double getRerPosThreshold( int netIdx ){ return threshold_RERs[ netIdx ]; }
  public double getIrlPosThreshold( int netIdx ){ return threshold_IRLs[ netIdx ]; }
  public String getRerIGMeasurement( int netIdx ){ return rerIGMeasures[ netIdx ]; }
  public String getIrlIGMeasurement( int netIdx ){ return irlIGMeasures[ netIdx ]; }
  public double getRerIGParaAbove( int netIdx ){ return c1s[ netIdx ]; }
  public double getRerIGParaUnder( int netIdx ){ return c2s[ netIdx ]; }
  public double getIrlIGParaAbove( int netIdx ){ return c5s[ netIdx ]; }
  public double getIrlIGParaUnder( int netIdx ){ return c6s[ netIdx ]; }
  public double getPMThreshold( int netIdx ){ return threshold_PMs[ netIdx ]; }
  public double getRerGenThreshold( int netIdx ){ return threshold1s[ netIdx ]; }
  public double getRerSpecThreshold( int netIdx ){ return threshold2s[ netIdx ]; }
  public double getIrlGenThreshold( int netIdx ){ return threshold6s[ netIdx ]; }
  public double getIrlSpecThreshold( int netIdx ){ return threshold7s[ netIdx ]; }
  public double getIrlDeleteThreshold( int netIdx ){ return threshold4s[ netIdx ]; }
  public int getRerGenSpecValueRange( int netIdx ){return rerGsValueRanges[ netIdx ]; }
  public int getIrlGenSpecValueRange( int netIdx ){return irlGsValueRanges[ netIdx ]; }
  public int getRuleMergeInterval( int netIdx ){ return ruleMergeIntervals[ netIdx ]; }
 /******************* SETTINGS ABOUT RULE REPRESENTATIN ***********************/
  // the following for ACS rules including RER, FR, IRL.
  public boolean[] isRuleUtilityOn( int netIdx, int ruleType )
  { return ruleUtilityOns[netIdx][ruleType]; }
  public boolean isRuleUtilityOn(int netIdx,int ruleType,int setIdx)
  {return ruleUtilityOns[netIdx][ruleType][setIdx];}
  public boolean[] isRulePartMatchOn( int netIdx, int ruleType )
  { return rulePartMatchOns[ netIdx ][ ruleType ]; }
  public boolean isRulePartMatchOn( int netIdx, int ruleType, int setIdx )
  {return rulePartMatchOns[netIdx][ruleType][setIdx];}
  public boolean[] isRuleBlaOn( int netIdx, int ruleType )
  { return ruleBlaOns[ netIdx ][ ruleType ]; }
  public boolean isRuleBlaOn( int netIdx, int ruleType, int setIdx )
  {return ruleBlaOns[netIdx][ruleType][setIdx]; }
  public boolean[] isActionChunkBlaOn( int netIdx, int ruleType )
  {return actionChunkBlaOns[ netIdx ][ ruleType ]; }
  public boolean isActionChunkBlaOn(int netIdx,int ruleType,int setIdx)
  {return actionChunkBlaOns[netIdx][ruleType][setIdx];}
  public short[] getRuleConditionWeightMode( int netIdx, int ruleType )
  { return ruleConditionWeightModes[netIdx][ruleType];}
  public int getRuleCondtionWeightMode(int netIdx,int ruleType,int setIdx)
  {return ruleConditionWeightModes[netIdx][ruleType][setIdx];}
  public short[] getRuleWeightMode( int netIdx, int ruleType )
  { return ruleWeightModes[netIdx][ruleType];}
  public int getRuleWeightMode(int netIdx,int ruleType,int setIdx)
  {return ruleWeightModes[netIdx][ruleType][setIdx];}
 /*!!!!!!!!!!! the following 7 options for all types or each??? !!!!!!!!!!!!!!*/
  public short[] getActionRuleBlaOption( int netIdx, int ruleType )
  {return actionRuleBlaOptions[ netIdx ][ ruleType ]; }
  public int getActionRuleBlaOption(int netIdx,int ruleType,int setIdx)
  {return actionRuleBlaOptions[netIdx][ruleType][setIdx];}
  public double[] getActionRuleBla( int netIdx, int ruleType )
  { return actionRuleBlas[ netIdx ][ ruleType ]; }
  public double getActionRuleBla( int netIdx, int ruleType, int setIdx )
  {return actionRuleBlas[ netIdx ][ ruleType ][ setIdx ];}
  public double[][] getActionRuleBlaParas( int netIdx, int ruleType )
  { return actionRuleBlaParas[ netIdx ][ ruleType ];}
  public double[] getActionRuleBlaParas(int netIdx,int ruleType,int setIdx)
  {return actionRuleBlaParas[netIdx][ruleType][setIdx];}
  public short[] getRuleUtilityOption( int netIdx, int ruleType )
  { return ruleUtilityOptions[ netIdx ][ ruleType ];}
  public int getRuleUtilityOption( int netIdx, int ruleType, int setIdx )
  { return ruleUtilityOptions[netIdx][ruleType][setIdx];}
  public double[][] getRuleUtilityParas( int netIdx, int ruleType )
  {return ruleUtilityParas[ netIdx ][ ruleType ];}
  public double[] getRuleUtilityParas( int netIdx, int ruleType, int setIdx )
  {return ruleUtilityParas[netIdx][ruleType][setIdx];}
  public short[] getAcsChunkBlaOption( int netIdx, int ruleType )
  { return acsChunkBlaOptions[ netIdx ][ ruleType ]; }
  public int getAcsChunkBlaOption( int netIdx, int ruleType, int setIdx )
  { return acsChunkBlaOptions[netIdx][ruleType][setIdx];}
  public double[] getAcsChunkBla( int netIdx, int ruleType )
  { return acsChunkBlas[ netIdx ][ ruleType ];}
  public double getAcsChunkBla( int netIdx, int ruleType, int setIdx )
  { return acsChunkBlas[netIdx][ruleType][setIdx]; }
  public double[][] getAcsChunkBlaParas( int netIdx, int ruleType )
  { return acsChunkBlaParas[ netIdx ][ ruleType ]; }
  public double[] getAcsChunkBlaParas(int netIdx,int ruleType,int setIdx)
  { return acsChunkBlaParas[netIdx][ruleType][setIdx]; }
  /********************* SETTINGS ABOUT GOAL STRUCTURE *************************/
  public GoalStructure getGS(){ return gs; }
  public String[] getGoalFeatureLabels(){ return goalFeatLabels; }
  public String[] getGoalChunkLabels(){ return goalChunkLabels; }
  public short[][] getGoals( int goalIdx ){ return goals[ goalIdx ]; }
  public int getGoalChunkSize(){   return size_gc; }
  // return the length of each goal parameter dimension.
  public short[] getGoalParaDimVals(){ return goalParaDimVals;}
  public int getGoalStructSize(){ return size_g; }
  // threshold when checking qualification for participating in the competition
  // of the goal list.
  public double getGoalActivationThreshold(){ return threshold_GS;}
  // goal decided by TL or both of levels.
  public int getGoalDecisionMode(){ return goalDecisionMode; }
  public int getGoalBlaOption(){ return goalBlaOption; }
  public double getGoalBla(){ return goalBla; }
  public double[] getGoalBlaParas(){ return goalBlaParas; }
  public double getExternCorConstant(){ return P_EXT; }
  public double getGoalCorConstant(){ return P_GS; }
  public short[][] getGsItemDimInfo(){ return gsItemDimInfo; }
  public int getGsItemLength()
  { return gsItemLength = CommonMethods.calcGsItemLength(gsItemDimInfo,featureLengths); }
  public int getGsInputLength()
  { return gsInputLength = CommonMethods.calcGsItemLength(gsItemDimInfo,featureLengths); }
 /********************* SETTINGS ABOUT WORKING MEMORY *************************/
  public WM getWM(){ return wm; }
  public int getWmSize(){ return size_wm; }
  public double getWmActivationThreshold(){ return threshold_WM; }
  public boolean isWmFlagsOn(){ return wmFlagsOn; }
  public int getWmFlagNum(){ return wmFlagNum; }
  public int getWmActionDecisionMode(){ return wmActionDecisionMode; }
  public int getWmActionFlagDecisionMode(){ return wmActionFlagDecisionMode; }
  public int getWmBlaOption(){ return wmBlaOption; }
  public double getWmBla(){ return wmBla;}
  public double[] getWmBlaParas(){ return wmBlaParas; }
  public short[] getWmItemNums(){ return wmItemNums; }
  public short[][][] getWmItemDimInfos(){ return wmItemDimInfos; }
  public short[] getWmItemLengths(){ return wmItemLengths; }
  public int getWmInputLength(){ return wmInputLength = CommonMethods.getSum( wmItemLengths ); }
  //{return wmItemLengths = CommonMethods.calcWmItemLengths(size_wm,wmItemNums,wmItemDimInfos,featureLengths);}
 /***************** SETTINGS ABOUT COORDINATING ARS AND IDN *******************/
  //public boolean isOverrideByIdn() { return overrideByIdnOn; }
  public boolean isOverrideByIdn() { return this.getGsOption() == Global.GS_NON; }
  public int getCombineDeterMode(){return combineDeterMode;}
  public short[] getAcsCrossCombineModes(int netIdx){ return acsCrossCombModes[netIdx]; }
  public int getAcsCrossCombineMode(int netIdx, int modeIdx){ return acsCrossCombModes[netIdx][modeIdx]; }
  public double[][] getAcsCrossCombineParas(){ return acsCrossCombParas; }
  public double[] getAcsCrossCombineParas( int netIdx ){ return acsCrossCombParas[ netIdx ];}
  public double[][] getAcsCrossCombineVarParaAbove(){return c3s;}
  public double[] getAcsCrossCombineVarParaAbove( int netIdx ){ return c3s[ netIdx ]; }
  public double[][] getAcsCrossCombineVarParaUnder(){ return c4s; }
  public double[] getAcsCrossCombineVarParaUnder( int netIdx ){ return c4s[ netIdx ];}
  public int getCorMultiExtMode(){ return corMultiExtMode;}
 /************** SETTINGS ABOUT NON-ACTION-CENTERED SUBSYSTEM *****************/
 /******************** SETTINGS ABOUT NACS COMPONENTS *************************/
  public boolean isGKSon(){ return gksOnFlag; }
  public boolean isEMon(){  return emOnFlag; }
  public boolean isAEMon(){ return aemOnFlag; }
 /******************** SETTINGS ABOUT GKS REPRESENATIONS **********************/
  public int getGksInputDimNum() { return gksInputDimVals.length; }
  public int getGksOutputDimNum() { return gksOutputDimVals.length; }
  public short[] getGksInputDimVals() { return gksInputDimVals; }
  public short[] getGksOutputDimVals() { return gksOutputDimVals; }
  // include each feature's type and order in that type.
  public short[][] getGksInputFeatureInfo(){ return gksInputFeatureInfo;}
  // include each feature's type and order in that type.
  public short[][] getGksOutputFeatureInfo(){ return gksOutputFeatureInfo; }
  public int getGksInputNum() { return CommonMethods.getSum( gksInputDimVals ); }
  public int getGksOutputNum() { return CommonMethods.getSum( gksOutputDimVals ); }
  public int getGksMaxChunkNum(){ return gksMaxChunkNum;}
  public double getGksOutputTemperature(){ return gksOutputTemperature; }
  /******************* the following for NACS rules ********************/
  public boolean isESChunkOn(){ return esChunkOn; }
  public boolean isSimilarityOn(){ return similarityOn; }
  public boolean isAssocBlaOn(){ return assocBlaOn; }
  public boolean isNacsChunkBlaOn(){ return nacsChunkBlaOn;}
  public int getAssocConditionWeightMode(){ return assocConditionWeightMode; }
  public int getAssocDimensionWeightMode(){ return assocDimensionWeightMode; }
  public int getAssocBlaOption(){ return assocBlaOption; }
  public double getAssocBla(){ return assocBla; }
  public double[] getAssocBlaParas(){ return assocBlaParas; }
  public int getChunkBlaOption(){ return chunkBlaOption; }
  public double getChunkBla(){ return chunkBla;}
  public double[] getChunkBlaParas(){ return chunkBlaParas; }
  public int getEmBlaOption( int emType ){ return emBlaOptions[ emType ]; }
  public double getEmBla(  int emType ){ return emBlas[ emType ]; }
  public double[] getEmBlaParas( int emType ){ return emBlaParas[ emType ]; }
  public double getThreshold_EM(){ return threshold_EM; }
  public int getEmCapacity(){ return emCapacity; }
  public double getEmTrainAcsProb(){  return emTrainAcsProb; }
  public int getEmTrainAcsItemNum(){  return c9; }
  public int getEmTrainAcsIterNum(){ return c10; }
  public double getThreshold_sim(){ return threshold_sim; }
  public double getSBR_BalConstant(){ return c14; }
  public double getRBR_BalConstant(){ return c15; }
 /*********************** SETTINGS ABOUT GKS LEARNINGS ************************/
  public boolean isAssocExtractOn(){ return assocExtractOn; }
  public boolean isAssocEncodeExternOn(){ return assocEncodeExternOn; }
  public boolean isChunkEncodeExternOn() { return chunkEncodeExternOn; }
  public boolean isChunkFromAmnOn(){ return chunkFromAmnOn;}
  public double getChunkEncodeProb(){  return p_c; }
  public double getAssocEncodeProb(){  return p_a; }
  public double getChunkDensity(){ return d_c; }
  public double getAssocDensity(){ return d_a; }
  public double getAssocExtThreshold(){ return threshold_ae; }
  public double getChunkExtThreshold(){ return threshold_ce; }
  public double getAssocExtProb(){ return p_ae; }
  public double getChunkExtProb(){ return p_ce;}
  public int getActivationMeasure(){ return activationMeasure; }
 /******************** SETTINGS ABOUT AMN REPRESENATIONS **********************/
 /*-------- AMN SETTINGS --------*/
  public int getAmnNum(){return amnNum; }
  public int getAmnInputDimNum( int idx ){return amnInputDimVals[idx].length;}
  public int getAmnOutputDimNum( int idx ){return amnOutputDimVals[idx].length;}
  public short[] getAmnInputDimVals( int idx ){ return amnInputDimVals[ idx ];}
  public short[] getAmnOutputDimVals( int idx ){ return amnOutputDimVals[ idx ]; }
  // include each feature's type and order in that type.
  public short[][] getAmnInputFeatureInfo( int idx ){ return amnInputFeatureInfo[ idx ];}
  // include each feature's type and order in that type.
  public short[][] getAmnOutputFeatureInfo( int idx ){ return amnOutputFeatureInfo[ idx ]; }
  public int getAmnInputNum( int idx ){ return CommonMethods.getSum( amnInputDimVals[idx]); }
  public int getAmnOutputNum( int idx ){ return CommonMethods.getSum( amnOutputDimVals[idx]); }
  public int getAmnHiddenNum( int idx ){ return amnHiddenNums[ idx ]; }
  public short getAmnInputChunkType( int idx ){ return amnInputChunkTypes[ idx ]; }
  public short getAmnOutputChunkType( int idx ){ return amnOutputChunkTypes[ idx ]; }
  public boolean isAmnMultiPassOn( int idx ){ return amnMultiPassOns[ idx ]; }
  public short getAmnPassNum( int idx ){ return amnPassNums[ idx ]; }
  public boolean isAmnMultiFlagOn(){ return amnMultiFlagOn; }
 /*-------- AMN TRAINING SETTINGS --------*/
  public double getAmnTrainedByEmProb(){ return amnTrainedByEmProb; }
  public int getAmnTrainedByEmItemType(){ return amnTrainedByEmItemType; }
  public int getAmnTrainedByEmItemNum(){ return amnTrainedByEmItemNum; }
  public int getAmnTrainedByEmIterNum(){ return amnTrainedByEmIterNum; }
 /*-------- AMN BP SETTINGS --------*/
  public double getAmnBPLearningRate( int idx ){ return amnLRs[ idx ]; }
  public double getAmnBPMomentum( int idx ){ return amnMoms[ idx ]; }
 /*-------- AEM SETTINGS --------*/
  public int    getAemStateNumBound(){ return aemStateNumBound; }
  public int    getAemActionNumBound(){ return aemActionNumBound; }
  public int    getAemPotentialNextStateNum(){ return aemPotentialNextStateNum; }
  public double getAemReinIncrement(){ return aemReinIncrement; }
  public double getAemReinRangeLowerBound(){ return aemReinRangeLowerBound; }
  public double getAemReinRangeUpperBound(){ return aemReinRangeUpperBound; }
  public double getAemTrainedByEmProb(){ return aemTrainedByEmProb; }
  public int getAemTrainedByEmItemNum(){ return aemTrainedByEmItemNum; }
  public int getAemTrainedByEmIterNum(){ return aemTrainedByEmIterNum; }
  public double getAemTrainedByEmBPLearningRate(){ return aemTrainedByEmLR; }
  public double getAemTrainedByEmBPMomentum(){ return aemTrainedByEmMom; }
  public double getAemTrainAcsProb(){ return aemTrainAcsProb; }
  public int getAemTrainAcsSampleNum(){ return aemTrainAcsSampleNum; }
  public int getAemTrainAcsIterNum(){ return aemTrainAcsIterNum; }
  public double getAemTrainAcsOutputTemp(){ return aemTrainAcsOutputTemp; }
 /**************** SETTINGS ABOUT COORDINATING ACS and NACS *******************/
  public int getCorAcsNacsType() { return corAcsNacsType; }
  /** forward chaining, simi-based forward. */
  public int getGksReasoningMethod(){return gksReasoningMethod;}
  public int getGksReasonIterNum(){return gksReasonIterNum;}
  public boolean isGksReasonByMCSOn(){ return gksReasonByMCSOn;}
  public double getReasonThreshold(){ return threshold_r; }
 /****************** SETTINGS ABOUT SUPERVISORY SUBSYSTEM *********************/
  public boolean isMsOn(){ return msOn; }
  public boolean isMcsOn(){ return mcsOn; }
  public MS getMS(){ return ms; }
  public MCS getMCS(){ return mcs; }
  public int getMonitorAcsTopConcNum(){ return c20; }
  public int getMonitorAcsBottomConcNum(){ return c24; }
  public int getMonitorAcsTopStepNum(){ return c22; }
  public int getMonitorAcsBottomStepNum(){ return c26; }
  public int getMonitorAcsTopEpisodeNum(){ return c28; }
  public int getMonitorAcsBottomEpisodeNum(){ return c30; }
  public int getMonitorNacsTopConcNum(){ return c21; }
  public int getMonitorNacsBottomConcNum(){ return c25; }
  public int getMonitorNacsTopStepNum(){ return c23; }
  public int getMonitorNacsBottomStepNum(){ return c27; }
  public int getMonitorNacsTopEpisodeNum(){ return c29; }
  public int getMonitorNacsBottomEpisodeNum(){ return c31; }
 /*************** SETTINGS ABOUT RESPONSE TIME DETERMINATION ******************/
  public boolean isRTOn(){ return rtOn; }
  //public int getMaxUsageNum(){ return maxUsageNum; }
  public String getOverallFormula(){ return overallFormula; }
  public String getBLFormula(){ return blFormula; }
  public String getTLFormula(){ return tlFormula; }
  public int getPT_BL(){ return PT_BL; }
  public int getDT_BL(){ return DT_BL; }
  public int getAT_BL(){ return AT_BL; }
  public int getPT_TL(){ return PT_TL; }
  // used for calculating DT_TL = DT_TL_actioTime + t0/rule_bla + t1/chunk_bla;
  public int getDT_TL_actionTime(){ return DT_TL_actionTime; }
  public double getDT_TL_t0(){ return t0; }
  public double getDT_TL_t1(){ return t1; }
  public int getAT_TL(){ return AT_TL; }
  public String getAssoAppTimeFormula(){ return assocAppTimeFormula;}
  public double getAssocAppTime_t4(){ return t4; }
  public double getAssocAppTime_t5(){ return t5; }
  public String getChunkRetTimeFormula(){return chunkRetTimeFormula;}
  public double getChunkRetTime_t2(){ return t2; }
  public double getChunkRetTime_t3(){ return t3; }
  public int getAmnOnePassTime(){ return t_AMN; }
 /************************** SETTINGS ABOUT INTERFACE *************************/
  public int getRuleDisplayMode(){ return ruleDisplayMode;}
  public int getRuleDisplayFreq(){ return ruleDisplayFreq; }
 /************************ SETTING ABOUT TASK-SPECIFIC ************************/
  public boolean isLearningTestOn(){ return learningTestOn; }
  public int getEpisodeLength(){ return episodeLength; }
  public int getSensoryInputLength(){ return CommonMethods.getSum(featureLengths[INPUT]); }
  public int getAgentGroupId(){ return agentGroupId; }
  public TaskAgent getTaskAgent(){ return taskAgent; }
  public ACS getACS(){ return acs; }
  public NACS getNACS(){ return nacs; }
  public String getTaskSettingName(){ return taskSettingName; }
  static public String getRootDir(){ return rootDir; }
  static public String getSettingDir(){ return settingDir; }
  static public String getDataDir(){ return dataDir; }
  static public Task getTask(){ return task; }
  static public Class getTaskClarionClass(){ return taskClarionClass; }
  public TaskClarion getTaskClarion(){ return taskClarion; }
  static public String getTaskDir(){ return taskDir; }
  static public String getTaskName(){ return taskName; }
  static public String getTaskPackageName(){ return taskPkgName; }
  static public String getTaskClassName(){ return taskClsName; }
  static public boolean isGuiMode(){ return guiFlag; }
 /***************************** SET METHODS ***********************************/
 /*--------------------- FEATURES INFORMATION. -------------------------------*/
  //public void setFeatureTypeNum( int value ){ featureTypeNum = value; }
  public void setFeatureNums( short[] values ){ featureNums = CommonMethods.copy( values );}
  public void setFeatureNum( int typeIdx, int value ){ featureNums[typeIdx] = (short)value;}
  public void setFeatureLengths( short[][] values ){ featureLengths = CommonMethods.copy( values );}
  public void setFeatureLengths( int type, short[] values ){ featureLengths[type] = CommonMethods.copy( values );}
  public void setFeatureLength( int type, int idx, int value ){ featureLengths[type][idx] = (short)value;}
  public void setFeatureNames( String[][] values ){ featureNames = CommonMethods.copy( values ); }
  public void setFeatureNames( int type, String[] values ){ featureNames[type] = CommonMethods.copy( values ); }
  public void setFeatureValueNames( String[][][] values )
  { featureValueNames = CommonMethods.copy( values ); }
  public void setFeatureValueNames( int type, String[][] values )
  { featureValueNames[type] = CommonMethods.copy( values ); }
  public void setFeatureValueNames( int type, int idx, String[] values )
  { featureValueNames[type][idx] = CommonMethods.copy( values ); }
  public void setFeatureMultiValueProperty( int featType, short[] values )
  { featureMultiValueProperties[featType] = CommonMethods.copy( values ); }
  // actionType : EX_TYPE, WM_TYPE, GS_TYPE, CONTROL_TYPE
  public void setOutputMultiValueProperty( int actionType, short[] values )
  { outputMultiValueProperties[actionType] = CommonMethods.copy( values ); }
 /**************** SETTINGS ABOUT ACTION-CENTERED SUBSYSTEM *******************/
  public void setClarion( Clarion value ){ clarion = value; }
  public void setNacsOn(boolean value){ nacsOn = value; }
  public void setSupervisorySubsysOn(boolean value){ superOn = value; }
  public void setRtOn(boolean value){ rtOn = value; }
  public void setInputDimCharacter(int idx,int value){inputDimCharacters[idx]=(short)value; }
 /******************** SETTINGS ABOUT ACS COMPONENTS **************************/
  public void setAcsNetNum( int value ){ acsNetNum = value; }
  public void setGsOption( int value ){ gsOption = value; }
  public void setWmOption( boolean value ){ wmOn = value; }
  public void setExNetNum( int value ){ exNetNum = value; }
  public void setIdnOn( int netIdx, boolean value ){
       if(isWmNet(netIdx)) wmIdnOn = value;
       if(isGsNet(netIdx)) gsIdnOn = value;
  }
  public void setGsIdnOn( boolean value ){ gsIdnOn = value; }
  public void setWmIdnOn( boolean value ){ wmIdnOn = value; }
  public void setAcsPhysicOutputNums( short[] values )
  { acsPhysicOutputNums = CommonMethods.copy( values ); }
 /************************** SETTINGS ABOUT IDN *******************************/
  public void setIdnInputDimVals( short[][] values )
  { idnInputDimVals = CommonMethods.copy( values ); }
  public void setIdnInputDimVals( int netIdx, short[] values )
  { idnInputDimVals[ netIdx ] = CommonMethods.copy( values ); }
  public void setIdnOutputDimVals( short[][] values )
  { idnOutputDimVals = CommonMethods.copy( values ); }
  public void setIdnOutputDimVals( int netIdx, short[] values )
  { idnOutputDimVals[ netIdx ] = CommonMethods.copy( values ); }
  public void setAcsPhysicOutputDimVals( short[][] values )
  { acsPhysicOutputDimVals = CommonMethods.copy( values ); }
  public void setNacsControlOutputDimVals( short[] values )
  { nacsControlOutputDimVals = CommonMethods.copy( values ); }
  public void setNacsControlPhysicOutputDimVals( short[] values )
  { nacsControlPhysicOutputDimVals = CommonMethods.copy( values ); }
  // give the info of each input dim's type and order in that type.
  public void setIdnInputFeatureInfo( short[][][] values )
  { idnInputFeatureInfo = CommonMethods.copy( values ); }
  public void setIdnInputFeatureInfo( int netIdx, short[][] values )
  { idnInputFeatureInfo[ netIdx ] = CommonMethods.copy( values ); }
  // give the info of each input dim's type and order in that type.
  public void setIdnOutputFeatureInfo( short[][][] values )
  { idnOutputFeatureInfo = CommonMethods.copy( values ); }
  public void setIdnOutputFeatureInfo( int netIdx, short[][] values )
  { idnOutputFeatureInfo[ netIdx ] = CommonMethods.copy( values ); }
  public void setIdnHiddenNums( short[] values )
  { idnHiddenNums = CommonMethods.copy( values ); }
  public void setIdnHiddenNum( int netIdx, int value )
  { idnHiddenNums[ netIdx ] = (short)value; }
  public void setExIdnEligibleCondiitons( String[] value )
  { exIdnEligibleConditions = value; }
  public void setExIdnEligibleCondiitons( int netIdx, String value )
  { exIdnEligibleConditions[ netIdx ] = value; }
  public void setIdnLearningOptions( short[] values )
  { idnBLLearningOptions = CommonMethods.copy( values ); }
  public void setIdnLearningOption( int netIdx, int value )
  { idnBLLearningOptions[ netIdx ] = (short)value; }
  public void setIdnAssimilationFreqs( short[][] values )
  { idnBlAssimFreqs = CommonMethods.copy( values ); }
  public void setIdnAssimilationFreqs( int netIdx, short[] values )
  { idnBlAssimFreqs[ netIdx ] = CommonMethods.copy( values ); }
  public void setIdnBPLearningRates( double[] values ){ idnBPLRs = CommonMethods.copy( values ); }
  public void setIdnBPLearningRate( int netIdx, double value ){ idnBPLRs[ netIdx ] = value; }
  public void setIdnBPMomentums( double[] values ){ idnBPMoms = CommonMethods.copy( values ); }
  public void setIdnBPMomentum( int netIdx, double value ){ idnBPMoms[ netIdx ] = value; }
  public void setIdnQDiscountRates( double[] values )
  { idnQDiscountRates = CommonMethods.copy( values ); }
  public void setIdnQDiscountRate( int netIdx, double value )
  { idnQDiscountRates[ netIdx ] = value; }
  public void setIdnActionSelectionTemps( double[] values )
  { idnActionSelTemps = CommonMethods.copy( values ); }
  public void setIdnActionSelectionTemp( int netIdx, double value )
  { idnActionSelTemps[ netIdx ] = value; }
  public void setOverallActionSelectionTemps( double[] values )
  { overallActionSelTemps = CommonMethods.copy( values ); }
  public void setOverallActionSelectionTemp( int netIdx, double value )
  { overallActionSelTemps[ netIdx ] = value; }
  public void setIdnRLFunctionOptions( short[] values )
  { idnRLFuncOpts = CommonMethods.copy( values ); }
  public void setIdnRLFunctionOption( int netIdx, int value )
  { idnRLFuncOpts[ netIdx ] = (short)value; }
  public void setOverallIdnRLFunction( String value )
  { overallIdnRLFunction = value; }
  public void setIdnRLFunctions( String[] values )
  { idnRLFunctions = CommonMethods.copy( values ); }
  public void setIdnRLFunction( int netIdx, String value )
  { idnRLFunctions[ netIdx ] = value; }
  public void setIdnRLFuncByMCS( boolean[] values )
  { idnRLFuncByMCSs = CommonMethods.copy( values ); }
  public void setIdnRLFuncByMCS( int netIdx, boolean value )
  { idnRLFuncByMCSs[ netIdx ] = value; }
 /************************** SETTINGS ABOUT ARS *******************************/
  public void setARInputDimVals( short[][][] values )
  { arInputDimVals = CommonMethods.copy( values ); }
  public void setARInputDimVals( int netIdx, int ruleType, short[] values )
  { arInputDimVals[netIdx][ruleType] = CommonMethods.copy( values ); }
  public void setAROutputDimVals( short[][][] values )
  { arOutputDimVals = CommonMethods.copy( values );}
  public void setAROutputDimVals( int netIdx, int ruleType, short[] values )
  { arOutputDimVals[netIdx][ruleType] = CommonMethods.copy( values );}
  // give the info of each input dim's type and order in that type.
  public void setARInputFeatureInfo( int netIdx, int ruleType, short[][] values )
  { arInputFeatureInfo[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  // give the info of each output dim's type and order in that type.
  public void setAROutputFeatureInfo( int netIdx, int ruleType, short[][] values )
  { arOutputFeatureInfo[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setARActionSelectionTemps(double[][] values)
  { arActionSelTemps = CommonMethods.copy( values ); }
  public void setARActionSelectionTemp(int netIdx,int ruleType,double value)
  { arActionSelTemps[netIdx][ruleType]=value;}
  public void setARReasoningMethods( short[][] values )
  { arReasoningMethods = CommonMethods.copy( values ); }
  public void setARReasoningMethod( int netIdx, int ruleType, int value )
  { arReasoningMethods[netIdx][ruleType]=(short)value; }
 /********************* SETTINGS ABOUT RULE LEARNING **************************/
  public void setAcsKnowledgeNum( int value ){ acsKnowledgeNum = value; }
  public void setRerOns( boolean[] values ){ rerOns = CommonMethods.copy( values ); }
  public void setRerOn( int netIdx, boolean value ){ rerOns[ netIdx ] = value;}
  public void setIrlOns( boolean[] values ){ irlOns = CommonMethods.copy( values ); }
  public void setIrlOn( int netIdx, boolean value ){ irlOns[ netIdx ] = value; }
  public void setExtInstructOns(boolean[] values){ extInstructOns = CommonMethods.copy( values ); }
  public void setExtInstructOn(int netIdx,boolean value){extInstructOns[netIdx]=value;}
  public void setILOns( boolean[] values ){ ilOns = CommonMethods.copy( values ); }
  public void setILOn( int netIdx, boolean value ){ ilOns[ netIdx ] = value; }
  public void setFrOns( boolean[] values ){ frOns = CommonMethods.copy( values ); }
  public void setFrOn( int netIdx, boolean value ){ frOns[ netIdx ] = value; }
  public void setMaxRerRuleNum( int value ){ maxRerRuleNum = value; }
  public void setFrActionVariedOns( boolean[][] values )
  {  frActionVariedOns = CommonMethods.copy( values ); }
  public void setFrActionVariedOn( int netIdx, int setIdx, boolean value )
  {  frActionVariedOns[ netIdx ][ setIdx ] = value; }
  public void setFixRuleSet( String[][] values ){ frSets = CommonMethods.copy( values ); }
  public void setFixRuleSet( int netIdx, String[] values ){ frSets[ netIdx ] = CommonMethods.copy( values ); }
  public void setFixRuleNums( short[] values ){ frNums = CommonMethods.copy( values ); }
  public void setFixRuleNum( int netIdx, int value ){ frNums[ netIdx ] = (short)value; }
  public void setFixRuleConditions( short[][][][] values ){ frConditions = CommonMethods.copy( values ); }
  public void setFixRuleConditions( int netIdx, short[][][] values )
  { frConditions[netIdx] = CommonMethods.copy( values );}
  public void setFixRuleConditions( int netIdx, int ruleIdx, short[][] values )
  { frConditions[netIdx][ruleIdx] = CommonMethods.copy( values );}
  public void setFixRuleActions( short[][][][][] values ){ frActions = CommonMethods.copy( values ); }
  public void setFixRuleActions( int netIdx, short[][][][] values )
  { frActions[netIdx] = CommonMethods.copy( values ); }
  public void setFixRuleAction( int netIdx, int ruleIdx, short[][][] values )
  { frActions[netIdx][ruleIdx] = CommonMethods.copy( values ); }
  public void setIrlSets( String[][][] values ){ irlSets = CommonMethods.copy( values ); }
  public void setIrlSet( int netIdx, int setIdx, String[] values )
  { irlSets[ netIdx ][ setIdx ] = CommonMethods.copy( values ); }
  public void setIrlSetNum( short[] values ){ irlSetNums = CommonMethods.copy( values ); }
  public void setIrlSetNum( int netIdx, int value ){ irlSetNums[ netIdx ]= (short)value; }
  public void setIrlRuleNums( short[][] values ){ irlRuleNums = CommonMethods.copy( values ); }
  public void setIrlRuleNums( int netIdx, short[] values ){ irlRuleNums[netIdx] = CommonMethods.copy( values ); }
  public void setIrlRuleNum( int netIdx, int setIdx, int value ){ irlRuleNums[netIdx][setIdx]=(short)value; }
  public void setIrlRuleParameters( double[][][][] values ){ irlRuleParams = CommonMethods.copy( values );}
  public void setIrlRuleParameters( int netIdx, double[][][] values )
  { irlRuleParams[ netIdx ] = CommonMethods.copy( values ); }
  public void setIrlRuleParameters( int netIdx, int setIdx, double[][] values )
  {irlRuleParams[netIdx][setIdx] = CommonMethods.copy( values );}
  public void setIrlRuleParameters(int netIdx,int setIdx,int ruleIdx,double[] values)
  { irlRuleParams[netIdx][setIdx][ruleIdx] = CommonMethods.copy( values ); }
  public void setIrlGenOns( boolean[] values ){ irlGenOns = CommonMethods.copy( values ); }
  public void setIrlGenOn( int netIdx, boolean value){ irlGenOns[ netIdx ] = value; }
  public void setIrlSpecOns( boolean[] values ){ irlSpecOns = CommonMethods.copy( values ); }
  public void setIrlSpecOn( int netIdx, boolean value){ irlSpecOns[ netIdx ] = value; }
  public void setIrlDensities( double[] values ){ d_is = CommonMethods.copy( values ); }
  public void setIrlDensity( int netIdx, double value){ d_is[ netIdx ] = value; }
  public void setRerDensities( double[] values ){ d_rs = CommonMethods.copy( values ); }
  public void setRerDensity( int netIdx, double value){ d_rs[ netIdx ] = value; }
  public void setRerExtractionProbs( double[] values){ p_res = CommonMethods.copy( values ); }
  public void setRerExtractionProb( int netIdx,double value){ p_res[ netIdx ] = value; }
  public void setRerGenSpecModes( short[] values ){ rerGenSpecModes = CommonMethods.copy( values ); }
  public void setRerGenSpecMode( int netIdx, int value ){ rerGenSpecModes[ netIdx ] = (short)value; }
  public void setIrlGenSpecModes( short[] values ){ irlGenSpecModes = CommonMethods.copy( values ); }
  public void setIrlGenSpecMode( int netIdx, int value ){ irlGenSpecModes[ netIdx ] = (short)value; }
  public void setRerPosCriterions(String[] values){ rerPosCriterions = CommonMethods.copy( values ); }
  public void setRerPosCriterion(int netIdx,String value){ rerPosCriterions[ netIdx ] = value; }
  public void setIrlPosCriterions(String[] values){ irlPosCriterions = CommonMethods.copy( values ); }
  public void setIrlPosCriterion(int netIdx,String value){ irlPosCriterions[ netIdx ] = value; }
  public void setRerPosThresholds(double[] values){ threshold_RERs = CommonMethods.copy( values ); }
  public void setRerPosThreshold(int netIdx,double value){ threshold_RERs[ netIdx ] = value; }
  public void setIrlPosThresholds(double[] values){ threshold_IRLs = CommonMethods.copy( values ); }
  public void setIrlPosThreshold(int netIdx,double value){ threshold_IRLs[ netIdx ] = value; }
  public void setRerIGMeasurements(String[] values){ rerIGMeasures = CommonMethods.copy( values ); }
  public void setRerIGMeasurement(int netIdx,String value){ rerIGMeasures[ netIdx ] = value; }
  public void setIrlIGMeasurements(String[] values){ irlIGMeasures = CommonMethods.copy( values ); }
  public void setIrlIGMeasurement(int netIdx,String value){ irlIGMeasures[ netIdx ] = value; }
  public void setRerIGParaAboves(double[] values){ c1s = CommonMethods.copy( values ); }
  public void setRerIGParaAbove(int netIdx,double value){ c1s[ netIdx ] = value; }
  public void setRerIGParaUnders(double[] values){ c2s = CommonMethods.copy( values ); }
  public void setRerIGParaUnder(int netIdx,double value){ c2s[ netIdx ] = value; }
  public void setIrlIGParaAboves(double[] values){ c5s = CommonMethods.copy( values ); }
  public void setIrlIGParaAbove(int netIdx,double value){ c5s[ netIdx ] = value; }
  public void setIrlIGParaUnders(double[] values){ c6s = CommonMethods.copy( values ); }
  public void setIrlIGParaUnder(int netIdx,double value){ c6s[ netIdx ] = value; }
  public void setPMThresholds(double[] values){ threshold_PMs = CommonMethods.copy( values ); }
  public void setPMThreshold(int netIdx,double value){ threshold_PMs[ netIdx ] = value; }
  public void setRerGenThresholds(double[] values){ threshold1s = CommonMethods.copy( values ); }
  public void setRerGenThreshold(int netIdx,double value){ threshold1s[ netIdx ] = value; }
  public void setRerSpecThresholds(double[] values){ threshold2s = CommonMethods.copy( values ); }
  public void setRerSpecThreshold(int netIdx,double value){ threshold2s[ netIdx ] = value; }
  public void setIrlGenThresholds(double[] values){ threshold6s = CommonMethods.copy( values ); }
  public void setIrlGenThreshold(int netIdx,double value){ threshold6s[ netIdx ] = value; }
  public void setIrlSpecThresholds(double[] values){ threshold7s = CommonMethods.copy( values ); }
  public void setIrlSpecThreshold(int netIdx,double value){ threshold7s[ netIdx ] = value; }
  public void setIrlDeleteThresholds(double[] values){ threshold4s = CommonMethods.copy( values ); }
  public void setIrlDeleteThreshold(int netIdx,double value){ threshold4s[ netIdx ] = value; }
  public void setRerGenSpecValueRanges(short[] values){ rerGsValueRanges = CommonMethods.copy( values ); }
  public void setRerGenSpecValueRange(int netIdx,int value){ rerGsValueRanges[ netIdx ] = (short)value; }
  public void setIrlGenSpecValueRanges(short[] values){ irlGsValueRanges = CommonMethods.copy( values ); }
  public void setIrlGenSpecValueRange(int netIdx,int value){ irlGsValueRanges[ netIdx ] = (short)value; }
  public void setRuleMergeIntervals(short[] values){ ruleMergeIntervals = CommonMethods.copy( values ); }
  public void setRuleMergeInterval(int netIdx,int value){ ruleMergeIntervals[ netIdx ] = (short)value; }
 /******************* SETTINGS ABOUT RULE REPRESENTATIN ***********************/
 // the following for ACS rules including RER, FR, IRL.
  public void setRuleUtilityOns( boolean[][][] values )
  {  ruleUtilityOns = CommonMethods.copy( values ); }
  public void setRuleUtilityOns( int netIdx, int ruleType, boolean[] values )
  {  ruleUtilityOns[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setRuleUtilityOn( int netIdx, int ruleType, int setIdx, boolean value )
  {  ruleUtilityOns[ netIdx ][ ruleType ][ setIdx ] = value; }
  public void setRulePartMatchOns( boolean[][][] values )
  {  rulePartMatchOns = CommonMethods.copy( values ); }
  public void setRulePartMatchOns( int netIdx, int ruleType, boolean[] values )
  {  rulePartMatchOns[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setRulePartMatchOn( int netIdx, int ruleType, int setIdx, boolean value )
  {  rulePartMatchOns[ netIdx ][ ruleType ][ setIdx ] = value; }
  public void setRuleBlaOns( boolean[][][] values )
  {  ruleBlaOns = CommonMethods.copy( values ); }
  public void setRuleBlaOns( int netIdx, int ruleType, boolean[] values )
  {  ruleBlaOns[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setRuleBlaOn( int netIdx, int ruleType, int setIdx, boolean value )
  {  ruleBlaOns[ netIdx ][ ruleType ][ setIdx ] = value; }
  public void setActionChunkBlaOns( boolean[][][] values )
  {  actionChunkBlaOns = CommonMethods.copy( values ); }
  public void setActionChunkBlaOns( int netIdx, int ruleType, boolean[] values )
  {  actionChunkBlaOns[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setActionChunkBlaOn( int netIdx, int ruleType, int setIdx, boolean value )
  {  actionChunkBlaOns[ netIdx ][ ruleType ][ setIdx ] = value; }
  public void setRuleWeightMode( short[][][] values )
  {  ruleWeightModes = CommonMethods.copy( values ); }
  public void setRuleWeightMode( int netIdx, int ruleType, short[] values )
  {  ruleWeightModes[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setRuleWeightMode( int netIdx, int ruleType, int setIdx, int value )
  {  ruleWeightModes[ netIdx ][ ruleType ][ setIdx ] = (short)value; }
  public void setRuleConditionWeightMode( short[][][] values )
  {  ruleConditionWeightModes = CommonMethods.copy( values ); }
  public void setRuleConditionWeightMode( int netIdx, int ruleType, short[] values )
  {  ruleConditionWeightModes[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setRuleConditionWeightMode( int netIdx, int ruleType, int setIdx, int value )
  {  ruleConditionWeightModes[ netIdx ][ ruleType ][ setIdx ] = (short)value; }
 /*!!!!!!!!!!! the following 7 options for all types or each??? !!!!!!!!!!!!!!*/
  public void setActionRuleBlaOption( short[][][] values )
  {  actionRuleBlaOptions = CommonMethods.copy( values ); }
  public void setActionRuleBlaOption( int netIdx, int ruleType, short[] values )
  {  actionRuleBlaOptions[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setActionRuleBlaOption( int netIdx, int ruleType, int setIdx, int value )
  { actionRuleBlaOptions[ netIdx ][ ruleType ][ setIdx ] = (short)value; }
  public void setActionRuleBla( double[][][] values )
  {  actionRuleBlas = CommonMethods.copy( values ); }
  public void setActionRuleBla( int netIdx, int ruleType, double[] values )
  {  actionRuleBlas[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setActionRuleBla( int netIdx, int ruleType, int setIdx, double value )
  {  actionRuleBlas[ netIdx ][ ruleType ][ setIdx ] = value; }
  public void setActionRuleBlaParas( double[][][][] values )
  {  actionRuleBlaParas = CommonMethods.copy( values ); }
  public void setActionRuleBlaParas( int netIdx, int ruleType, double[][] values )
  {  actionRuleBlaParas[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setActionRuleBlaParas( int netIdx, int ruleType, int setIdx, double[] values )
  {  actionRuleBlaParas[ netIdx ][ ruleType ][ setIdx ] = CommonMethods.copy( values ); }
  public void setRuleUtilityOption( short[][][] values )
  {  ruleUtilityOptions = CommonMethods.copy( values ); }
  public void setRuleUtilityOption( int netIdx, int ruleType, short[] values )
  {  ruleUtilityOptions[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setRuleUtilityOption( int netIdx, int ruleType, int setIdx, int value )
  {  ruleUtilityOptions[ netIdx ][ ruleType ][ setIdx ] = (short)value; }
  public void setRuleUtilityParas( double[][][][] values )
  {  ruleUtilityParas = CommonMethods.copy( values ); }
  public void setRuleUtilityParas( int netIdx, int ruleType, double[][] values )
  {  ruleUtilityParas[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setRuleUtilityParas( int netIdx, int ruleType, int setIdx, double[] values )
  {  ruleUtilityParas[ netIdx ][ ruleType ][ setIdx ] = CommonMethods.copy( values ); }
  public void setAcsChunkBlaOption( short[][][] values )
  {  acsChunkBlaOptions = CommonMethods.copy( values ); }
  public void setAcsChunkBlaOption( int netIdx, int ruleType, short[] values )
  {  acsChunkBlaOptions[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setAcsChunkBlaOption( int netIdx, int ruleType, int setIdx, int value )
  {  acsChunkBlaOptions[ netIdx ][ ruleType ][ setIdx ] = (short)value; }
  public void setAcsChunkBla( double[][][] values )
  {  acsChunkBlas = CommonMethods.copy( values ); }
  public void setAcsChunkBla( int netIdx, int ruleType, double[] values )
  {  acsChunkBlas[ netIdx ][ ruleType ] = CommonMethods.copy( values ); }
  public void setAcsChunkBla( int netIdx, int ruleType, int setIdx, double value )
  {  acsChunkBlas[ netIdx ][ ruleType ][ setIdx ] = value; }
  public void setAcsChunkBlaParas( double[][][][] values )
  {  acsChunkBlaParas = CommonMethods.copy( values );  }
  public void setAcsChunkBlaParas( int netIdx, int ruleType, double[][] values )
  {  acsChunkBlaParas[ netIdx ][ ruleType ] = CommonMethods.copy( values );  }
  public void setAcsChunkBlaParas( int netIdx, int ruleType, int setIdx, double[] values )
  {  acsChunkBlaParas[ netIdx ][ ruleType ][ setIdx ] = CommonMethods.copy( values ); }
 /********************* SETTINGS ABOUT GOAL STRUCTURE *************************/
  public void setGS( GoalStructure value ){ gs = value; }
  public void setGoalFeatureLabels( String[] values ){  goalFeatLabels = CommonMethods.copy( values ); }
  public void getGoalChunkLabels( String[] values ){  goalChunkLabels = CommonMethods.copy( values );  }
  public void setGoals( short[][][] values ){  goals = CommonMethods.copy( values );  }
  public void setGoalChunkSize( int value ){  size_gc = value; }
  //the length of each goal parameter dimension.
  public void setGoalParaDimVals( short[] values ){  goalParaDimVals = CommonMethods.copy( values ); }
  public void setGoalStructSize( int value ){  size_g = value; }
  // threshol when checking qualification for participating in the competition of the goal list.
  public void setGoalActivationThreshold( double value ){  threshold_GS = value; }
  // goal decided by TL or both of levels.
  public void setGoalDecisionMode( int value ){  goalDecisionMode = value;  }
  public void setGoalBlaOption( int value ){ goalBlaOption = value; }
  public void setGoalBla( double value ){ goalBla = value; }
  public void setGoalBlaParas( double[] values ){goalBlaParas = CommonMethods.copy( values );}
  public void setExternCorConstant( double value ){ P_EXT = value; }
  public void setGoalCorConstant( double value ){ P_GS = value; }
  public void setGsItemDimInfo( short[][] values ){ gsItemDimInfo = CommonMethods.copy( values ); }
  public void setGsItemDimInfo( int dimIdx, short[] values )
  { gsItemDimInfo[dimIdx] = CommonMethods.copy( values ); }
  public void setGsItemLength( int value ){ gsItemLength = value; }
 /********************* SETTINGS ABOUT WORKING MEMORY *************************/
  public void setWM( WM value ){ wm = value; }
  public void setWmSize( int value ){ size_wm = value; }
  public void setWmActivationThreshold( double value ) { threshold_WM = value; }
  public void setWmFlagsOn( boolean value ){ wmFlagsOn = value; }
  public void setWmFlagNum( int value ){ wmFlagNum = value; }
  public void setWmActionDecisionMode( int value ){ wmActionDecisionMode = value; }
  public void setWmActionFlagDecisionMode( int value ){ wmActionFlagDecisionMode = value; }
  public void setWmBlaOption( int value ){ wmBlaOption = value; }
  public void setWmBla( double value ){ wmBla = value; }
  public void setWmBlaParas( double[] values ){ wmBlaParas = CommonMethods.copy( values ); }
  public void setWmItemNums( short[] values ){ wmItemNums = CommonMethods.copy( values ); }
  public void setWmItemDimInfos( short[][][] values ){ wmItemDimInfos = CommonMethods.copy( values ); }
  public void setWmItemDimInfos( int sectIdx, short[][] values ){ wmItemDimInfos[sectIdx] = CommonMethods.copy( values ); }
  public void setWmItemDimInfo( int sectIdx, int dimIdx, short[] values )
  { wmItemDimInfos[sectIdx][dimIdx] = CommonMethods.copy( values ); }
  public void setWmItemLengths( short[] values ){ wmItemLengths = CommonMethods.copy( values ); }
  /***************** SETTINGS ABOUT COORDINATING ARS AND IDN *******************/
  public void setOverrideByIdn( boolean value ) { overrideByIdnOn = value; }
  public void setCombineDeterMode(int value){ combineDeterMode=value;}
  public void setAcsCrossCombineModes(short[][] values ){ acsCrossCombModes = CommonMethods.copy( values ); }
  public void setAcsCrossCombineMode(int netIdx, short[] values)
  { acsCrossCombModes[netIdx] = CommonMethods.copy( values ); }
  public void setAcsCrossCombineMode(int netIdx,int modelIdx, int value)
  { acsCrossCombModes[netIdx][modelIdx] = (short)value; }
  public void setAcsCrossCombineParas(double[][] values ){ acsCrossCombParas = CommonMethods.copy( values ); }
  public void setAcsCrossCombineParas(int netIdx,double[] values){acsCrossCombParas[netIdx]=values;}
  public void setAcsCrossCombineVarParaAboves(double[][] values ){ c3s = CommonMethods.copy( values ); }
  public void setAcsCrossCombineVarParaAboves(int netIdx,double[] values){ c3s[netIdx]=values;}
  public void setAcsCrossCombineVarParaUnders(double[][] values ){  c4s = CommonMethods.copy( values ); }
  public void setAcsCrossCombineVarParaUnders(int netIdx, double[] values ){ c4s[netIdx]=values;}
  public void setCorMultiExtMode( int value ){ corMultiExtMode = value; }
 /************** SETTINGS ABOUT NON-ACTION-CENTERED SUBSYSTEM *****************/
 /******************** SETTINGS ABOUT NACS COMPONENTS *************************/
  public void setGKSon( boolean value ){ gksOnFlag = value; }
  public void setAmnNum( int value ){ amnNum = value; }
  public void setEMon( boolean value ){ emOnFlag = value; }
  public void setAEMon( boolean value ){ aemOnFlag = value; }
 /******************** SETTINGS ABOUT GKS REPRESENATIONS **********************/
  public void setGksInputDimVals( short[] values ){ gksInputDimVals = CommonMethods.copy( values ); }
  public void setGksOutputDimVals( short[] values ){ gksOutputDimVals = CommonMethods.copy( values ); }
 // include each feature's type and order in that type.
  public void setGksInputFeatureInfo( short[][] values ){ gksInputFeatureInfo=values; }
  // include each feature's type and order in that type.
  public void setGksOutputFeatureInfo( short[][] values ){ gksOutputFeatureInfo=values; }
  public void setGksMaxChunkNum( int value ){ gksMaxChunkNum = value; }
  // chunk selection temperature.
  public void setGksOutputTemperature( double value ){ gksOutputTemperature = value; }
  /******************* the following for NACS rules ********************/
  public void setESChunkOn( boolean value ){ esChunkOn = value; }
  public void setSimilarityOn( boolean value ){ similarityOn = value; }
  public void setAssocBlaOn( boolean value ){ assocBlaOn = value; }
  public void setNacsChunkBlaOn( boolean value ){ nacsChunkBlaOn = value; }
  public void setAssocConditionWeightMode( int value ){ assocConditionWeightMode = value; }
  public void setAssocDimensionWeightMode( int value ){ assocDimensionWeightMode = value; }
  public void setAssocBlaOption( int value ){ assocBlaOption = value; }
  public void setAssocBla( double value ){ assocBla = value; }
  public void setAssocBlaParas( double[] values ){ assocBlaParas = CommonMethods.copy( values ); }
  public void setChunkBlaOption( int value ){ chunkBlaOption = value; }
  public void setChunkBla( double value ){ chunkBla = value; }
  public void setChunkBlaParas( double[] values ){ chunkBlaParas = CommonMethods.copy( values ); }
  public void setEmBlaOptions( short[] values ){ emBlaOptions = CommonMethods.copy( values ); }
  public void setEmBlas( double[] values ){ emBlas = CommonMethods.copy( values ); }
  public void setEmBlaParas( double[][] values ){ emBlaParas = CommonMethods.copy( values ); }
  public void setThreshold_EM( double value ){ threshold_EM = value; }
  public void setEmCapacity( int value ){ emCapacity = value; }
  public void setEmTrainAcsProb( double value ){ emTrainAcsProb = value; }
  public void setEmTrainAcsItemNum( int value ){ c9 = value; }
  public void setEmTrainAcsIterNum( int value ){ c10 = value; }
  public void setThreshold_sim( double value ){ threshold_sim = value; }
  public void setSBR_BalConstant( double value ){ c14 = value; }
  public void setRBR_BalConstant( double value ){ c15 = value; }
 /*********************** SETTINGS ABOUT GKS LEARNINGS ************************/
  public void setAssocExtractOn( boolean value ){ assocExtractOn = value; }
  public void setAssocEncodeExternOn( boolean value ){ assocEncodeExternOn = value; }
  public void setChunkEncodeExternOn( boolean value ){ chunkEncodeExternOn = value; }
  public void setChunkFromAmnOn( boolean value ){ chunkFromAmnOn = value; }
  public void setChunkEncodeProb( double value ){ p_c = value; }
  public void setAssocEncodeProb( double value ){ p_a = value; }
  public void setChunkDensity( double value ){ d_c = value; }
  public void setAssocDensity( double value ){ d_a = value; }
  public void setAssocExtThreshold( double value ){ threshold_ae = value; }
  public void setChunkExtThreshold( double value ){ threshold_ce = value; }
  public void setAssocExtProb( double value ){ p_ae = value; }
  public void setChunkExtProb( double value ){ p_ce = value; }
  public void setActivationMeasure( int value ){ activationMeasure = value; }
 /******************** SETTINGS ABOUT AMN REPRESENATIONS **********************/
 /*-------- AMN SETTINGS --------*/
  public void setAmnInputDimVals( short[][] values ){ amnInputDimVals = CommonMethods.copy( values ); }
  public void setAmnInputDimVals( int netIdx, short[] values ){ amnInputDimVals[ netIdx ] = CommonMethods.copy( values ); }
  public void setAmnOutputDimVals( short[][] values ){ amnOutputDimVals = CommonMethods.copy( values ); }
  public void setAmnOutputDimVals( int netIdx, short[] values ){ amnOutputDimVals[ netIdx ] = CommonMethods.copy( values ); }
 // include each feature's type and order in that type.
  public void setAmnInputFeatureInfo( short[][][] values ){ amnInputFeatureInfo=values; }
  public void setAmnInputFeatureInfo( int netIdx, short[][] values ){ amnInputFeatureInfo[netIdx]=values; }
  // include each feature's type and order in that type.
  public void setAmnOutputFeatureInfo(short[][][] values){ amnOutputFeatureInfo=values; }
  public void setAmnOutputFeatureInfo(int idx,short[][] values){ amnOutputFeatureInfo[idx]=values; }
  public void setAmnHiddenNums( short[] values ){ amnHiddenNums = CommonMethods.copy( values ); }
  public void setAmnHiddenNum( int netIdx, int value ){ amnHiddenNums[netIdx] = (short)value; }
  //public void setAmnInputChunkTypes( short[] values ){ amnInputChunkTypes = CommonMethods.copy( values ); }
  //public void setAmnInputChunkType( int idx, int value ){ amnInputChunkTypes[ idx ] = (short)value; }
  //public void setAmnOutputChunkTypes( short[] values ){ amnOutputChunkTypes = CommonMethods.copy( values ); }
  //public void setAmnOutputChunkType( int idx, int value ){ amnOutputChunkTypes[idx] = (short)value; }
  public void setAmnMultiPassOns( boolean[] values ){ amnMultiPassOns = CommonMethods.copy( values ); }
  public void setAmnMultiPassOn( int idx, boolean value ){ amnMultiPassOns[ idx ] = value; }
  public void setAmnMultiFlagOn( boolean value ) { amnMultiFlagOn = value; }
 /*-------- AMN TRAINING SETTINGS --------*/
  public void setAmnPassNums( short[] values ){ amnPassNums = CommonMethods.copy( values ); }
  public void setAmnPassNum( int idx, int value ){ amnPassNums[ idx ] = (short)value; }
  public void setAmnTrainedByEmProb( double value ){ amnTrainedByEmProb = value; }
  public void setAmnTrainedByEmItemType( int value ){ amnTrainedByEmItemType = value; }
  public void setAmnTrainedByEmItemNum( int value ){ amnTrainedByEmItemNum = value; }
  public void setAmnTrainedByEmIterNum( int value ){ amnTrainedByEmIterNum = value; }
 /*-------- AMN BP SETTINGS --------*/
  public void setAmnBPLearningRates( double[] values ){ amnLRs = CommonMethods.copy( values ); }
  public void setAmnBPLearningRate( int idx, double value ){ amnLRs[ idx ] = value; }
  public void setAmnBPMomentums( double[] values ){ amnMoms = CommonMethods.copy( values ); }
  public void setAmnBPMomentum( int idx, double value ){ amnMoms[ idx ] = value; }
 /*-------- AEM SETTINGS --------*/
  public void setAemStateNumBound( int value ){ aemStateNumBound = value; }
  public void setAemActionNumBound( int value ){ aemActionNumBound = value; }
  public void setAemPotentialNextStateNum( int value ){ aemPotentialNextStateNum = value; }
  public void setAemReinIncrement( double value ){ aemReinIncrement = value; }
  public void setAemReinRangeLowerBound( double value ){ aemReinRangeLowerBound = value; }
  public void setAemReinRangeUpperBound( double value ){ aemReinRangeUpperBound = value; }

  public void setAemTrainedByEmProb( double value ){ aemTrainedByEmProb = value; }
  public void setAemTrainedByEmItemNum( int value ){ aemTrainedByEmItemNum = value; }
  public void setAemTrainedByEmIterNum( int value ){ aemTrainedByEmIterNum = value; }
  public void setAemTrainedByEmBPLearningRate( double value ){ aemTrainedByEmLR = value; }
  public void setAemTrainedByEmBPMomentum( double value ){ aemTrainedByEmMom = value; }
  public void setAemTrainAcsProb( double value ){ aemTrainAcsProb = value; }
  public void setAemTrainAcsSampleNum( int value ){ aemTrainAcsSampleNum = value; }
  public void setAemTrainAcsIterNum( int value ){ aemTrainAcsIterNum = value; }
  public void setAemTrainAcsOutputTemp( double value ){ aemTrainAcsOutputTemp = value; }
 /**************** SETTINGS ABOUT COORDINATING ACS and NACS *******************/
  public void setCorAcsNacsType( int value ) { corAcsNacsType = value; }
  /** forward chaining, simi-based forward. */
  public void setGksReasoningMethod( int value ){ gksReasoningMethod = value; }
  public void setGksReasonIterNum( int value ){ gksReasonIterNum = value; }
  public void setGksReasonByMCSOn( boolean value ){ gksReasonByMCSOn = value; }
  public void setReasonThreshold( double value){ threshold_r = value; }
 /****************** SETTINGS ABOUT SUPERVISORY SUBSYSTEM *********************/
  public void setMsOn( boolean value ){ msOn = value; }
  public void setMcsOn( boolean value ){ mcsOn = value; }
  public void setMS( MS value ){ ms = value; }
  public void setMCS( MCS value ){ mcs = value; }
  public void setMonitorAcsTopConcNum( int value ){  c20 = value; }
  public void setMonitorAcsBottomConcNum( int value ){  c24 = value; }
  public void setMonitorAcsTopStepNum( int value ){  c22 = value; }
  public void setMonitorAcsBottomStepNum( int value ){  c26 = value; }
  public void setMonitorAcsTopEpisodeNum( int value ){  c28 = value; }
  public void setMonitorAcsBottomEpisodeNum( int value ){  c30 = value; }
  public void setMonitorNacsTopConcNum( int value ){  c21 = value; }
  public void setMonitorNacsBottomConcNum( int value ){  c25 = value; }
  public void setMonitorNacsTopStepNum( int value ){  c23 = value; }
  public void setMonitorNacsBottomStepNum( int value ){  c27 = value; }
  public void setMonitorNacsTopEpisodeNum( int value ){  c29 = value; }
  public void setMonitorNacsBottomEpisodeNum( int value ){  c31 = value; }
 /*************** SETTINGS ABOUT RESPONSE TIME DETERMINATION ******************/
  public void setRTOn( boolean value ){ rtOn = value; }
  //public void setMaxUsageNum( int value ){ maxUsageNum = value;  }
  public void setOverallFormula( String value ){ overallFormula = value; }
  public void setBLFormula( String value ){ blFormula = value; }
  public void setTLFormula( String value ){ tlFormula = value; }
  public void setPT_BL( int value ){ PT_BL = value; }
  public void setDT_BL( int value ){ DT_BL = value; }
  public void setAT_BL( int value ){ AT_BL = value; }
  public void setPT_TL( int value ){ PT_TL = value; }
  // used for calculating DT_TL = DT_TL_actioTime + t0/rule_bla + t1/chunk_bla;
  public void setDT_TL_actionTime( int value ){ DT_TL_actionTime = value; }
  public void setDT_TL_t0( double value ){ t0 = value; }
  public void setDT_TL_t1( double value ){ t1 = value; }
  public void setAT_TL( int value ){ AT_TL = value; }
  public void setAssoAppTimeFormula( String value ){ assocAppTimeFormula = value; }
  public void setAssocAppTime_t4( double value ){ t4 = value; }
  public void setAssocAppTime_t5( double value ){ t5 = value; }
  public void setChunkRetTimeFormula( String value ){ chunkRetTimeFormula = value; }
  public void setChunkRetTime_t2( double value ){ t2 = value; }
  public void setChunkRetTime_t3( double value ){ t3 = value; }
  public void setAmnOnePassTime( int value ){ t_AMN = value; }
 /************************** SETTINGS ABOUT INTERFACE *************************/
  public void setRuleDisplayMode( int value ){ ruleDisplayMode = value; }
  public void setRuleDisplayFreq( int value ){ ruleDisplayFreq = value; }
 /************************ SETTING ABOUT TASK-SPECIFIC ************************/
  public void setLearningTestOn( boolean value ){ learningTestOn = value; }
  public void setEpisodeLength( int value ){ episodeLength = value; }
  public void setAgentGroupId( int value ){ agentGroupId = value; }
  public void setTaskAgent( TaskAgent value ){ taskAgent = value; }
  public void setACS( ACS value ){ acs = value; }
  public void setNACS( NACS value ){ nacs = value; }
  public void setTaskSettingName( String value ){ taskSettingName = value; }
  static public void setRootDir( String value ){ rootDir = value; }
  static public void setSettingDir( String value ){ settingDir = value; }
  static public void setDataDir( String value ){ dataDir = value; }
  static public void setTask( Task value ){ task = value; }
  static public void setTaskClarionClass( Class value ){ taskClarionClass = value; }
  public void setTaskClarion( TaskClarion value ){ taskClarion = value; }
  static public void setTaskDir( String value ){ taskDir = value; }
  static public void setTaskName( String value ){ taskName = value; }
  static public void setTaskPackageName( String value ){ taskPkgName = value; }
  static public void setTaskClassName( String value ){ taskClsName = value; }
  static public void setSettingMode( boolean value ) { guiFlag = value; }
  public void setInputFormat( boolean[] values ) { inputFormat = CommonMethods.copy( values ); };
  public void setOutputFormat( boolean[] values ) { outputFormat = CommonMethods.copy( values ); }

 /*=========================== INITIALIZATION CODES ===========================*/
  /** initialize the variables relevant to ACS networks.*/
  public void initAcsNets()
  {
    idnInputDimVals = new short[acsNetNum][];
    idnOutputDimVals = new short[acsNetNum][];
    idnInputFeatureInfo = new short[acsNetNum][][];
    idnOutputFeatureInfo = new short[acsNetNum][][];
    //String  exIdnEligibleConditions[];
    idnHiddenNums = new short[acsNetNum];
    idnBLLearningOptions = new short[acsNetNum];
    idnBlAssimFreqs = new short[acsNetNum][];
    idnBPLRs = new double[acsNetNum];
    idnBPMoms = new double[acsNetNum];
    idnQDiscountRates = new double[acsNetNum];
    idnActionSelTemps = new double[acsNetNum];
    overallActionSelTemps = new double[acsNetNum];
    idnRLFuncOpts = new short[acsNetNum];
    //String  overallIdnRLFunction;
    //String  idnRLFunctions[];
    idnRLFuncByMCSs = new boolean[acsNetNum];
    arInputDimVals = new short[acsNetNum][ACS_KNOW_TYPE_NUM][];
    arOutputDimVals = new short[acsNetNum][ACS_KNOW_TYPE_NUM][];
    arInputFeatureInfo = new short[acsNetNum][ACS_KNOW_TYPE_NUM][][];
    arOutputFeatureInfo = new short[acsNetNum][ACS_KNOW_TYPE_NUM][][];
    arActionSelTemps = new double[acsNetNum][ACS_KNOW_TYPE_NUM];
    arReasoningMethods = new short[acsNetNum][ACS_KNOW_TYPE_NUM];
   /********************* SETTINGS ABOUT RULE LEARNING **************************/
    acsKnowledgeNum = 4;
    rerOns = new boolean[acsNetNum];
    irlOns = new boolean[acsNetNum];
    extInstructOns = new boolean[acsNetNum];
    ilOns = new boolean[acsNetNum];
    frOns = new boolean[acsNetNum];
    frActionVariedOns = new boolean[acsNetNum][];
    maxRerRuleNum = 500;
    //String  frSets[][];
    frConditions = new short[acsNetNum][][][];
    frActions = new short[acsNetNum][][][][];
    frNums = new short[acsNetNum];
    irlSetNums = new short[acsNetNum];
    //String  irlSets[][][];
    irlRuleNums = new short[acsNetNum][];
    irlRuleParams = new double[acsNetNum][][][];
    irlGenOns = new boolean[acsNetNum];
    irlSpecOns = new boolean[acsNetNum];
    d_is = new double[acsNetNum];
    d_rs = new double[acsNetNum];
    p_res = new double[acsNetNum];
    rerGenSpecModes = new short[acsNetNum];
    irlGenSpecModes = new short[acsNetNum];
    //String  rerPosCriterions[];
    //String  irlPosCriterions[];
    threshold_RERs = new double[acsNetNum];
    threshold_IRLs = new double[acsNetNum];
    //String  rerIGMeasures[];
    //String  irlIGMeasures[];
    c1s = new double[acsNetNum];
    c2s = new double[acsNetNum];
    c5s = new double[acsNetNum];
    c6s = new double[acsNetNum];
    threshold_PMs = new double[acsNetNum];
    threshold1s = new double[acsNetNum];
    threshold2s = new double[acsNetNum];
    threshold6s = new double[acsNetNum];
    threshold7s = new double[acsNetNum];
    threshold4s = new double[acsNetNum];
    rerGsValueRanges = new short[acsNetNum];
    irlGsValueRanges = new short[acsNetNum];
    ruleMergeIntervals = new short[acsNetNum];
   /******************* SETTINGS ABOUT RULE REPRESENTATIN ***********************/
    ruleUtilityOns = new boolean[acsNetNum][ACS_KNOW_TYPE_NUM][];
    rulePartMatchOns = new boolean[acsNetNum][ACS_KNOW_TYPE_NUM][];
    ruleBlaOns = new boolean[acsNetNum][ACS_KNOW_TYPE_NUM][];
    actionChunkBlaOns = new boolean[acsNetNum][ACS_KNOW_TYPE_NUM][];
    ruleWeightModes = new short[acsNetNum][ACS_KNOW_TYPE_NUM][];
    ruleConditionWeightModes = new short[acsNetNum][ACS_KNOW_TYPE_NUM][];
    actionRuleBlaOptions = new short[acsNetNum][ACS_KNOW_TYPE_NUM][];
    actionRuleBlas = new double[acsNetNum][ACS_KNOW_TYPE_NUM][];
    actionRuleBlaParas = new double[acsNetNum][ACS_KNOW_TYPE_NUM][][];
    ruleUtilityOptions = new short[acsNetNum][ACS_KNOW_TYPE_NUM][];
    ruleUtilityParas = new double[acsNetNum][ACS_KNOW_TYPE_NUM][][];
    acsChunkBlaOptions = new short[acsNetNum][ACS_KNOW_TYPE_NUM][];
    acsChunkBlas = new double[acsNetNum][ACS_KNOW_TYPE_NUM][];
    acsChunkBlaParas = new double[acsNetNum][ACS_KNOW_TYPE_NUM][][];
 /***************** SETTINGS ABOUT COORDINATING ARS AND IDN *******************/
    acsCrossCombModes = new short[acsNetNum][2];
    acsCrossCombParas = new double[acsNetNum][];
    c3s = new double[acsNetNum][ACS_KNOW_TYPE_NUM];
    c4s = new double[acsNetNum][ACS_KNOW_TYPE_NUM];
  }

  /** initialize the variables relevant to ACS action rules.*/
  public void initAcsRules( int netIdx, int knowType, int ruleSetNum )
  {
   /******************* SETTINGS ABOUT RULE REPRESENTATIN ***********************/
    ruleUtilityOns[netIdx][knowType] = new boolean[ruleSetNum];
    rulePartMatchOns[netIdx][knowType] = new boolean[ruleSetNum];
    ruleBlaOns[netIdx][knowType] = new boolean[ruleSetNum];
    actionChunkBlaOns[netIdx][knowType] = new boolean[ruleSetNum];
    ruleWeightModes[netIdx][knowType] = new short[ruleSetNum];
    ruleConditionWeightModes[netIdx][knowType] = new short[ruleSetNum];
    actionRuleBlaOptions[netIdx][knowType] = new short[ruleSetNum];
    actionRuleBlas[netIdx][knowType] = new double[ruleSetNum];
    actionRuleBlaParas[netIdx][knowType] = new double[ruleSetNum][];
    ruleUtilityOptions[netIdx][knowType] = new short[ruleSetNum];
    ruleUtilityParas[netIdx][knowType] = new double[ruleSetNum][];
    acsChunkBlaOptions[netIdx][knowType] = new short[ruleSetNum];
    acsChunkBlas[netIdx][knowType] = new double[ruleSetNum];
    acsChunkBlaParas[netIdx][knowType] = new double[ruleSetNum][];
    //String  frSets[][];
    if( knowType == FR ){
        frConditions[netIdx] = new short[ruleSetNum][][];
        frActions[netIdx] = new short[ruleSetNum][2][][];
    }
    //String  irlSets[][][];
    if( knowType == IRL ){
        irlRuleNums[netIdx] = new short[ruleSetNum];
        irlRuleParams[netIdx] = new double[ruleSetNum][][];
    }
  }

  /** initialize the variables relevant to NACS networks.*/
  public void initNacsNets()
  {
    amnInputChunkTypes = new short[amnNum];
    amnOutputChunkTypes = new short[amnNum];
    amnInputDimVals = new short[amnNum][];
    amnOutputDimVals = new short[amnNum][];
    amnInputFeatureInfo = new short[amnNum][][];
    amnOutputFeatureInfo = new short[amnNum][][];
    amnHiddenNums = new short[amnNum];
    amnMultiPassOns = new boolean[amnNum];
    amnPassNums = new short[amnNum];
    amnLRs = new double[amnNum];
    amnMoms = new double[amnNum];
  }
}
