package main;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Vector;
import clarion.system.*;
import clarion.common.*;
import clarion.options.*;
import clarion.tools.*;
import tasksimulators.*;
//import tasksimulators.oldsimu.*;


/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is the main class of the whole CLARION software system. It includes
 * both the simulated tasks of old version and the simulated tasks of new version.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public static void main( String args[] ): the entry of the whole system to
 * start.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is on the top of the whole CLARION software system and is the start
 * point of the whole software system to run. It contains a task simulator to
 * control the simulation process of a specific human task.
 * <li> Usage of this class. <br>
 * It is the running start of the whole software and instantiated by the Java
 * system.
 * </ul>
 * @author Xi Zhang
 *
 */

public class ClarionMain extends JFrame {
/***************************VARIABLES DECLARATION BEGIN************************/
   /** constants for designing window in which results are shown. */
   final int WINDOW_WIDTH = 430,  WINDOW_HEIGHT = 520;
   final int OLD = 0, NEW = 1;
   public final static String CLARION_TASK_TEXT[] ={
       "PROCESS CONTROL TASK", "LETTER COUNTING TASK", "TOWER OF HANOI TASK",
       "CATEGORY INFERENCE TASK", "DISCOVERY TASK", "ARTIFICIAL GRAMMAR LEARNING TASK"
   };
   int simuType;
   /** the name of the loaded task. */
   String loadTaskName;
   /** the names of the tasks in the new version. */
   Vector newTaskNames = new Vector();
   JFrame thisFrame;
   /** caption label. */
   JLabel capLabel = new JLabel();
   /** panel for old version. */
   JPanel oldTaskPanel = new JPanel();
   /** label for old version. */
   JLabel oldTaskLabel = new JLabel();
   /** buttons for old version. */
   JButton oldTaskButtons[];
   /** panel for new version. */
   JPanel newTaskPanel = new JPanel();
   /** label for new version. */
   JLabel newTaskLabel = new JLabel();
   JLabel loadLabel = new JLabel();
   /** the button to load. */
   JButton loadButton = new JButton();
   JButton newTaskButton = new JButton();
   //JButton clarionButton = new JButton();
   JPopupMenu loadTaskMenu;
   JMenuItem loadTaskItem[];
   /** button for exit. */
   JButton exitButton = new JButton();
   /** the new task. */
   Task  newTask;
   /** the assoicated task-specific CLARION settings. */
   TaskClarion taskClarion;
   /** the old simulators. */
   Simulator oldSimulators[];
   /** the new simulators. */
   NewSimulator newSimulator;

 public ClarionMain() {

     System.out.println( System.getProperty("user.dir") );
     thisFrame = this;
     initGui();
 }

 /** Initializes the GUI (Graphical User Interface). */
 private void initGui()
 {
     final int WIDTH = WINDOW_WIDTH - 30;
     setSize( WINDOW_WIDTH, WINDOW_HEIGHT );
     this.getContentPane().setLayout(null);
     // the caption.
     capLabel.setFont(new java.awt.Font("Dialog", 0, 18));
     capLabel.setText("CLARION  MODEL");
     capLabel.setBounds(new Rectangle(110, 2, 160, 26));
     // new simulation panel.
     newTaskPanel.setLayout( null );
     newTaskPanel.setBorder( BorderFactory.createLineBorder(Color.black) );
     newTaskPanel.setBounds( new Rectangle(10, 30, WIDTH, 160) );
     newTaskLabel.setText( "Simulating a new task," );
     newTaskLabel.setBounds( new Rectangle((WIDTH-200)/2, 4, 200, 20) );
     loadLabel.setText( "No task loaded" );
     loadLabel.setBounds( new Rectangle(230, 40, 160, 20) );
     loadButton.setText( "LOAD A TASK" );
     loadButton.setBounds(new Rectangle(100, 40, 120, 20));
     loadButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
           loadButton_actionPerformed( e );
        }
     });
     newTaskButton.setText( "NEW TASK SIMULATOR" );
     newTaskButton.setBounds(new Rectangle((WIDTH-200)/2, 80, 200, 20));
     newTaskButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
          newTaskButton_actionPerformed(e);
        }
     });
     newTaskButton.setEnabled( false );
     /*
     clarionButton.setText("CLARION SETTINGS");
     clarionButton.setBounds(new Rectangle((WIDTH-200)/2, 120, 200, 20));
     clarionButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
          clarionButton_actionPerformed(e);
        }
     });
     clarionButton.setEnabled( false );
     */
     // old simulation panel.
     oldTaskPanel.setLayout( null );
     oldTaskPanel.setBorder( BorderFactory.createLineBorder(Color.black) );
     oldTaskPanel.setBounds( new Rectangle(10, 200, WIDTH, 240) );
     oldTaskLabel.setText( "Old simulation" );
     oldTaskLabel.setBounds( new Rectangle((WIDTH-180)/2, 4, 180, 20) );
     OldTaskButtonHandler handler = new OldTaskButtonHandler();
     oldTaskButtons = new JButton[ World.CLARION_TASK_NUM ];
     for( int i = 0; i < World.CLARION_TASK_NUM; i++ ){
         oldTaskButtons[i] = new JButton( CLARION_TASK_TEXT[i] );
         oldTaskButtons[i].setBounds(new Rectangle((WIDTH-260)/2, 40+40*i, 260, 20));
         //oldTaskButtons[i].setEnabled( true );
         oldTaskButtons[i].setEnabled( false );
         oldTaskButtons[i].addActionListener( handler );
         oldTaskPanel.add( oldTaskButtons[i] );
     }
     exitButton.setText("EXIT");
     exitButton.setBounds(new Rectangle((WIDTH-100)/2, 460, 100, 20));
     exitButton.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(ActionEvent e) {
          exitButton_actionPerformed(e);
      }
     });
     // add components.
     newTaskPanel.add( newTaskLabel );
     newTaskPanel.add( loadLabel );
     newTaskPanel.add( loadButton );
     newTaskPanel.add( newTaskButton );
     //newTaskPanel.add( clarionButton );
     oldTaskPanel.add( oldTaskLabel );
     this.getContentPane().add(capLabel, null);
     this.getContentPane().add(newTaskPanel, null);
     this.getContentPane().add(oldTaskPanel, null);
     this.getContentPane().add(exitButton, null);
     validate();
     setEnabled( true );
     setVisible( true );
     // for old task simulatons.
     //setModelParaSet();
     //setTaskParaSet();
 }

 /** the entry of the whole system to start. */
 public static void main( String args[] )
 {
    ClarionMain aDemo = new ClarionMain();
 }

  /** Search current simulated tasks.
   *  @param names the array to store the simulated task names. */
  public int searchCurrentTasks( Vector names )
  {
     String path = Global.getTaskDir();
     File file = new File( path );
     int len = file.list().length;
     int count = 0;
     for( int i = 0; i < len; i++ )
     {
    	 //Ignore Hidden Mac Directories
    	 if(file.list()[i].indexOf(".DS_Store") == -1)
    	 {
          names.addElement( file.list()[i] );
    	  count ++;
    	 }
     }
     return count;
  }

/********************************PRIVATE METHODS *******************************/
  /** task selecting buttons handler. */
  private class OldTaskButtonHandler implements ActionListener
  {
     public void actionPerformed( ActionEvent e ){
       for( int i = 0; i < World.CLARION_TASK_NUM; i++ )
          if( e.getSource() == oldTaskButtons[ i ] ){
              /*
              simuType = OLD;
              taskNo = i;
              setModelParaSet();
              setTaskParaSet();
              setEnabled( false );
              setVisible( false );
              oldSimulators[taskNo].setEnabled(true);
              oldSimulators[taskNo].setVisible(true);
              */
          }
     }
  }

  /** the routine for processing the loading button events. */
  private void loadButton_actionPerformed(ActionEvent e)
  {
     loadTaskName = Global.getTaskClassName();
     int currentTaskNum = searchCurrentTasks( newTaskNames );
     loadTaskMenu = new JPopupMenu( "EXISTING TASKS" );
     loadTaskItem = new JMenuItem[ currentTaskNum ];
     for( int i = 0; i < currentTaskNum; i++ ){
          loadTaskItem[i] = new JMenuItem( (String)(newTaskNames.elementAt(i)) );
          loadTaskItem[i].setFont( new Font( "Serif", Font.PLAIN, 14 ) );
          final int focus = i;
          loadTaskItem[ focus ].addActionListener( new ActionListener(){
             public void actionPerformed( ActionEvent e){
                 if( !newTaskNames.elementAt(focus).equals( loadTaskName ) &&
                     JOptionPane.showConfirmDialog( null, "want to load this task?",
                         "WARNING", JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION ) {
                     loadTaskName = (String)(newTaskNames.elementAt(focus));
                     loadLabel.setText( loadTaskName  + " is loaded" );
                     Global.setTaskName( loadTaskName );
                     newTaskButton.setEnabled( true );
                     //clarionButton.setEnabled( true );
                     ClarionClassLoader classLoader = new ClarionClassLoader();
                     String loadTaskDir = Global.getTaskDir() + loadTaskName;
                     try{
                          System.out.println( loadTaskDir );
                          String[] classNames = new File( loadTaskDir ).list();

                          //System.out.println( "class len = !" + classNames.length );
                          


                          for( int i = 0; i < classNames.length; i++ ){
                              //Skip over classes which have $ in them (i.e. enumerators)
                              if(classNames[i].indexOf("$") != -1 || classNames[i].indexOf(".DS_Store") != -1)
                              {
                            	  continue;  
                              }
                              String className = classNames[i].substring(0, classNames[i].length()-6 );

                              //System.out.println( "obj name = " + className );

                              String fullClassName = Global.getTaskPackageName() + loadTaskName +"."+className;

                              //System.out.println( "full name = " + fullClassName );

                              Object obj = classLoader.loadClass( fullClassName ).newInstance();

                              //System.out.println( "obj = " + obj );

                              if( obj instanceof Task ){     // load Task
                                  newTask = (Task)obj;
                                  Global.setTaskClassName( className );
                              }
                              if( obj instanceof TaskClarion ){   // load TaskClarion
                                  taskClarion = (TaskClarion)obj;
                                  //Global.setTaskClarion( taskClarion );
                                  Global.setTaskClarionClass( taskClarion.getClass() );
                                  System.out.println( "samples are loaded!" );
                              }
                          }
                          Global.setTask( newTask );
                          newSimulator = new NewSimulator( (ClarionMain)thisFrame, (SimuTask)newTask );

                     }
                     catch( Exception e1 ){
                            System.out.println( e1 );
                            System.out.println("Nope, I Bwoke");
                     }
                }
            }
          } );
          loadTaskMenu.add( loadTaskItem[ i ] );
     }
     loadTaskMenu.show( loadButton, 10, 10 );
  }

  /** the routine for processing the 'new task' button events. */
  private void newTaskButton_actionPerformed(ActionEvent e)
  {
       simuType = NEW;
       setEnabled( false );
       setVisible( false );
       newSimulator.setEnabled(true);
       newSimulator.setVisible(true);
  }

  /*
  private void clarionButton_actionPerformed(ActionEvent e)
  {
       simuType = NEW;
       guiClarion.setMainFrame( thisFrame );
       setEnabled( false );
       setVisible( false );
       guiClarion.setEnabled( true );
       guiClarion.setVisible( true );
  }
  */

  /** the routine for processing the 'exit' button events. */
  private void exitButton_actionPerformed(ActionEvent e) {
       System.exit( 0 );
  }
}