package tasksimulators.newsimu.AGL;
import clarion.system.*;
import clarion.common.*;
import clarion.nacs.*;
import java.util.Vector;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to simulate one subject in Artificial Grammarical Learning task.
 * It extends the <tt>TaskAgent</tt> class.
 * <li> important methods <br>
 * <ol>
 *  <li> public void run() : implements the overall process of a subject during
 *  the task experiment. It is assumed to be called by the <tt>TaskAGL</tt> class
 *  to implement what a subject have done during the experiment.
 *  <li> public void performAction( int netIdx, short[][][] action ) : perform
 *  the selected action from a particular ACS network.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model. <br>
 * This class implemnts the Artificial Grammarical Learning task using CLARION and
 * part of AGL task simulator.
 * <li> Usage of this class. <br>
 *  Routines in this class will be called by the system when the AGL task is loaded
 *  for simulation.
 * </ul>
 * @author Xi Zhang
 *
 **/

public class AgentAGL extends TaskAgent{

  final int MAX_STR_LEN = 11, MIN_STR_LEN = 5, SAMPLE_NUM = 177;
  final int INPUT_DIM_NUM = MAX_STR_LEN + 2, OUTPUT_DIM_NUM = MAX_STR_LEN + 1;
  final int LETTER_DIM_LEN = 7, LEARN_TYPE_NUM = 4, STAGE_NUM = 2, POSITION_NUM = MAX_STR_LEN + 1;
  final int SESSION_TIME = 1200000, SESSION_NUM = 2, NUM_OF_MINUTE = 20;
  final int CONTROL_TYPE = 0, IMPLICIT_TYPE = 1;
  final int EXPLICIT_TYPE = 2, SYNERGY_TYPE = 3;
  final int TRAIN = 0, TEST = 1;
  //final double LOCK_PROB = 0.8;
  final double LOCK_PROB = 0.75;
  static short[][][] trainSet;
  static short[][][] testSet;
  char[] groupType;  // one of the groups: "CC", "IE", "EI", "II", "EE", "SS"
  int sessionIdx, stageIdx;
  int learningType;
  int currentTime;
  int trainSampleNum;
  int testSampleNum;
  double[] achievement, quality, attempts;
  String letterFeatureName;
  String learnTypeFeatureName;
  String stageFeatureName;
  String positionFeatureName;
  short[][] nacsConditionInfo; {
     nacsConditionInfo = new short[1][INPUT_DIM_NUM-2];
     for( int i = 0; i < INPUT_DIM_NUM-2; i++)
          nacsConditionInfo[0][i] = (short)i;
  }

 public AgentAGL(){}

 public AgentAGL( Global g, short[][][] trainSet, short[][][] testSet )
 {
   super( g );
   this.trainSet = trainSet;
   this.testSet = testSet;
   trainSampleNum = trainSet.length;
   testSampleNum = testSet.length;
   reinforcements = new double[ 1 ];
   achievement = new double[ SESSION_NUM ];
   quality = new double[ SESSION_NUM ];
   attempts = new double[ SESSION_NUM ];
   letterFeatureName = global.getFeatureNames( Global.FT_INPUT, 0 );
   learnTypeFeatureName =  global.getFeatureNames( Global.FT_INPUT, INPUT_DIM_NUM - 2 );
   stageFeatureName =  global.getFeatureNames( Global.FT_INPUT, INPUT_DIM_NUM - 1 );
   positionFeatureName =  global.getFeatureNames( Global.FT_OUTPUT, OUTPUT_DIM_NUM - 1 );
 }

 public void reinit()
 {
    super.reinit();
    currentTime = 0;
    sessionIdx = stageIdx = 0;
    for( int i = 0; i < SESSION_NUM; i++ ){
         achievement[i] = 0.0;
         quality[i] = 0.0;
         attempts[i] = 0.0;
    }
 }

 public boolean isControlNacsAction( int netIdx, short[] chosenAction )
 {
    return true;
 }

 /** Get the data of achievement and populate the array.
  * @param arr the array to store the data. */
 public void getAchievement( double[] arr )
 {
    for( int i = 0; i < arr.length; i++ )
         arr[ i ] = achievement[i] / NUM_OF_MINUTE;
 }

 /** Get the data of Accuracy and populate the array.
  * @param arr the array to store the data. */
 public void getAccuracy( double[] arr )
 {
    for( int i = 0; i < arr.length; i++ )
         arr[ i ] = (100 * quality[i] / attempts[i]) / NUM_OF_MINUTE;
 }

 /** Get the data of Efficiency and populate the array.
  * @param arr the array to store the data. */
 public void getEfficiency( double[] arr )
 {
    for( int i = 0; i < arr.length; i++ )
         arr[ i ] = 100 * achievement[i] / attempts[i];
 }

 /** Get the data of Speed and populate the array.
  * @param arr the array to store the data. */
 public void getSpeed( double[] arr )
 {
    for( int i = 0; i < arr.length; i++ )
         arr[ i ] = attempts[ i ] / NUM_OF_MINUTE;
 }

 /** Get the data of Attempts and populate the array.
  * @param arr the array to store the data. */
 public void getAttempts( double[] arr )
 {
    for( int i = 0; i < arr.length; i++ )
         arr[ i ] = attempts[ i ];
 }

 public void getOutput( Object arr )
 {
 }

 public int getCurSensoryInputType()
 {
    return 0;
 }

 public String getCurSensoryInputLabel()
 {
    return "label";
 }

 public void setRetrievalResults( int netIdx, GKSChunk[] results )
 {
 }

 public void setGroupType( char[] groupType )
 {
    this.groupType = groupType;
 }

 public short[] decideIrlAction( int netIdx, int setIdx, int ruleNo, short[] input )
 {
    return (short[])null;
 }

 public short[] decideFrAction( int netIdx, int setIdx, short[] input )
 {
    return (short[])null;
 }

 public int determineAmnNet( int netIdx, short[][][] action )
 {
    activeAmnIdx = 0;
    return activeAmnIdx;
 }

 /** perform the selected action from a particular ACS network.
  * @param netIdx the ACS network index
  * @param action the selected action from the specified ACS network. */
 public void performAction( int netIdx, short[][][] action )
 {

   //System.out.println( "cur sensory input len = " + curNacsInput.length);

   ((TaskClarionAGL)taskClarion).setLearningType( learningType );
   ((TaskClarionAGL)taskClarion).resetNacsResults();
   if( stageIdx == TRAIN ){
       ((TaskClarionAGL)taskClarion).setTrainSample( curNacsInput );
       //((TaskClarionAGL)taskClarion).training( learningType );
   }
   if( stageIdx == TEST ){
       //decideLearningType();
       ((TaskClarionAGL)taskClarion).setTestSample( curNacsInput );
       //((TaskClarionAGL)taskClarion).testing( learningType );
   }
   super.performAction( netIdx, action );
   reinforcements[ 0 ] = taskClarion.getReinforcement( netIdx, curSensoryInput, null,
                                                       null, action, null, null, null );
 }

 /** Implements the overall process of a subject during the AGL task experiment.
  * It is assumed to be called by the <tt>TaskAGL</tt> class to implement what a
  * subject have done during the experiment. */
 public void run()
 {
   setupBeginFeatures();
   currentTime = 1000;
   clarion.setNacsConditionDimInfo( nacsConditionInfo );
   ((TaskClarionAGL)taskClarion).setGks();
   ((TaskClarionAGL)taskClarion).setAmns();
   //((TaskClarionAGL)taskClarion).createChunkMap();
   for( sessionIdx = 0; sessionIdx < SESSION_NUM; sessionIdx++ ){
        getLearningType( groupType[sessionIdx] );
        stageIdx = 0;
        training();
        stageIdx = 1;
        testing();
   }
 }

/*************************** PRIVATE METHODS *********************************/
 private void setupBeginFeatures()
 {
    Feature[][] features = new Feature[ 2 ][ 1 ];
    short[] values = new short[ 1 ];
    values[ 0 ] = 0;
    //int LETTER_DIM_VAL_NUM = 7;
    features[0][0] = new Feature( letterFeatureName, (short)Global.FT_INPUT, (short)0,
                        (short)LETTER_DIM_LEN, values, global );
    //int POSITION_DIM_VAL_NUM = 12;
    features[1][0] = new Feature( positionFeatureName, (short)Global.FT_OUTPUT, (short)(OUTPUT_DIM_NUM-1),
                        (short)POSITION_NUM, values, global );
    ((TaskClarionAGL)global.getTaskClarion()).setBeginFeatures( features );
 }

 /** Get the learning type given a letter. */
 private void getLearningType( char ch )
 {
    learningType = global.EMPTY;
    switch( ch ){
     case 'C':
          learningType = CONTROL_TYPE;
          break;
     case 'I':
          learningType = IMPLICIT_TYPE;
          break;
     case 'E':
          learningType = EXPLICIT_TYPE;
          break;
     case 'S':
          learningType = SYNERGY_TYPE;
          break;
    }
 }

 /** Perform the process of training. */
 private void training()
 {
   System.out.println( "in training" );

    Vector sands = new Vector( testSampleNum );
    for( int i = 0; i < trainSampleNum; i++ )
         sands.addElement( new Integer(i) );
    //int TRAIN_ITER_NUMS[] = { 1, 6, 6, 20 };
    //int TRAIN_ITER_NUMS[] = { 1, 7, 7, 18 };
    int TRAIN_ITER_NUMS[] = { 1, 7, 7, 7 };
    //int TRAIN_ITER_NUMS[] = { 1, 1, 1, 1 };
    for( int i = 0; i < TRAIN_ITER_NUMS[ learningType ]; i++ ){
         System.out.println( "current iter no = " + i );
         Vector s = (Vector)(sands.clone());
         for( int j = 0; j < trainSampleNum; j++ ){
              clarion.setCurrentTime( currentTime );
              int sampleIdx = ((Integer)(s.remove((int)(Math.random()*s.size())))).intValue();
              initAcsSensory( sampleIdx, trainSet );
              initNacsSensory( sampleIdx, trainSet );
              clarion.training();
              //currentTime += clarion.getACS().getRT();
              currentTime += clarion.getRT();
         }
    }
 }

 /** Perform the process of testing. */
 private void testing()
 {
   System.out.println( "in testing" );

    ((TaskClarionAGL)global.getTaskClarion()).resetTotSearchNum();

    Vector sands = new Vector( testSampleNum );
    for( int i = 0; i < testSampleNum; i++ )
         sands.addElement( new Integer(i) );
    int deltaTime = 0;
    int impNum = 0, expNum = 0;
    while( deltaTime < SESSION_TIME && !sands.isEmpty() ){
       clarion.setCurrentTime( currentTime );
       int sampleIdx = ((Integer)(sands.remove((int)(Math.random()*sands.size())))).intValue();
       decideLearningType();
       initAcsSensory( sampleIdx, testSet );
       initNacsSensory( sampleIdx, testSet );
       clarion.testing();
       // get retrieval results from clarion.
       Object obj = clarion.getRetrievalResults();
       nacsResults = ( obj instanceof Feature[] ) ? (Feature[])obj : null;
       attempts[ sessionIdx ]++;
       checkCompletion();

       if( learningType == IMPLICIT_TYPE )
           impNum++;
       if( learningType == EXPLICIT_TYPE )
           expNum++;

       int RT = clarion.getRT();
       deltaTime += RT;
       currentTime += RT;

       //System.out.println( "current attempt # = " + attempts[ sessionIdx ] +
       //                    ", current time = " + currentTime + ", delta time = " + deltaTime + ", RT = " + RT );
    }

    //System.out.println( "imp num = " + impNum );
    //System.out.println( "exp num = " + expNum );
 }

 /** init ACS input sensory. */
 private void initAcsSensory( int sampleIdx, short[][][] dataSet )
 {
   int DIM_NUM = global.getIdnInputDimNum( 0 );
   int DIM_VAL_NUM = global.getFeatureLength(Global.FT_INPUT,0);
   short[] con = new short[ DIM_VAL_NUM ];
   for( int i = 0; i < con.length; i++ )
        con[ i ] = 0;
   curSensoryInput = new Feature[ DIM_NUM ];
   // letter features.
   for( int i = 0; i < DIM_NUM-2; i++ ){
        String featureName = global.getFeatureNames( Global.FT_INPUT, i );
        if( i < dataSet[sampleIdx].length )
            curSensoryInput[ i ] = new Feature( featureName, (short)Global.FT_INPUT,
                     (short)i, dataSet[sampleIdx][i], global );
        else
            curSensoryInput[ i ] = new Feature( featureName, (short)Global.FT_INPUT,
                     (short)i, con, global );
   }
   // learning type feature.
   short[] learnTypeValues = new short[global.getFeatureLength(Global.FT_INPUT, DIM_NUM - 2)];
   for( int i = 0; i < learnTypeValues.length; i++ )
        learnTypeValues[ i ] = 0;
   learnTypeValues[ learningType ] = 1;
   curSensoryInput[ DIM_NUM-2 ] = new Feature( learnTypeFeatureName, (short)Global.FT_INPUT,
                   (short)(DIM_NUM-2), learnTypeValues, global );
   // stage feature.
   short[] stageValues = new short[ global.getFeatureLength(Global.FT_INPUT, DIM_NUM - 1) ];
   for( int i = 0; i < stageValues.length; i++ )
        stageValues[ i ] = 0;
   stageValues[ stageIdx ] = 1;
   curSensoryInput[ DIM_NUM-1 ] = new Feature( stageFeatureName, (short)Global.FT_INPUT,
                   (short)(DIM_NUM-1), stageValues, global );
 }

 /** init NACS input sensory. */
 private void initNacsSensory( int sampleIdx, short[][][] dataSet )
 {
   int DIM_NUM = dataSet[sampleIdx].length;
    curNacsInput = new Feature[ DIM_NUM ];

    //System.out.println( "dim num = " + DIM_NUM + ", sample idx = " + sampleIdx);

    for( int i = 0; i < DIM_NUM; i++ )
         curNacsInput[ i ] = new Feature( global.getFeatureNames(Global.FT_INPUT, i ), (short)Global.FT_INPUT, (short)i,
                      dataSet[sampleIdx][i], global );
 }

 /** check the completion is valid. */
 private void checkCompletion()
 {
   char[] letters = { '$', 'C', 'P', 'S', 'T', 'V', 'X', '-' };
   char[][][] stateTrans = {
      {{'S','1'},{'C','2'}},  {{'C','3'}},            {{'V','1'},{'X','3'}},
      {{'T','4'},{'P','5'}},  {{'S','4'},{'X','6'}},  {{'T','5'},{'V','7'}},
      {{'S','8'},{'X','5'}},  {{'P','6'},{'V','8'}}
   };
   int length = curNacsInput.length;
   int validNum = 0;
   int stateIdx = 0;

   if( nacsResults == null )
       return;

   if( length > nacsResults.length  )
       System.out.println( "AgentAGL.java, nacs input len = " + curNacsInput.length +
                           ", nacs results len  = " + nacsResults.length );

   for( int i = 0; i < length; i++ ){
        if( i >= nacsResults.length || nacsResults[i] == null || nacsResults[i].getValueNum() == 0 || stateIdx == 8 )
            break;
        if( curNacsInput[i].getValueNum() > 0 && !curNacsInput[i].equals(nacsResults[i]) )
            break;
        char ch = letters[ nacsResults[i].getValues()[0] ];
        for( int j = 0; j < stateTrans[stateIdx].length; j++ ){
             if( ch == stateTrans[stateIdx][j][0] ){
                 validNum++;
                 stateIdx = Integer.parseInt(String.valueOf(stateTrans[stateIdx][j][1]));
                 if( stateIdx == 8 )
                     break;
             }
        }
   }
   //System.out.println( "rate = " + (double)validNum / length );
   if( validNum == length ){
       if( stateIdx == 8 )
           quality[ sessionIdx ]++;
   }
   if( (double)validNum / length >= 0.7 )
       achievement[ sessionIdx ]++;
 }

 /** the the actual learning process during a given session. */
 private void decideLearningType()
 {
   if( sessionIdx == 0 || groupType[0] == groupType[1] )
       return;
   if( Math.random() < LOCK_PROB )
       getLearningType( groupType[0] );
   else
       getLearningType( groupType[1] );
 }
}
